@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -40,11 +40,11 @@ public:
     virtual ~EaseInOutAnim();
 
 protected:
-    virtual double interpolate(double t);
+    virtual float interpolate(float t);
     
 private:
-    double m_EaseInDuration;
-    double m_EaseOutDuration;
+    float m_EaseInDuration;
+    float m_EaseOutDuration;
 };
 
 }
