@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -40,5 +40,11 @@
 #define AVG_PLUGIN_API extern "C"
 #endif
 
+#ifdef __APPLE__
+// Workaround for snow leopard incompatibility between c++ headers and python headers:
+// <iostream> always needs to be included before Python.h.
+#include <iostream>
+#endif
+
 #endif
 
