@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -33,15 +33,22 @@ template<class RECEIVER>
 class AVG_TEMPLATE_API CmdQueue: public Queue<Command<RECEIVER> >
 {
 public:
+    CmdQueue(int maxSize=-1);
     typedef typename Queue<Command<RECEIVER> >::QElementPtr CmdPtr;
     void pushCmd(typename Command<RECEIVER>::CmdFunc func);
     
 };
 
+template<class RECEIVER>
+CmdQueue<RECEIVER>::CmdQueue(int maxSize)
+    : Queue<Command<RECEIVER> >(maxSize)
+{
+}
+
 template<class RECEIVER>
 void CmdQueue<RECEIVER>::pushCmd(typename Command<RECEIVER>::CmdFunc func)
 {
-    push(CmdPtr(new Command<RECEIVER>(func)));
+    this->push(CmdPtr(new Command<RECEIVER>(func)));
 }
 
 }
