@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,8 @@
 #define _CubicSpline_H_
 
 #include "../api.h"
+#include "../glm/glm.hpp"
+
 #include <boost/shared_ptr.hpp>
 #include <vector>
 
@@ -30,14 +32,18 @@ namespace avg {
     
 class AVG_API CubicSpline {
 public:
-    CubicSpline(const std::vector<double>& x, const std::vector<double>& y);
+    CubicSpline(const std::vector<float>& x, const std::vector<float>& y, 
+            bool bLoop=false);
+    CubicSpline(const std::vector<glm::vec2>& pts, bool bLoop=false);
     virtual ~CubicSpline();
 
-    double interpolate(double orig);
+    float interpolate(float orig);
 
 private:
-    std::vector<double> m_X;
-    std::vector<double> m_Y;
+    void init();
+
+    std::vector<glm::vec2> m_Pts;
+    std::vector<float> m_Y2; // Second derivatives
 };
 
 typedef boost::shared_ptr<CubicSpline> CubicSplinePtr;
