@@ -1,6 +1,6 @@
 //
-//  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  libavg - Media Playback Engine.
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,6 @@
 //
 
 #include "FileHelper.h"
-#include "ConfigMgr.h"
 #include "Exception.h"
 
 #ifndef _WIN32
@@ -31,7 +30,10 @@
 #include <stdio.h>
 #include <sys/stat.h>
 #include <string.h>
+#include <unistd.h>
+#include <stdlib.h>
 
+#include <vector>
 #include <map>
 #include <cstring>
 #include <iostream>
@@ -112,7 +114,7 @@ bool isAbsPath(const std::string& path)
 #else
     return path[0] == '/';
 #endif
-    
+
 }
 
 bool fileExists(const string& sFilename)
@@ -154,7 +156,7 @@ void writeWholeFile(const string& sFilename, const string& sContent)
 void copyFile(const string& sSourceFile, const string& sDestFile)
 {
     string sData;
-            
+
     readWholeFile(sSourceFile, sData);
     writeWholeFile(sDestFile, sData);
 }
