@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -35,6 +35,7 @@ class AVG_API ProfilingZone
 public:
     ProfilingZone(const ProfilingZoneID& zoneID);
     virtual ~ProfilingZone();
+    void restart();
     
     void start() 
     {
@@ -53,13 +54,12 @@ public:
     const std::string& getName() const;
 
 private:
-    const ProfilingZoneID& m_ZoneID;
     long long m_TimeSum;
     long long m_AvgTime;
     long long m_StartTime;
     int m_NumFrames;
     int m_Indent;
-    bool m_bIsRegistered;
+    const ProfilingZoneID& m_ZoneID;
 };
 
 typedef boost::shared_ptr<ProfilingZone> ProfilingZonePtr;
