@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,8 +20,11 @@
 //
 
 #include "ThreadProfiler.h"
+
 #include "Logger.h"
 #include "Exception.h"
+#include "ProfilingZone.h"
+#include "ScopeTimer.h"
 
 #include <sstream>
 #include <iomanip>
@@ -32,33 +35,37 @@ using namespace boost;
 
 namespace avg {
     
-thread_specific_ptr<ThreadProfilerPtr> ThreadProfiler::s_pInstance;
+thread_specific_ptr<ThreadProfiler*> ThreadProfiler::s_pInstance;
 
-ThreadProfilerPtr& ThreadProfiler::get() 
+ThreadProfiler* ThreadProfiler::get() 
 {
     if (s_pInstance.get() == 0) {
-        s_pInstance.reset(new ThreadProfilerPtr(new ThreadProfiler()));
+        s_pInstance.reset(new (ThreadProfiler*));
+        *s_pInstance = new ThreadProfiler();
     }
     return *s_pInstance;
 }
 
 void ThreadProfiler::kill()
 {
+    delete *s_pInstance;
     s_pInstance.reset();
 }
 
 ThreadProfiler::ThreadProfiler()
     : m_sName(""),
-      m_LogCategory(Logger::PROFILE)
+      m_LogCategory(Logger::category::PROFILE)
 {
     m_bRunning = false;
+    ScopeTimer::enableTimers(Logger::get()->shouldLog(m_LogCategory,
+            Logger::severity::INFO));
 }
 
 ThreadProfiler::~ThreadProfiler() 
 {
 }
 
-void ThreadProfiler::setLogCategory(long category)
+void ThreadProfiler::setLogCategory(category_t category)
 {
     AVG_ASSERT(!m_bRunning);
     m_LogCategory = category;
@@ -69,9 +76,12 @@ void ThreadProfiler::start()
     m_bRunning = true;
 }
 
-bool ThreadProfiler::isRunning()
+void ThreadProfiler::restart()
 {
-    return m_bRunning;
+    ZoneVector::iterator it;
+    for (it = m_Zones.begin(); it != m_Zones.end(); ++it) {
+        (*it)->restart();
+    }
 }
 
 void ThreadProfiler::startZone(const ProfilingZoneID& zoneID)
@@ -93,47 +103,34 @@ void ThreadProfiler::startZone(const ProfilingZoneID& zoneID)
 void ThreadProfiler::stopZone(const ProfilingZoneID& zoneID)
 {
     ZoneMap::iterator it = m_ZoneMap.find(&zoneID);
-    AVG_ASSERT(it != m_ZoneMap.end());
     ProfilingZonePtr& pZone = it->second;
-    AVG_ASSERT(m_ActiveZones.back() == pZone);
     pZone->stop();
     m_ActiveZones.pop_back();
 }
 
-void ThreadProfiler::dumpFrame()
-{
-    AVG_TRACE(Logger::PROFILE_LATEFRAMES, "Frame Profile:");
-    ZoneList::iterator it;
-    for (it = m_Zones.begin(); it != m_Zones.end(); ++it) {
-        AVG_TRACE(Logger::PROFILE_LATEFRAMES,
-                std::setw(35) << std::left 
-                << ((*it)->getIndentString() + (*it)->getName()) 
-                << std::setw(9) << std::right << (*it)->getUSecs());
-    }
-    AVG_TRACE(Logger::PROFILE_LATEFRAMES, "");
-}
-
 void ThreadProfiler::dumpStatistics()
 {
     if (!m_Zones.empty()) {
-        AVG_TRACE(m_LogCategory, "Thread " << m_sName);
-        AVG_TRACE(m_LogCategory, "Zone name                          Avg. time");
-        AVG_TRACE(m_LogCategory, "---------                          ---------");
+        AVG_TRACE(m_LogCategory, Logger::severity::INFO, "Thread " << m_sName);
+        AVG_TRACE(m_LogCategory, Logger::severity::INFO,
+                "Zone name                          Avg. time");
+        AVG_TRACE(m_LogCategory, Logger::severity::INFO,
+                "---------                          ---------");
 
-        ZoneList::iterator it;
+        ZoneVector::iterator it;
         for (it = m_Zones.begin(); it != m_Zones.end(); ++it) {
-            AVG_TRACE(m_LogCategory,
+            AVG_TRACE(m_LogCategory, Logger::severity::INFO,
                     std::setw(35) << std::left 
                     << ((*it)->getIndentString()+(*it)->getName())
                     << std::setw(9) << std::right << (*it)->getAvgUSecs());
         }
-        AVG_TRACE(m_LogCategory, "");
+        AVG_TRACE(m_LogCategory, Logger::severity::INFO, "");
     }
 }
 
 void ThreadProfiler::reset()
 {
-    ZoneList::iterator it;
+    ZoneVector::iterator it;
     for (it = m_Zones.begin(); it != m_Zones.end(); ++it) {
         (*it)->reset();
     }
@@ -159,7 +156,7 @@ ProfilingZonePtr ThreadProfiler::addZone(const ProfilingZoneID& zoneID)
 {
     ProfilingZonePtr pZone(new ProfilingZone(zoneID));
     m_ZoneMap[&zoneID] = pZone;
-    ZoneList::iterator it;
+    ZoneVector::iterator it;
     int parentIndent = -2;
     if (m_ActiveZones.empty()) {
         it = m_Zones.end();
@@ -176,7 +173,7 @@ ProfilingZonePtr ThreadProfiler::addZone(const ProfilingZoneID& zoneID)
         AVG_ASSERT(bParentFound);
         parentIndent = pActiveZone->getIndentLevel();
         ++it;
-        for (; it != m_Zones.end() && (*it)->getIndentLevel() > parentIndent; ++it);
+        for (; it != m_Zones.end() && (*it)->getIndentLevel() > parentIndent; ++it) {};
     }
     m_Zones.insert(it, pZone);
     pZone->setIndentLevel(parentIndent+2);
