@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -54,4 +54,9 @@ UTF8String& UTF8String::operator =(const char* psz)
     return *this;
 }
 
+std::size_t hash_value(const avg::UTF8String& x)
+{
+    boost::hash<std::string> hasher;
+    return hasher(x);
+};
 }
