@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@
 #define _UTF8String_H_
 
 #include "../api.h"
+#include <boost/functional/hash.hpp>
 #include <string>
 
 namespace avg {
@@ -39,5 +40,7 @@ public:
 
 };
 
+std::size_t hash_value(const avg::UTF8String& x);
+
 }
 #endif
