@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -47,9 +47,7 @@ namespace avg {
 template<class Pixel>
 void createTrueColorCopy(Bitmap& destBmp, const Bitmap & srcBmp);
 
-bool Bitmap::s_bGTKInitialized = false;
-
-Bitmap::Bitmap(DPoint size, PixelFormat pf, const UTF8String& sName, int stride)
+Bitmap::Bitmap(glm::vec2 size, PixelFormat pf, const UTF8String& sName, int stride)
     : m_Size(size),
       m_PF(pf),
       m_pBits(0),
@@ -118,73 +116,17 @@ Bitmap::Bitmap(Bitmap& origBmp, const IntRect& rect)
     AVG_ASSERT(rect.br.x <= origBmp.getSize().x);
     AVG_ASSERT(rect.br.y <= origBmp.getSize().y);
     AVG_ASSERT(rect.tl.x >= 0 && rect.tl.y >= 0);
+    AVG_ASSERT(rect.width() > 0 && rect.height() > 0);
     if (!origBmp.getName().empty()) {
         m_sName = origBmp.getName()+" part";
     } else {
         m_sName = "";
     }
-    unsigned char * pRegionStart = origBmp.getPixels()+rect.tl.y*origBmp.getStride()+
-            rect.tl.x*getBytesPerPixel();
+    unsigned char * pRegionStart = origBmp.getPixels()
+            + size_t(rect.tl.y)*origBmp.getStride() + rect.tl.x*getBytesPerPixel();
     initWithData(pRegionStart, origBmp.getStride(), false);
 }
 
-Bitmap::Bitmap(const UTF8String& sName)
-    : m_pBits(0),
-      m_sName(sName)
-{
-    if (!s_bGTKInitialized) {
-        g_type_init();
-        s_bGTKInitialized = true;
-    }
-
-    GError* pError = 0;
-    GdkPixbuf* pPixBuf = gdk_pixbuf_new_from_file(sName.c_str(), &pError);
-    if (!pPixBuf) {
-        string sErr = pError->message;
-        g_error_free(pError);
-        throw Exception(AVG_ERR_FILEIO, sErr);
-    }
-    m_Size = IntPoint(gdk_pixbuf_get_width(pPixBuf), gdk_pixbuf_get_height(pPixBuf));
-    if (gdk_pixbuf_get_has_alpha(pPixBuf)) {
-        m_PF = B8G8R8A8;
-    } else {
-        m_PF = B8G8R8X8;
-    }
-    int stride = gdk_pixbuf_get_rowstride(pPixBuf);
-    allocBits();
-    guchar* pSrc = gdk_pixbuf_get_pixels(pPixBuf);
-    for (int y = 0; y < m_Size.y; ++y) {
-        unsigned char* pDestLine = m_pBits+m_Stride*y;
-        guchar* pSrcLine = pSrc + y*stride;
-        if (m_PF == B8G8R8A8) {
-            unsigned char* pDestPixel = pDestLine;
-            guchar* pSrcPixel = pSrcLine;
-            for (int x = 0; x < m_Size.x; ++x) {
-                pDestPixel[0] = pSrcPixel[2];
-                pDestPixel[1] = pSrcPixel[1];
-                pDestPixel[2] = pSrcPixel[0];
-                pDestPixel[3] = pSrcPixel[3];
-                pDestPixel += 4;
-                pSrcPixel += 4;
-            }
-        } else {
-            unsigned char* pDestPixel = pDestLine;
-            guchar* pSrcPixel = pSrcLine;
-            for (int x = 0; x < m_Size.x; ++x) {
-                pDestPixel[0] = pSrcPixel[2];
-                pDestPixel[1] = pSrcPixel[1];
-                pDestPixel[2] = pSrcPixel[0];
-                pDestPixel[3] = 255;
-                pDestPixel += 4;
-                pSrcPixel += 3;
-           } 
-        }
-    }
-    m_bOwnsBits = true;
-    g_object_unref(pPixBuf);
-    ObjectCounter::get()->incRef(&typeid(*this));
-}
-
 Bitmap::~Bitmap()
 {
     ObjectCounter::get()->decRef(&typeid(*this));
@@ -222,10 +164,10 @@ void Bitmap::copyPixels(const Bitmap& origBmp)
         const unsigned char * pSrc = origBmp.getPixels();
         unsigned char * pDest = m_pBits;
         int height = min(origBmp.getSize().y, m_Size.y);
-        int LineLen = min(origBmp.getLineLen(), getLineLen());
+        int lineLen = min(origBmp.getLineLen(), getLineLen());
         int srcStride = origBmp.getStride();
         for (int y = 0; y < height; ++y) {
-            memcpy(pDest, pSrc, LineLen);
+            memcpy(pDest, pSrc, lineLen);
             pDest += m_Stride;
             pSrc += srcStride;
         }
@@ -239,6 +181,7 @@ void Bitmap::copyPixels(const Bitmap& origBmp)
                         YCbCrtoBGR(origBmp);
                         break;
                     case I8:
+                    case A8:
                         YCbCrtoI8(origBmp);
                     default: {
                             Bitmap TempBmp(getSize(), B8G8R8X8, "TempColorConversion");
@@ -249,7 +192,7 @@ void Bitmap::copyPixels(const Bitmap& origBmp)
                 }
                 break;
             case I16:
-                if (m_PF == I8) {
+                if (m_PF == I8 || m_PF == A8) {
                     I16toI8(origBmp);
                 } else {
                     Bitmap TempBmp(getSize(), I8, "TempColorConversion");
@@ -258,6 +201,7 @@ void Bitmap::copyPixels(const Bitmap& origBmp)
                 }
                 break;
             case I8:
+            case A8:
                 switch(m_PF) {
                     case I16:
                         I8toI16(origBmp);
@@ -281,6 +225,7 @@ void Bitmap::copyPixels(const Bitmap& origBmp)
             case BAYER8_BGGR:
                 switch(m_PF) {
                     case I8:
+                    case A8:
                         {
                             // Bayer patterns are saved as I8 bitmaps. 
                             // So simply copy that.
@@ -346,6 +291,7 @@ void Bitmap::copyPixels(const Bitmap& origBmp)
                         createTrueColorCopy<Pixel16>(*this, origBmp);
                         break;
                     case I8:
+                    case A8:
                         createTrueColorCopy<Pixel8>(*this, origBmp);
                         break;
                     default:
@@ -570,19 +516,15 @@ void Bitmap::copyYUVPixels(const Bitmap& yBmp, const Bitmap& uBmp, const Bitmap&
 
 void Bitmap::save(const UTF8String& sFilename)
 {
-    if (!s_bGTKInitialized) {
-        g_type_init();
-        s_bGTKInitialized = true;
-    }
     Bitmap* pTempBmp;
     switch (m_PF) {
         case B8G8R8X8:
             pTempBmp = new Bitmap(m_Size, R8G8B8);
-            for (int y = 0; y < m_Size.y; y++) {
-                unsigned char * pSrcLine = m_pBits+y * m_Stride;
+            for (size_t y = 0; y < size_t(m_Size.y); y++) {
+                unsigned char * pSrcLine = m_pBits + y*m_Stride;
                 unsigned char * pDestLine = pTempBmp->getPixels() + 
                         y*pTempBmp->getStride();
-                for (int x = 0; x < m_Size.x; x++) { 
+                for (size_t x = 0; x < size_t(m_Size.x); x++) { 
                     pDestLine[x*3] = pSrcLine[x*4 + 2];
                     pDestLine[x*3 + 1] = pSrcLine[x*4 + 1];
                     pDestLine[x*3 + 2] = pSrcLine[x*4];
@@ -591,11 +533,11 @@ void Bitmap::save(const UTF8String& sFilename)
             break;
         case B8G8R8A8:
             pTempBmp = new Bitmap(m_Size, R8G8B8A8);
-            for (int y = 0; y < m_Size.y; y++) {
+            for (size_t y = 0; y < size_t(m_Size.y); y++) {
                 unsigned char * pSrcLine = m_pBits+y * m_Stride;
                 unsigned char * pDestLine = pTempBmp->getPixels() + 
                         y*pTempBmp->getStride();
-                for (int x = 0; x < m_Size.x; x++) { 
+                for (size_t x = 0; x < size_t(m_Size.x); x++) { 
                     pDestLine[x*4] = pSrcLine[x*4 + 2];
                     pDestLine[x*4 + 1] = pSrcLine[x*4 + 1];
                     pDestLine[x*4 + 2] = pSrcLine[x*4];
@@ -603,6 +545,19 @@ void Bitmap::save(const UTF8String& sFilename)
                 }
             }
             break;
+        case B8G8R8:
+            pTempBmp = new Bitmap(m_Size, R8G8B8);
+            for (size_t y = 0; y < size_t(m_Size.y); y++) {
+                unsigned char * pSrcLine = m_pBits+y * m_Stride;
+                unsigned char * pDestLine = pTempBmp->getPixels() + 
+                        y*pTempBmp->getStride();
+                for (size_t x = 0; x < size_t(m_Size.x); x++) { 
+                    pDestLine[x*3] = pSrcLine[x*3 + 2];
+                    pDestLine[x*3 + 1] = pSrcLine[x*3 + 1];
+                    pDestLine[x*3 + 2] = pSrcLine[x*3];
+                }
+            }
+            break;
         default:
             if (hasAlpha()) {
                 pTempBmp = new Bitmap(m_Size, R8G8B8A8);
@@ -616,9 +571,14 @@ void Bitmap::save(const UTF8String& sFilename)
             pTempBmp->getStride(), 0, 0);
 
     string sExt = getExtension(sFilename);
+    if (sExt == "jpg") {
+        sExt = "jpeg";
+    }
 
     GError* pError = 0;
-    gboolean bOk = gdk_pixbuf_save(pPixBuf, sFilename.c_str(), sExt.c_str(), &pError, NULL);
+    gboolean bOk = gdk_pixbuf_save(pPixBuf, sFilename.c_str(), sExt.c_str(), &pError, 
+            NULL);
+    g_object_unref(pPixBuf);
     if (!bOk) {
         string sErr = pError->message;
         g_error_free(pError);
@@ -711,7 +671,7 @@ bool Bitmap::hasAlpha() const
 
 HistogramPtr Bitmap::getHistogram(int stride) const
 {
-    AVG_ASSERT (m_PF == I8);
+    AVG_ASSERT (getBytesPerPixel() == 1);
     HistogramPtr pHist(new Histogram(256,0));
     const unsigned char * pSrcLine = m_pBits;
     for (int y = 0; y < m_Size.y; y += stride) {
@@ -727,7 +687,7 @@ HistogramPtr Bitmap::getHistogram(int stride) const
 
 void Bitmap::getMinMax(int stride, int& min, int& max) const
 {
-    AVG_ASSERT (m_PF == I8);
+    AVG_ASSERT (getBytesPerPixel() == 1);
     const unsigned char * pSrcLine = m_pBits;
     min = 255;
     max = 0;
@@ -749,7 +709,7 @@ void Bitmap::getMinMax(int stride, int& min, int& max) const
 void Bitmap::setAlpha(const Bitmap& alphaBmp)
 {
     AVG_ASSERT(hasAlpha());
-    AVG_ASSERT(alphaBmp.getPixelFormat() == I8);
+    AVG_ASSERT(alphaBmp.getBytesPerPixel() == 1);
     unsigned char * pLine = m_pBits;
     const unsigned char * pAlphaLine = alphaBmp.getPixels();
     for (int y = 0; y < m_Size.y; y++) {
@@ -765,7 +725,7 @@ void Bitmap::setAlpha(const Bitmap& alphaBmp)
     }
 }
 
-Pixel32 Bitmap::getPythonPixel(const DPoint& pos)
+Pixel32 Bitmap::getPythonPixel(const glm::vec2& pos)
 {
     IntPoint intPos(pos);
     if (intPos.x < 0 || intPos.y < 0 || intPos.x >= m_Size.x || intPos.y >= m_Size.y) {
@@ -776,6 +736,12 @@ Pixel32 Bitmap::getPythonPixel(const DPoint& pos)
     }
     const unsigned char * pPixel = m_pBits+intPos.y*m_Stride+intPos.x*getBytesPerPixel();
     switch(getPixelFormat()) {
+        case R8G8B8A8:
+            return Pixel32(pPixel[0], pPixel[1], pPixel[2], pPixel[3]);
+        case R8G8B8X8:
+            return Pixel32(pPixel[0], pPixel[1], pPixel[2], 255);
+        case R8G8B8:
+            return Pixel32(pPixel[0], pPixel[1], pPixel[2]);
         case B8G8R8A8:
             return Pixel32(pPixel[2], pPixel[1], pPixel[0], pPixel[3]);
         case B8G8R8X8:
@@ -783,6 +749,7 @@ Pixel32 Bitmap::getPythonPixel(const DPoint& pos)
         case B8G8R8:
             return Pixel32(pPixel[2], pPixel[1], pPixel[0]);
         case I8:
+        case A8:
             return Pixel32(pPixel[0], pPixel[0], pPixel[0]);
         default:
             cerr << getPixelFormat() << endl;
@@ -795,14 +762,13 @@ bool Bitmap::operator ==(const Bitmap& otherBmp)
 {
     // We allow Name, Stride and bOwnsBits to be different here, since we're looking for
     // equal value only.
-    if (m_Size != otherBmp.m_Size || m_PF != otherBmp.m_PF)
-    {
+    if (m_Size != otherBmp.m_Size || m_PF != otherBmp.m_PF) {
         return false;
     }
 
     const unsigned char * pSrc = otherBmp.getPixels();
     unsigned char * pDest = m_pBits;
-    int LineLen = getLineLen();
+    int lineLen = getLineLen();
     for (int y = 0; y < getSize().y; ++y) {
         switch(m_PF) {
             case R8G8B8X8:
@@ -816,7 +782,7 @@ bool Bitmap::operator ==(const Bitmap& otherBmp)
                 }
                 break;
             default:
-                if (memcmp(pDest, pSrc, LineLen) != 0) {
+                if (memcmp(pDest, pSrc, lineLen) != 0) {
                     return false;
                 }
         }
@@ -828,15 +794,17 @@ bool Bitmap::operator ==(const Bitmap& otherBmp)
 
 BitmapPtr Bitmap::subtract(const Bitmap& otherBmp)
 {
-    if (m_PF != otherBmp.getPixelFormat())
+    if (m_PF != otherBmp.getPixelFormat()) {
         throw Exception(AVG_ERR_UNSUPPORTED, 
                 string("Bitmap::subtract: pixel formats differ(")
                 + getPixelFormatString(m_PF)+", "
                 + getPixelFormatString(otherBmp.getPixelFormat())+")");
-    if (m_Size != otherBmp.getSize())
+    }
+    if (m_Size != otherBmp.getSize()) {
         throw Exception(AVG_ERR_UNSUPPORTED, 
                 string("Bitmap::subtract: bitmap sizes differ (this=")
                 + toString(m_Size) + ", other=" + toString(otherBmp.getSize()) + ")");
+    }
     BitmapPtr pResultBmp = BitmapPtr(new Bitmap(m_Size, m_PF));
     const unsigned char * pSrcLine1 = otherBmp.getPixels();
     const unsigned char * pSrcLine2 = m_pBits;
@@ -881,41 +849,64 @@ BitmapPtr Bitmap::subtract(const Bitmap& otherBmp)
     
 void Bitmap::blt(const Bitmap& otherBmp, const IntPoint& pos)
 {
-    AVG_ASSERT(getBytesPerPixel() == 4);
-
+    AVG_ASSERT(getBytesPerPixel() == 4 || getBytesPerPixel() == 3);
+    AVG_ASSERT(otherBmp.getBytesPerPixel() == 4 || otherBmp.getBytesPerPixel() == 3);
+    if (pos.x < 0 || pos.y < 0) {
+        throw Exception(AVG_ERR_UNSUPPORTED, 
+                string("Bitmap::blt: pos < 0 is not supported."));
+    }
+    
     IntRect destRect(pos.x, pos.y, pos.x+otherBmp.getSize().x, 
             pos.y+otherBmp.getSize().y);
     destRect.intersect(IntRect(IntPoint(0,0), getSize()));
     for (int y = 0; y < destRect.height(); y++) {
-        unsigned char * pSrcPixel = getPixels()+(pos.y+y)*getStride()+pos.x*4;
+        unsigned char * pSrcPixel = getPixels()+size_t(pos.y+y)*getStride()
+                +size_t(pos.x*getBytesPerPixel());
         const unsigned char * pOtherPixel = otherBmp.getPixels()+
-                y*otherBmp.getStride(); 
-        if (otherBmp.hasAlpha()) {
-            for (int x = 0; x < destRect.width(); x++) {
-                int srcAlpha = 255-pOtherPixel[3];
-                pSrcPixel[0] = 
-                        (srcAlpha*pSrcPixel[0]+int(pOtherPixel[3])*pOtherPixel[0])/255;
-                pSrcPixel[1] = 
-                        (srcAlpha*pSrcPixel[1]+int(pOtherPixel[3])*pOtherPixel[1])/255;
-                pSrcPixel[2] = 
-                        (srcAlpha*pSrcPixel[2]+int(pOtherPixel[3])*pOtherPixel[2])/255;
-                pSrcPixel += 4;
-                pOtherPixel += 4;
+                size_t(y*otherBmp.getStride());
+        if (getBytesPerPixel() == 4) {
+            if (otherBmp.hasAlpha()) {
+                for (int x = 0; x < destRect.width(); x++) {
+                    int srcAlpha = 255-pOtherPixel[3];
+                    pSrcPixel[0] = 
+                            (srcAlpha*pSrcPixel[0]+int(pOtherPixel[3])*pOtherPixel[0])/255;
+                    pSrcPixel[1] = 
+                            (srcAlpha*pSrcPixel[1]+int(pOtherPixel[3])*pOtherPixel[1])/255;
+                    pSrcPixel[2] = 
+                            (srcAlpha*pSrcPixel[2]+int(pOtherPixel[3])*pOtherPixel[2])/255;
+                    pSrcPixel += 4;
+                    pOtherPixel += 4;
+                }
+            } else {
+                for (int x = 0; x < destRect.width(); x++) {
+                    *(Pixel32*)pSrcPixel = *(Pixel32*)pOtherPixel;
+                    pSrcPixel[3] = 255;
+                    pSrcPixel += 4;
+                    pOtherPixel += 4;
+                }
             }
         } else {
-            for (int x = 0; x < destRect.width(); x++) {
-                *(Pixel32*)pSrcPixel = *(Pixel32*)pOtherPixel;
-                pSrcPixel[3] = 255;
-                pSrcPixel += 4;
-                pOtherPixel += 4;
+            if (otherBmp.getBytesPerPixel() == 4) {
+                // Incomplete: Missing alpha support.
+                for (int x = 0; x < destRect.width(); x++) {
+                    *(Pixel24*)pSrcPixel = *(Pixel24*)pOtherPixel;
+                    pSrcPixel += 3;
+                    pOtherPixel += 4;
+                }
+            } else {
+                for (int x = 0; x < destRect.width(); x++) {
+                    *(Pixel24*)pSrcPixel = *(Pixel24*)pOtherPixel;
+                    pSrcPixel += 3;
+                    pOtherPixel += 3;
+                }
             }
         }
     }
 }
 
-double Bitmap::getAvg() const
+float Bitmap::getAvg() const
 {
-    double sum = 0;
+    float sum = 0;
     unsigned char * pSrc = m_pBits;
     int componentsPerPixel = getBytesPerPixel();
     for (int y = 0; y < getSize().y; ++y) {
@@ -971,12 +962,12 @@ double Bitmap::getAvg() const
     return sum/(getSize().x*getSize().y);
 }
 
-double Bitmap::getChannelAvg(int channel) const
+float Bitmap::getChannelAvg(int channel) const
 {
     AVG_ASSERT(!pixelFormatIsPlanar(m_PF) && !pixelFormatIsBayer(m_PF) && !(m_PF == I16));
     int bytesPerPixel = getBytesPerPixel();
     AVG_ASSERT(channel < bytesPerPixel);
-    double sum = 0;
+    float sum = 0;
     unsigned char * pSrcLine = m_pBits;
     for (int y = 0; y < getSize().y; ++y) {
         unsigned char * pSrcPixel = pSrcLine;
@@ -989,10 +980,10 @@ double Bitmap::getChannelAvg(int channel) const
     return sum/(getSize().x*getSize().y);
 }
 
-double Bitmap::getStdDev() const
+float Bitmap::getStdDev() const
 {
-    double average = getAvg();
-    double sum = 0;
+    float average = getAvg();
+    float sum = 0;
 
     unsigned char * pSrc = m_pBits;
     int componentsPerPixel = getBytesPerPixel();
@@ -1094,23 +1085,25 @@ void Bitmap::dump(bool bDumpPixels) const
     cerr << dec;
 }
 
+int Bitmap::getPreferredStride(int width, PixelFormat pf)
+{
+    return (((width*avg::getBytesPerPixel(pf))-1)/4+1)*4;
+}
+
 void Bitmap::initWithData(unsigned char * pBits, int stride, bool bCopyBits)
 {
 //    cerr << "Bitmap::initWithData()" << endl;
-    if (m_PF == A8) {
-        m_PF = I8;
-    }
     if (m_PF == YCbCr422) {
         if (m_Size.x%2 == 1) {
-            AVG_TRACE(Logger::WARNING, "Odd size for YCbCr bitmap.");
+            AVG_LOG_WARNING("Odd size for YCbCr bitmap.");
             m_Size.x++;
         }
         if (m_Size.y%2 == 1) {
-            AVG_TRACE(Logger::WARNING, "Odd size for YCbCr bitmap.");
+            AVG_LOG_WARNING("Odd size for YCbCr bitmap.");
             m_Size.y++;
         }
         if (m_Size.x%2 == 1 || m_Size.y%2 == 1) {
-            AVG_TRACE(Logger::ERROR, "Odd size for YCbCr bitmap.");
+            AVG_LOG_ERROR("Odd size for YCbCr bitmap.");
         }
     }
     if (bCopyBits) {
@@ -1137,28 +1130,25 @@ void Bitmap::allocBits(int stride)
     AVG_ASSERT(m_Size.x > 0 && m_Size.y > 0);
 //    cerr << "Bitmap::allocBits():" << m_Size <<  endl;
     if (stride == 0) {
-        m_Stride = getLineLen();
+        m_Stride = getPreferredStride(m_Size.x, m_PF);
     } else {
         m_Stride = stride;
     }
-    if (m_PF == A8) {
-        m_PF = I8;
-    }
     if (m_PF == YCbCr422) {
         if (m_Size.x%2 == 1) {
-            AVG_TRACE(Logger::WARNING, "Odd width for YCbCr bitmap.");
+            AVG_LOG_WARNING("Odd width for YCbCr bitmap.");
             m_Size.x++;
         }
         if (m_Size.y%2 == 1) {
-            AVG_TRACE(Logger::WARNING, "Odd height for YCbCr bitmap.");
+            AVG_LOG_WARNING("Odd height for YCbCr bitmap.");
             m_Size.y++;
         }
         //XXX: We allocate more than nessesary here because ffmpeg seems to
         // overwrite memory after the bits - probably during yuv conversion.
         // Yuck.
-        m_pBits = new unsigned char[(m_Stride+1)*(m_Size.y+1)];
+        m_pBits = new unsigned char[size_t(m_Stride+1)*(m_Size.y+1)];
     } else {
-        m_pBits = new unsigned char[m_Stride*m_Size.y];
+        m_pBits = new unsigned char[size_t(m_Stride)*m_Size.y];
     }
 }
 
@@ -1328,7 +1318,7 @@ void YUV411toI8Line(const unsigned char* pSrcLine, unsigned char* pDestLine, int
  
 void Bitmap::YCbCrtoI8(const Bitmap& origBmp)
 {
-    AVG_ASSERT(m_PF==I8);
+    AVG_ASSERT(origBmp.getBytesPerPixel() == 1);
     const unsigned char * pSrc = origBmp.getPixels();
     unsigned char * pDest = m_pBits;
     int height = min(origBmp.getSize().y, m_Size.y);
@@ -1365,7 +1355,7 @@ void Bitmap::YCbCrtoI8(const Bitmap& origBmp)
 
 void Bitmap::I16toI8(const Bitmap& origBmp)
 {
-    AVG_ASSERT(m_PF == I8);
+    AVG_ASSERT(getBytesPerPixel() == 1);
     AVG_ASSERT(origBmp.getPixelFormat() == I16);
     const unsigned short * pSrc = (const unsigned short *)origBmp.getPixels();
     unsigned char * pDest = m_pBits;
@@ -1386,7 +1376,7 @@ void Bitmap::I16toI8(const Bitmap& origBmp)
 void Bitmap::I8toI16(const Bitmap& origBmp)
 {
     AVG_ASSERT(m_PF == I16);
-    AVG_ASSERT(origBmp.getPixelFormat() == I8);
+    AVG_ASSERT(origBmp.getBytesPerPixel() == 1);
     const unsigned char * pSrc = origBmp.getPixels();
     unsigned short * pDest = (unsigned short *)m_pBits;
     int height = min(origBmp.getSize().y, m_Size.y);
@@ -1406,7 +1396,7 @@ void Bitmap::I8toI16(const Bitmap& origBmp)
 void Bitmap::I8toRGB(const Bitmap& origBmp)
 {
     AVG_ASSERT(getBytesPerPixel() == 4 || getBytesPerPixel() == 3);
-    AVG_ASSERT(origBmp.getPixelFormat() == I8);
+    AVG_ASSERT(origBmp.getBytesPerPixel() == 1);
     const unsigned char * pSrc = origBmp.getPixels();
     int height = min(origBmp.getSize().y, m_Size.y);
     int width = min(origBmp.getSize().x, m_Size.x);
@@ -1808,6 +1798,7 @@ void createTrueColorCopy(Bitmap& destBmp, const Bitmap& srcBmp)
             createTrueColorCopy<PIXEL, Pixel16>(destBmp, srcBmp);
             break;
         case I8:
+        case A8:
         case BAYER8_RGGB:
         case BAYER8_GBRG:
         case BAYER8_GRBG:
