@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,8 +26,8 @@
 #include "Pixel32.h"
 #include "PixelFormat.h"
 
-#include "../base/Point.h"
 #include "../base/Rect.h"
+#include "../base/GLMHelper.h"
 #include "../base/UTF8String.h"
 
 #include <boost/shared_ptr.hpp>
@@ -53,14 +53,13 @@ typedef boost::shared_ptr<Bitmap> BitmapPtr;
 class AVG_API Bitmap
 {
 public:
-    Bitmap(DPoint size, PixelFormat pf, const UTF8String& sName="", int stride=0);
+    Bitmap(glm::vec2 size, PixelFormat pf, const UTF8String& sName="", int stride=0);
     Bitmap(IntPoint size, PixelFormat pf, const UTF8String& sName="", int stride=0);
     Bitmap(IntPoint size, PixelFormat pf, unsigned char * pBits, 
             int stride, bool bCopyBits, const UTF8String& sName="");
     Bitmap(const Bitmap& origBmp);
     Bitmap(const Bitmap& origBmp, bool bOwnsBits);
     Bitmap(Bitmap& origBmp, const IntRect& rect);
-    Bitmap(const UTF8String& sName);
     virtual ~Bitmap();
 
     Bitmap &operator =(const Bitmap & origBmp);
@@ -90,7 +89,7 @@ public:
     void getMinMax(int stride, int& min, int& max) const;
     void setAlpha(const Bitmap& alphaBmp);
 
-    Pixel32 getPythonPixel(const DPoint& pos);
+    Pixel32 getPythonPixel(const glm::vec2& pos);
     template<class PIXEL>
     void setPixel(const IntPoint& p, PIXEL color);
     template<class PIXEL>
@@ -98,13 +97,15 @@ public:
 
     BitmapPtr subtract(const Bitmap& pOtherBmp);
     void blt(const Bitmap& otherBmp, const IntPoint& pos);
-    double getAvg() const;
-    double getChannelAvg(int channel) const;
-    double getStdDev() const;
+    float getAvg() const;
+    float getChannelAvg(int channel) const;
+    float getStdDev() const;
 
     bool operator ==(const Bitmap& otherBmp);
     void dump(bool bDumpPixels=false) const;
 
+    static int getPreferredStride(int width, PixelFormat pf);
+
 private:
     void initWithData(unsigned char* pBits, int stride, bool bCopyBits);
     void allocBits(int stride=0);
