@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,7 @@ using namespace std;
 
 namespace avg {
     
-FilterBandpass::FilterBandpass(double lowWidth, double highWidth)
+FilterBandpass::FilterBandpass(float lowWidth, float highWidth)
     : m_HighpassFilter(highWidth),
       m_LowpassFilter(lowWidth)
 {
