@@ -1,4 +1,4 @@
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -28,12 +28,12 @@ namespace avg {
 class AVG_API FilterResizeGaussian : public Filter
 {
 public:
-    FilterResizeGaussian(const IntPoint& newSize, double radius);
+    FilterResizeGaussian(const IntPoint& newSize, float radius);
     virtual BitmapPtr apply(BitmapPtr pBmpSrc);
 
 private:
     IntPoint m_NewSize;
-    double m_Radius;
+    float m_Radius;
 };
 
 }
