@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -31,11 +31,12 @@ namespace avg {
 class AVG_API FilterFlipRGB : public Filter
 {
 public:
-  FilterFlipRGB();
-  virtual ~FilterFlipRGB();
-  virtual void applyInPlace(BitmapPtr pBmp) ;
+    FilterFlipRGB(bool bChangePF = true);
+    virtual ~FilterFlipRGB();
+    virtual void applyInPlace(BitmapPtr pBmp) ;
 
 private:
+    bool m_bChangePF;
 };
 
 }
