@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,21 +26,20 @@
 #include "Bitmap.h"
 #include "OGLHelper.h"
 
-#include "../base/Point.h"
-
 #include <boost/shared_ptr.hpp>
 
 namespace avg {
 
 class TextureMover;
 typedef boost::shared_ptr<TextureMover> TextureMoverPtr;
+class GLContext;
 
 class AVG_API GLTexture {
 
 public:
     GLTexture(const IntPoint& size, PixelFormat pf, bool bMipmap=false,
-            unsigned wrapSMode=GL_CLAMP_TO_EDGE, unsigned wrapTMode=GL_CLAMP_TO_EDGE,
-            bool bForcePOT=false);
+            int potBorderColor=0, unsigned wrapSMode=GL_CLAMP_TO_EDGE,
+            unsigned wrapTMode=GL_CLAMP_TO_EDGE, bool bForcePOT=false);
     GLTexture(unsigned glTexID, const IntPoint& size, PixelFormat pf, bool bMipmap=false,
             bool bDeleteTex=false);
     virtual ~GLTexture();
@@ -53,7 +52,7 @@ public:
     BitmapPtr lockStreamingBmp();
     void unlockStreamingBmp(bool bUpdated);
     void moveBmpToTexture(BitmapPtr pBmp);
-    BitmapPtr moveTextureToBmp();
+    BitmapPtr moveTextureToBmp(int mipmapLevel=0);
 
     const IntPoint& getSize() const;
     const IntPoint& getGLSize() const;
@@ -71,6 +70,8 @@ public:
     bool isDirty() const;
     void resetDirty();
 
+    void dump(unsigned wrapSMode=-1, unsigned wrapTMode=-1) const;
+
 private:
     IntPoint m_Size;
     IntPoint m_GLSize;
@@ -79,9 +80,12 @@ private:
     bool m_bDeleteTex;
     bool m_bUsePOT;
 
+    static unsigned s_LastTexID;
     unsigned m_TexID;
     bool m_bIsDirty;
     TextureMoverPtr m_pMover;
+
+    GLContext* m_pGLContext;
 };
 
 typedef boost::shared_ptr<GLTexture> GLTexturePtr;
