@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,29 +22,37 @@
 #include "GPUBandpassFilter.h"
 #include "Bitmap.h"
 #include "ShaderRegistry.h"
+#include "OGLShader.h"
 
 #include "../base/ObjectCounter.h"
 #include "../base/Exception.h"
 
 #include <iostream>
 
-#define SHADERID "BANDPASS"
+#define SHADERID "bandpass"
 
 using namespace std;
 
 namespace avg {
 
 GPUBandpassFilter::GPUBandpassFilter(const IntPoint& size, PixelFormat pfSrc, 
-        double min, double max, double postScale, bool bInvert, bool bStandalone)
-    : GPUFilter(pfSrc, B8G8R8A8, bStandalone),
+        float min, float max, float postScale, bool bInvert, bool bStandalone)
+    : GPUFilter(pfSrc, B8G8R8A8, bStandalone, SHADERID),
       m_PostScale(postScale),
       m_bInvert(bInvert),
-      m_MinFilter(size, pfSrc, R32G32B32A32F, min, true, false),
-      m_MaxFilter(size, pfSrc, R32G32B32A32F, max, true, false)
+      m_MinFilter(size, pfSrc, R32G32B32A32F, min, true, false, true),
+      m_MaxFilter(size, pfSrc, R32G32B32A32F, max, true, false, true)
 {
     ObjectCounter::get()->incRef(&typeid(*this));
+    
+    GLContext::getCurrent()->ensureFullShaders("GPUBandpassFilter");
+
     setDimensions(size);
-    initShader();
+    OGLShaderPtr pShader = getShader();
+    m_pMinTexParam = pShader->getParam<int>("u_MinTex");
+    m_pMaxTexParam = pShader->getParam<int>("u_MaxTex");
+    m_pPostScaleParam = pShader->getParam<float>("u_PostScale");
+    m_pInvertParam = pShader->getParam<int>("u_bInvert");
 }
 
 GPUBandpassFilter::~GPUBandpassFilter()
@@ -58,39 +66,13 @@ void GPUBandpassFilter::applyOnGPU(GLTexturePtr pSrcTex)
     m_MaxFilter.apply(pSrcTex);
 
     getFBO()->activate();
-    OGLShaderPtr pShader = getShader(SHADERID);
-    pShader->activate();
-    pShader->setUniformIntParam("minTex", 0);
-    pShader->setUniformIntParam("maxTex", 1);
-    pShader->setUniformFloatParam("postScale", float(m_PostScale));
-    pShader->setUniformIntParam("bInvert", m_bInvert);
+    getShader()->activate();
+    m_pMinTexParam->set(0);
+    m_pMaxTexParam->set(1);
+    m_pPostScaleParam->set(float(m_PostScale));
+    m_pInvertParam->set(m_bInvert);
     m_MaxFilter.getDestTex()->activate(GL_TEXTURE1);
     draw(m_MinFilter.getDestTex());
-
-    glproc::UseProgramObject(0);
 }
 
-void GPUBandpassFilter::initShader()
-{
-    string sProgram =
-        "uniform sampler2D minTex;\n"
-        "uniform sampler2D maxTex;\n"
-        "uniform float postScale;\n"
-        "uniform bool bInvert;\n"
-
-        "void main(void)\n"
-        "{\n"
-        "  vec4 min = texture2D(minTex, gl_TexCoord[0].st);\n" 
-        "  vec4 max = texture2D(maxTex, gl_TexCoord[0].st);\n"
-        "  gl_FragColor = vec4(0.502, 0.502, 0.502, 0)+(max-min)*postScale;\n"
-        "  if (bInvert) {\n"
-        "    gl_FragColor = vec4(1.004,1.004,1.004,1)-gl_FragColor;\n"
-        "  }\n"
-        "  gl_FragColor.a = 1.0;\n"
-        "}\n"
-        ;
-
-    getOrCreateShader(SHADERID, sProgram);
 }
-
-} // namespace
