@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 
 #include "../api.h"
 #include "GPUFilter.h"
+#include "GLShaderParam.h"
 #include "Bitmap.h"
 
 namespace avg {
@@ -31,18 +32,19 @@ namespace avg {
 class AVG_API GPUBrightnessFilter: public GPUFilter
 {
 public:
-    GPUBrightnessFilter(const IntPoint& size, PixelFormat pf, double alpha, 
+    GPUBrightnessFilter(const IntPoint& size, bool bUseAlpha, float alpha, 
             bool bStandalone=true);
     virtual ~GPUBrightnessFilter();
 
     virtual void applyOnGPU(GLTexturePtr pSrcTex);
 
 private:
-    static void initShader();
+    float m_Alpha;
 
-    double m_Alpha;
+    IntGLShaderParamPtr m_pTextureParam;
+    FloatGLShaderParamPtr m_pAlphaParam;
 };
 
-} // namespace
+}
 #endif
 
