@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,42 +26,45 @@
 #include "Filter.h"
 #include "VertexArray.h"
 #include "Bitmap.h"
-#include "PBO.h"
+#include "TextureMover.h"
 #include "FBO.h"
 
-#include <boost/thread/tss.hpp>
-
 namespace avg {
 
 class ImagingProjection;
 typedef boost::shared_ptr<ImagingProjection> ImagingProjectionPtr;
+class OGLShader;
+typedef boost::shared_ptr<OGLShader> OGLShaderPtr;
 
 class AVG_API GPUFilter: public Filter
 {
 public:
-    GPUFilter(PixelFormat pfSrc, PixelFormat pfDest, bool bStandalone,
+    GPUFilter(const std::string& sShaderID, bool bUseAlpha, bool bStandalone=false,
             unsigned numTextures=1, bool bMipmap=false);
+    GPUFilter(PixelFormat pfSrc, PixelFormat pfDest, bool bStandalone,
+            const std::string& sShaderID, unsigned numTextures=1, bool bMipmap=false);
     virtual ~GPUFilter();
-    void setDimensions(const IntPoint& srcSize);
-    void setDimensions(const IntPoint& srcSize, const IntRect& destRect,
-            unsigned texMode);
 
     virtual BitmapPtr apply(BitmapPtr pBmpSource);
     virtual void apply(GLTexturePtr pSrcTex);
     virtual void applyOnGPU(GLTexturePtr pSrcTex) = 0;
     GLTexturePtr getDestTex(int i=0) const;
     BitmapPtr getImage() const;
-    FBOPtr getFBO();
+    FBOPtr getFBO(int i=0);
 
     const IntRect& getDestRect() const;
     const IntPoint& getSrcSize() const;
-    DRect getRelDestRect() const;
+    FRect getRelDestRect() const;
     
 protected:
+    void setDimensions(const IntPoint& srcSize);
+    void setDimensions(const IntPoint& srcSize, const IntRect& destRect,
+            unsigned texMode);
+    const OGLShaderPtr& getShader() const;
+
     void draw(GLTexturePtr pTex);
-    const std::string& getStdShaderCode() const;
-    int getBlurKernelRadius(double stdDev) const;
-    GLTexturePtr calcBlurKernelTex(double stdDev, double opacity=1) const;
+    int getBlurKernelRadius(float stdDev) const;
+    GLTexturePtr calcBlurKernelTex(float stdDev, float opacity, bool bUseFloat) const;
 
 private:
     PixelFormat m_PFSrc;
@@ -71,15 +74,16 @@ private:
     bool m_bMipmap;
 
     GLTexturePtr m_pSrcTex;
-    PBOPtr m_pSrcPBO;
-    FBOPtr m_pFBO;
+    TextureMoverPtr m_pSrcMover;
+    std::vector<FBOPtr> m_pFBOs;
     IntPoint m_SrcSize;
     IntRect m_DestRect;
+    OGLShaderPtr m_pShader;
     ImagingProjectionPtr m_pProjection;
 };
 
 typedef boost::shared_ptr<GPUFilter> GPUFilterPtr;
 
-} // namespace
+}
 #endif
 
