@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,25 +22,27 @@
 #include "GPUNullFilter.h"
 #include "Bitmap.h"
 #include "ShaderRegistry.h"
+#include "OGLShader.h"
+#include "BitmapLoader.h"
 
 #include "../base/ObjectCounter.h"
 #include "../base/Exception.h"
 
 #include <iostream>
 
-#define SHADERID "NULL"
+#define SHADERID "null"
 
 using namespace std;
 
 namespace avg {
 
 GPUNullFilter::GPUNullFilter(const IntPoint& size, bool bStandalone)
-    : GPUFilter(B8G8R8A8, B8G8R8A8, bStandalone)
+    : GPUFilter(SHADERID, true, bStandalone)
 {
     ObjectCounter::get()->incRef(&typeid(*this));
 
     setDimensions(size);
-    initShader();
+    m_pTextureParam = getShader()->getParam<int>("u_Texture");
 }
 
 GPUNullFilter::~GPUNullFilter()
@@ -50,27 +52,10 @@ GPUNullFilter::~GPUNullFilter()
 
 void GPUNullFilter::applyOnGPU(GLTexturePtr pSrcTex)
 {
-    OGLShaderPtr pShader = getShader(SHADERID);
-    pShader->activate();
-    pShader->setUniformIntParam("Texture", 0);
+    getShader()->activate();
+    m_pTextureParam->set(0);
     draw(pSrcTex);
 
-    glproc::UseProgramObject(0);
 }
 
-void GPUNullFilter::initShader()
-{
-    string sProgram =
-        "uniform sampler2D Texture;\n"
-
-        "void main(void)\n"
-        "{\n"
-        "  vec4 tex = texture2D(Texture, gl_TexCoord[0].st);\n" 
-        "  gl_FragColor = tex;\n"
-        "}\n"
-        ;
-
-    getOrCreateShader(SHADERID, sProgram);
 }
-
-} // namespace
