@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,7 @@
 
 #include "GraphicsTest.h"
 #include "Bitmap.h"
+#include "BitmapLoader.h"
 #include "Filterfliprgb.h"
 #include "Filtergrayscale.h"
 
@@ -58,11 +59,8 @@ void GraphicsTest::createResultImgDir()
 BitmapPtr GraphicsTest::loadTestBmp(const std::string& sFName, PixelFormat pf)
 {
     try {
-        BitmapPtr pBmp(new Bitmap(getSrcDirName()+"testfiles/"+sFName+".png"));
-        if (pf == I8) {
-            return FilterGrayscale().apply(pBmp);
-        }
-        return pBmp;
+        string sFullName = getSrcDirName()+"../test/media/"+sFName+".png";
+        return loadBitmap(sFullName, pf);
     } catch (Exception & ex) {
         cerr << ex.getStr() << endl;
         throw;
@@ -70,18 +68,12 @@ BitmapPtr GraphicsTest::loadTestBmp(const std::string& sFName, PixelFormat pf)
 }
 
 void GraphicsTest::testEqual(Bitmap& resultBmp, const string& sFName, PixelFormat pf, 
-        double maxAverage, double maxStdDev) 
+        float maxAverage, float maxStdDev) 
 {
     BitmapPtr pBaselineBmp;
     try {
-        pBaselineBmp = BitmapPtr(new Bitmap(getSrcDirName()+"baseline/"+sFName+".png"));
-        switch (pf) {
-            case I8:
-                FilterGrayscale().applyInPlace(pBaselineBmp);
-                break;
-            default:
-                break;
-        }
+        string sFullName = getSrcDirName()+"baseline/"+sFName+".png";
+        pBaselineBmp = loadBitmap(sFullName, pf);
     } catch (Exception & ex) {
         cerr << ex.getStr() << endl;
         resultBmp.save("resultimages/"+sFName+".png");
@@ -91,7 +83,7 @@ void GraphicsTest::testEqual(Bitmap& resultBmp, const string& sFName, PixelForma
 }
 
 void GraphicsTest::testEqual(Bitmap& resultBmp, Bitmap& baselineBmp, 
-        const string& sFName, double maxAverage, double maxStdDev)
+        const string& sFName, float maxAverage, float maxStdDev)
 {
     BitmapPtr pDiffBmp;
     try {
@@ -103,8 +95,8 @@ void GraphicsTest::testEqual(Bitmap& resultBmp, Bitmap& baselineBmp,
         baselineBmp.save(sResultName+"_baseline.png");
     }
     if (pDiffBmp) {
-        double average = pDiffBmp->getAvg();
-        double stdDev = pDiffBmp->getStdDev();
+        float average = pDiffBmp->getAvg();
+        float stdDev = pDiffBmp->getStdDev();
         if (average > maxAverage || stdDev > maxStdDev) {
             TEST_FAILED("Error: Decoded image differs from baseline '" << 
                     sFName << "'. average=" << average << ", stdDev=" << stdDev);
@@ -120,9 +112,9 @@ void GraphicsTest::testEqual(Bitmap& resultBmp, Bitmap& baselineBmp,
 }
 
 void GraphicsTest::testEqualBrightness(Bitmap& resultBmp, Bitmap& baselineBmp, 
-        double epsilon)
+        float epsilon)
 {
-    double diff = fabs(resultBmp.getAvg()-baselineBmp.getAvg());
+    float diff = fabs(resultBmp.getAvg()-baselineBmp.getAvg());
     if (diff >= epsilon) {
         TEST_FAILED("Error: Baseline brightness: " << baselineBmp.getAvg()
                 << ", Result brightness: " << resultBmp.getAvg() << ", difference: " 
