@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,7 @@
 #include "OGLHelper.h"
 #include "GLTexture.h"
 
-#include "../base/Point.h"
+#include "../base/GLMHelper.h"
 
 #include <boost/shared_ptr.hpp>
 
@@ -52,8 +52,11 @@ public:
     BitmapPtr movePBOToBmp() const;
 
     bool isReadPBO() const;
+    int getID() const;
 
 private:
+    unsigned getMemNeeded() const;
+    unsigned getStride() const;
     unsigned getTarget() const;
 
     unsigned m_Usage;
