@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -68,6 +68,8 @@ string getPixelFormatString(PixelFormat pf)
             return "I8";
         case I16:
             return "I16";
+        case A8:
+            return "A8";
         case YCbCr411:
             return "YCbCr411";
         case YCbCr422:
@@ -106,7 +108,7 @@ PixelFormat stringToPixelFormat(const string& s)
     if (s == "B5G6R5") {
         return B5G6R5;
     }
-    if (s == "B8G8R8" || s == "BGR") {
+    if (s == "B8G8R8") {
         return B8G8R8;
     }
     if (s == "B8G8R8A8") {
@@ -124,7 +126,7 @@ PixelFormat stringToPixelFormat(const string& s)
     if (s == "R5G6B5") {
         return R5G6B5;
     }
-    if (s == "R8G8B8" || s == "RGB") {
+    if (s == "R8G8B8") {
         return R8G8B8;
     }
     if (s == "R8G8B8A8") {
@@ -145,10 +147,13 @@ PixelFormat stringToPixelFormat(const string& s)
     if (s == "I16") {
         return I16;
     }
-    if (s == "YCbCr411" || s == "YUV411") {
+    if (s == "A8") {
+        return A8;
+    }
+    if (s == "YCbCr411") {
         return YCbCr411;
     }
-    if (s == "YCbCr422" || s == "YUV422") {
+    if (s == "YCbCr422") {
         return YCbCr422;
     }
     if (s == "YUYV422") {
@@ -187,6 +192,18 @@ PixelFormat stringToPixelFormat(const string& s)
     return NO_PIXELFORMAT;
 }
 
+std::vector<std::string> getSupportedPixelFormats()
+{
+    std::vector<std::string> pixelFormatsVector;
+    int itPixelFormat = 0;
+    while((PixelFormat)itPixelFormat != NO_PIXELFORMAT){
+        std::string format = getPixelFormatString((PixelFormat)itPixelFormat);
+        pixelFormatsVector.push_back(format);
+        itPixelFormat++;
+    }
+    return pixelFormatsVector;
+}
+
 bool pixelFormatIsColored(PixelFormat pf)
 {
     return (pf != I8 && pf != I16 && pf != I32F);
@@ -209,6 +226,11 @@ bool pixelFormatIsPlanar(PixelFormat pf)
     return pf == YCbCr420p || pf == YCbCrJ420p || pf == YCbCrA420p;
 }
 
+bool AVG_API pixelFormatIsBlueFirst(PixelFormat pf)
+{
+    return pf == B5G6R5 || pf == B8G8R8 || pf == B8G8R8X8 || pf == B8G8R8A8;
+}
+
 unsigned getNumPixelFormatPlanes(PixelFormat pf)
 {
     switch (pf) {
@@ -256,13 +278,11 @@ unsigned getBytesPerPixel(PixelFormat pf)
         case YCbCr422:
             return 2;
         default:
-            AVG_TRACE(Logger::ERROR, "getBytesPerPixel(): Unknown format " << 
+            AVG_LOG_ERROR("getBytesPerPixel(): Unknown format " <<
                     getPixelFormatString(pf) << ".");
             AVG_ASSERT(false);
             return 0;
     }
 }
 
-
-
 }
