@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,7 @@
 
 #include "GraphicsTest.h"
 #include "Bitmap.h"
+#include "BitmapLoader.h"
 #include "Pixel32.h"
 #include "Pixel24.h"
 #include "Pixel16.h"
@@ -65,6 +66,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <glib-object.h>
+
 using namespace avg;
 using namespace std;
 
@@ -78,19 +81,20 @@ BitmapPtr initBmp(PixelFormat pf)
 
     }
     BitmapPtr pBmp(new Bitmap(IntPoint(4, height), pf));
-    for(int y = 0; y < height; ++y) {
+    int bpp = pBmp->getBytesPerPixel();
+    for (int y = 0; y < height; ++y) {
         for (int x = 0; x < 4; ++x) {
             unsigned char * pPixel = 
-                pBmp->getPixels()+y*pBmp->getStride()+x*pBmp->getBytesPerPixel();
+                    pBmp->getPixels()+y*pBmp->getStride()+x*pBmp->getBytesPerPixel();
             *(pPixel) = x;
-            if (pBmp->getBytesPerPixel() > 1) {
+            if (bpp > 1) {
                 *(pPixel+1) = 0;
             }
-            if (pBmp->getBytesPerPixel() > 2) {
+            if (bpp > 2) {
                 *(pPixel+2) = x;
                 *(pPixel) = 16*y;
             }
-            if (pBmp->getBytesPerPixel() > 3) {
+            if (bpp > 3) {
                 *(pPixel+3) = 0x80;
             }
         }
@@ -108,27 +112,26 @@ public:
 
     void runTests()
     {
-        double h, s, l;
+        float h, s, l;
         Pixel32(255, 0, 0).toHSL(h, s, l);
-        TEST(h == 0 && s == 1.0 && l == 0.5);
+        TEST(h == 0 && almostEqual(s, 1.0f) && almostEqual(l, 0.5f));
         Pixel32(0, 255, 0).toHSL(h, s, l);
-        TEST(h == 120 && s == 1.0 && l == 0.5);
+        TEST(h == 120 && almostEqual(s, 1.0f) && almostEqual(l, 0.5f));
         Pixel32(0, 0, 255).toHSL(h, s, l);
-        TEST(h == 240 && s == 1.0 && l == 0.5);
+        TEST(h == 240 && almostEqual(s, 1.0f) && almostEqual(l, 0.5f));
         Pixel32(255, 255, 0).toHSL(h, s, l);
-        TEST(h == 60 && s == 1.0 && l == 0.5);
+        TEST(h == 60 && almostEqual(s, 1.0f) && almostEqual(l, 0.5f));
         Pixel32(255, 0, 255).toHSL(h, s, l);
-        TEST(h == 300 && s == 1.0 && l == 0.5);
+        TEST(h == 300 && almostEqual(s, 1.0f) && almostEqual(l, 0.5f));
         Pixel32(0, 255, 255).toHSL(h, s, l);
-        TEST(h == 180 && s == 1.0 && l == 0.5);
+        TEST(h == 180 && almostEqual(s, 1.0f) && almostEqual(l, 0.5f));
         Pixel32(0, 0, 0).toHSL(h, s, l);
-        TEST(s == 0.0 && l == 0.0);
+        TEST(s == 0.0f && l == 0.0f);
         Pixel32(255, 255, 255).toHSL(h, s, l);
-        TEST(s == 0.0 && l == 1.0);
+        TEST(s == 0.0f && l == 1.0f);
         Pixel32(128, 128, 128).toHSL(h, s, l);
-        TEST(s == 0.0 && fabs(l-0.5) < 0.02);
+        TEST(s == 0.0f && almostEqual(l, 0.5f, 0.02f));
     }
-
 };
 
 class BitmapTest: public GraphicsTest {
@@ -293,9 +296,9 @@ private:
         cerr << "    Testing save for " << pf << endl;
         BitmapPtr pBmp = initBmp(pf);
         pBmp->save("test.png");
-        Bitmap LoadedBmp("test.png");
+        BitmapPtr pLoadedBmp = loadBitmap("test.png");
         ::remove("test.png");
-        testEqual(LoadedBmp, *pBmp, "BmpSave");
+        testEqual(*pLoadedBmp, *pBmp, "BmpSave");
     }
 
     template<class PIXEL>
@@ -314,10 +317,8 @@ private:
         bmp.drawLine(IntPoint(7,7), IntPoint(14,12), color);
         string sFName = getSrcDirName() + "baseline/LineResult" + getPixelFormatString(pf)
                 + ".png";
-        Bitmap baselineBmp(sFName);
-        Bitmap baselineBmp2(IntPoint(15,15), pf);
-        baselineBmp2.copyPixels(baselineBmp);
-        testEqual(bmp, baselineBmp2, "BmpLineDraw");
+        BitmapPtr pBaselineBmp = loadBitmap(sFName, pf);
+        testEqual(bmp, *pBaselineBmp, "BmpLineDraw");
     }
     
     void testCopyToGreyscale(PixelFormat pf)
@@ -356,7 +357,7 @@ private:
 
     template<class PIXEL>
     void testStatistics(PixelFormat pf, const PIXEL& p00, const PIXEL& p01,
-            const PIXEL& p10, const PIXEL& p11, double avg=1, double stdDev=1)
+            const PIXEL& p10, const PIXEL& p11, float avg=1, float stdDev=1)
     {
         BitmapPtr pBmp = BitmapPtr(new Bitmap(IntPoint(2,2), pf));
         pBmp->setPixel(IntPoint(0,0), p00);
@@ -568,12 +569,8 @@ public:
             if (pSrcDir) {
                 sFilename = (string)pSrcDir+"/";
             }
-            sFilename += "../test/rgb24-64x64.png";
-            Bitmap tempBmp(sFilename);
-            PixelFormat pf = R8G8B8;    
-            BitmapPtr pBmp;
-            pBmp = createBmp(tempBmp.getSize(), pf);
-            pBmp->copyPixels(tempBmp);
+            sFilename += "../test/media/rgb24-64x64.png";
+            BitmapPtr pBmp = loadBitmap(sFilename, R8G8B8);
             FilterColorize(15, 50).applyInPlace(pBmp);
             FilterFlipRGB().applyInPlace(pBmp);
         } catch (Exception & ex) {
@@ -611,7 +608,7 @@ private:
     {
         BitmapPtr pBmp(new Bitmap(IntPoint(4, 4), pf));
         initBmp<PIXEL>(pBmp);
-        double mat[9] = 
+        float mat[9] = 
                 {1,0,2,
                  0,1,0,
                  3,0,4};
@@ -659,7 +656,7 @@ private:
     {
         BitmapPtr pBmp(new Bitmap(IntPoint(4, 4), pf));
         initBmp<PIXEL>(pBmp);
-        double mat[3][3] = 
+        float mat[3][3] = 
                 {{1,0,2},
                  {0,1,0},
                  {3,0,4}};
@@ -855,10 +852,7 @@ private:
         string sFName = string("baseline/MaskResult")+sName+".png";
 //        pDestBmp->save(sFName);
         sFName = getSrcDirName()+sFName;
-        BitmapPtr pRGBXBaselineBmp = BitmapPtr(new Bitmap(sFName));
-        BitmapPtr pBaselineBmp = BitmapPtr(
-                new Bitmap(pRGBXBaselineBmp->getSize(), pBmp->getPixelFormat()));
-        pBaselineBmp->copyPixels(*pRGBXBaselineBmp);
+        BitmapPtr pBaselineBmp = loadBitmap(sFName, pBmp->getPixelFormat());
         TEST(*pDestBmp == *pBaselineBmp);
     }
 };
@@ -877,10 +871,7 @@ public:
         string sFName = "baseline/ThresholdResult.png";
 //        pDestBmp->save(sFName);
         sFName = getSrcDirName()+sFName;
-        BitmapPtr pRGBXBaselineBmp = BitmapPtr(new Bitmap(sFName));
-        BitmapPtr pBaselineBmp = BitmapPtr(
-                new Bitmap(pRGBXBaselineBmp->getSize(), pBmp->getPixelFormat()));
-        pBaselineBmp->copyPixels(*pRGBXBaselineBmp);
+        BitmapPtr pBaselineBmp = loadBitmap(sFName, pBmp->getPixelFormat());
         TEST(*pDestBmp == *pBaselineBmp);
     }
 };
@@ -966,6 +957,8 @@ public:
         runTestWithBitmap(pBmp);
         pBmp = loadTestBmp("rgb24-64x64", B8G8R8X8);
         runTestWithBitmap(pBmp);
+        pBmp = loadTestBmp("rgb24-65x65", B8G8R8);
+        runTestWithBitmap(pBmp);
     }
 
 private:
@@ -1049,6 +1042,7 @@ public:
 
 int main(int nargs, char** args)
 {
+    BitmapLoader::init(true);
     GraphicsTest::createResultImgDir();
     GraphicsTestSuite suite;
     suite.runTests();
