@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -36,9 +36,11 @@ namespace avg {
 
 void getVideoFormatAndMode(IntPoint& Size, PixelFormat pf,
         unsigned long* pVideoFormat, unsigned long* pVideoMode);
-unsigned long getFrameRateConst(double FrameRate);
+unsigned long getFrameRateConst(float FrameRate);
+float getFrameRateFloat(unsigned long FrameRate);
 CAMERA_FEATURE getFeatureID(CameraFeature Feature);
-
+void getImageSizeAndPF(unsigned long videoFormat, unsigned long videoMode,
+        IntPoint &pSize, PixelFormat &pPixelFormat);
 }
 
 #endif
