@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,7 @@
 
 #include "../graphics/Filter.h"
 
-#include "../base/Point.h"
+#include "../base/GLMHelper.h"
 #include "../base/Rect.h"
 
 #include <boost/shared_ptr.hpp>
