@@ -1,4 +1,7 @@
 //
+//  libavg - Media Playback Engine. 
+//  Copyright (C) 2003-2014 Ulrich von Zadow
+//
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
 //  License as published by the Free Software Foundation; either
@@ -47,9 +50,9 @@ public:
     std::string getParam(const std::string& sXPathExpr) const;
     bool getBoolParam(const std::string& sXPathExpr) const;
     int getIntParam(const std::string& sXPathExpr) const;
-    double getDoubleParam(const std::string& sXPathExpr) const;
-    DPoint getPointParam(const std::string& sXPathExpr) const;
-    DRect getRectParam(const std::string& sXPathExpr) const;
+    float getFloatParam(const std::string& sXPathExpr) const;
+    glm::vec2 getPointParam(const std::string& sXPathExpr) const;
+    FRect getRectParam(const std::string& sXPathExpr) const;
     xmlNodePtr getXmlNode(const std::string& sXPathExpr) const;
 
     DeDistortPtr getTransform() const;
