@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,7 @@
 #include "FilterWipeBorder.h"
 
 #include "../base/Logger.h"
-#include "../base/ProfilingZone.h"
+#include "../base/ProfilingZoneID.h"
 #include "../base/TimeSource.h"
 #include "../base/ScopeTimer.h"
 #include "../base/Exception.h"
@@ -37,9 +37,10 @@
 #include "../graphics/FilterBlur.h"
 #include "../graphics/FilterGauss.h"
 #include "../graphics/FilterMask.h"
-#include "../graphics/OGLImagingContext.h"
+#include "../graphics/GLContext.h"
 #include "../graphics/GPUBandpassFilter.h"
 #include "../graphics/GPUBlurFilter.h"
+#include "../graphics/BitmapLoader.h"
 
 #include <iostream>
 #include <stdlib.h>
@@ -102,12 +103,14 @@ TrackerThread::~TrackerThread()
 bool TrackerThread::init()
 {
     try {
-        m_pImagingContext = new OGLImagingContext();
+        m_pImagingContext = GLContext::create(
+            GLConfig(false, false, true, 1, GLConfig::AUTO, false));
         createBandpassFilter();
-        AVG_TRACE(Logger::CONFIG, "Using fragment shaders for imaging operations.");
+        AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO,
+                "Using fragment shaders for imaging operations.");
     } catch (Exception& e) {
-        AVG_TRACE(Logger::WARNING, e.getStr());
-        AVG_TRACE(Logger::CONFIG, 
+        AVG_LOG_WARNING(e.getStr());
+        AVG_TRACE(Logger::category::CONFIG, Logger::severity::WARNING,
                 "Using CPU for imaging operations (slow and inaccurate).");
         m_pImagingContext = 0;
         m_pBandpassFilter = FilterPtr(new FilterFastBandpass());
@@ -116,7 +119,7 @@ bool TrackerThread::init()
         m_StartTime = TimeSource::get()->getCurrentMillisecs(); 
         m_HistoryDelay = m_pConfig->getIntParam("/tracker/historydelay/@value");
     } catch (Exception& e) {
-        AVG_TRACE(Logger::WARNING, e.getStr());
+        AVG_LOG_WARNING(e.getStr());
     }
     
     // Done in TrackerInputDevice::ctor to work around Leopard/libdc1394 threading issue.
@@ -138,7 +141,7 @@ bool TrackerThread::work()
         ScopeTimer timer(ProfilingZoneCapture);
         pCamBmp = m_pCamera->getImage(true);
         BitmapPtr pTempBmp1;
-        while (pTempBmp1 = m_pCamera->getImage(false)) {
+        while ((pTempBmp1 = m_pCamera->getImage(false))) {
             m_NumCamFramesDiscarded++;
             m_NumFrames++;
             pCamBmp = pTempBmp1;
@@ -153,7 +156,7 @@ bool TrackerThread::work()
             FilterMask(m_pCameraMaskBmp).applyInPlace(pCamBmp);
         }
         if (m_bCreateDebugImages) {
-            boost::mutex::scoped_lock lock(*m_pMutex);
+            lock_guard lock(*m_pMutex);
             *(m_pBitmaps[TRACKER_IMG_CAMERA]) = *pCamBmp;
             ScopeTimer timer(ProfilingZoneHistogram);
             drawHistogram(m_pBitmaps[TRACKER_IMG_HISTOGRAM], pCamBmp);
@@ -171,7 +174,7 @@ bool TrackerThread::work()
         }
         BitmapPtr pCroppedBmp(new Bitmap(*pDistortedBmp, m_ROI));
         if (m_bCreateDebugImages) {
-            boost::mutex::scoped_lock lock(*m_pMutex);
+            lock_guard lock(*m_pMutex);
             m_pBitmaps[TRACKER_IMG_DISTORTED]->copyPixels(*pCroppedBmp);
         }
         if (m_pHistoryPreProcessor) {
@@ -179,7 +182,7 @@ bool TrackerThread::work()
             m_pHistoryPreProcessor->applyInPlace(pCroppedBmp);
         }
         if (m_bCreateDebugImages) {
-            boost::mutex::scoped_lock lock(*m_pMutex);
+            lock_guard lock(*m_pMutex);
             m_pBitmaps[TRACKER_IMG_NOHISTORY]->copyPixels(*pCroppedBmp);
             FilterNormalize(2).applyInPlace(m_pBitmaps[TRACKER_IMG_NOHISTORY]);
         }
@@ -191,7 +194,7 @@ bool TrackerThread::work()
                     pBmpBandpass = m_pBandpassFilter->apply(pCroppedBmp);
                 }
                 if (m_bCreateDebugImages) {
-                    boost::mutex::scoped_lock lock(*m_pMutex);
+                    lock_guard lock(*m_pMutex);
                     *(m_pBitmaps[TRACKER_IMG_HIGHPASS]) = *pBmpBandpass;
                 }
             }
@@ -205,8 +208,10 @@ bool TrackerThread::work()
 void TrackerThread::deinit()
 {
     m_pCamera = CameraPtr();
-    AVG_TRACE(Logger::PROFILE, "Total camera frames: " << m_NumFrames);
-    AVG_TRACE(Logger::PROFILE, "Camera frames discarded: " << m_NumCamFramesDiscarded);
+    AVG_TRACE(Logger::category::PROFILE, Logger::severity::INFO,
+            "Total camera frames: " << m_NumFrames);
+    AVG_TRACE(Logger::category::PROFILE, Logger::severity::INFO,
+            "Camera frames discarded: " << m_NumCamFramesDiscarded);
     if (m_pBandpassFilter) {
         m_pBandpassFilter.reset();
     }
@@ -218,7 +223,7 @@ void TrackerThread::deinit()
 void TrackerThread::setConfig(TrackerConfig config, IntRect roi, 
         BitmapPtr ppBitmaps[NUM_TRACKER_IMAGES])
 {
-    boost::mutex::scoped_lock lock(*m_pMutex);
+    lock_guard lock(*m_pMutex);
     try {
         m_TouchThreshold = config.getIntParam("/tracker/touch/threshold/@value");
     } catch (Exception&) {
@@ -272,10 +277,7 @@ void TrackerThread::setConfig(TrackerConfig config, IntRect roi,
         if (sCameraMaskFName == "") {
             m_pCameraMaskBmp = BitmapPtr();
         } else {
-            BitmapPtr pRGBXCameraMaskBmp = BitmapPtr(new Bitmap(sCameraMaskFName));
-            m_pCameraMaskBmp = BitmapPtr(
-                    new Bitmap(pRGBXCameraMaskBmp->getSize(), I8));
-            m_pCameraMaskBmp->copyPixels(*pRGBXCameraMaskBmp);        
+            BitmapPtr pRGBXCameraMaskBmp = loadBitmap(sCameraMaskFName, I8);
         }
     }
     m_pConfig = TrackerConfigPtr(new TrackerConfig(config));
@@ -309,10 +311,10 @@ void TrackerThread::setBitmaps(IntRect roi, BitmapPtr ppBitmaps[NUM_TRACKER_IMAG
 void TrackerThread::createBandpassFilter()
 {
     if (m_TouchThreshold != 0) {
-        double bandpassMin = m_pConfig->getDoubleParam("/tracker/touch/bandpass/@min");
-        double bandpassMax = m_pConfig->getDoubleParam("/tracker/touch/bandpass/@max");
-        double bandpassPostMult = 
-            m_pConfig->getDoubleParam("/tracker/touch/bandpasspostmult/@value");
+        float bandpassMin = m_pConfig->getFloatParam("/tracker/touch/bandpass/@min");
+        float bandpassMax = m_pConfig->getFloatParam("/tracker/touch/bandpass/@max");
+        float bandpassPostMult = 
+            m_pConfig->getFloatParam("/tracker/touch/bandpasspostmult/@value");
         if (m_pImagingContext) {
             m_pBandpassFilter = FilterPtr(new GPUBandpassFilter(m_ROI.size(), I8,
                         bandpassMin, bandpassMax, bandpassPostMult, m_bTrackBrighter));
@@ -366,16 +368,16 @@ void TrackerThread::drawHistogram(BitmapPtr pDestBmp, BitmapPtr pSrcBmp)
     }
 }
 
-inline bool isInbetween(double x, double min, double max)
+inline bool isInbetween(float x, float min, float max)
 {
     return x >= min && x <= max;
 }
 
 bool TrackerThread::isRelevant(BlobPtr pBlob, int minArea, int maxArea,
-        double minEccentricity, double maxEccentricity)
+        float minEccentricity, float maxEccentricity)
 {
     bool res;
-    res = isInbetween(pBlob->getArea(), minArea, maxArea) && 
+    res = isInbetween(pBlob->getArea(), float(minArea), float(maxArea)) && 
             isInbetween(pBlob->getEccentricity(), minEccentricity, maxEccentricity);
     return res;
 }
@@ -390,8 +392,10 @@ BlobVectorPtr TrackerThread::findRelevantBlobs(BlobVectorPtr pBlobs, bool bTouch
     }
     int minArea = m_pConfig->getIntParam(sConfigPrefix+"areabounds/@min");
     int maxArea = m_pConfig->getIntParam(sConfigPrefix+"areabounds/@max");
-    double minEccentricity = m_pConfig->getDoubleParam(sConfigPrefix+"eccentricitybounds/@min");
-    double maxEccentricity = m_pConfig->getDoubleParam(sConfigPrefix+"eccentricitybounds/@max");
+    float minEccentricity = m_pConfig->getFloatParam(sConfigPrefix+
+            "eccentricitybounds/@min");
+    float maxEccentricity = m_pConfig->getFloatParam(sConfigPrefix+
+            "eccentricitybounds/@max");
     
     BlobVectorPtr pRelevantBlobs(new BlobVector());
     for(BlobVector::iterator it = pBlobs->begin(); it != pBlobs->end(); ++it) {
@@ -420,9 +424,9 @@ void TrackerThread::drawBlobs(BlobVectorPtr pBlobs, BitmapPtr pSrcBmp,
     }
     int minArea = m_pConfig->getIntParam(sConfigPrefix+"areabounds/@min");
     int maxArea = m_pConfig->getIntParam(sConfigPrefix+"areabounds/@max");
-    double minEccentricity = m_pConfig->getDoubleParam(
+    float minEccentricity = m_pConfig->getFloatParam(
             sConfigPrefix+"eccentricitybounds/@min");
-    double maxEccentricity = m_pConfig->getDoubleParam(
+    float maxEccentricity = m_pConfig->getFloatParam(
             sConfigPrefix+"eccentricitybounds/@max");
     
     // Get max. pixel value in Bitmap
@@ -466,9 +470,9 @@ void TrackerThread::calcContours(BlobVectorPtr pBlobs)
     sConfigPrefix = "/tracker/track/";
     int minArea = m_pConfig->getIntParam(sConfigPrefix+"areabounds/@min");
     int maxArea = m_pConfig->getIntParam(sConfigPrefix+"areabounds/@max");
-    double minEccentricity = m_pConfig->getDoubleParam(
+    float minEccentricity = m_pConfig->getFloatParam(
             sConfigPrefix+"eccentricitybounds/@min");
-    double maxEccentricity = m_pConfig->getDoubleParam(
+    float maxEccentricity = m_pConfig->getFloatParam(
             sConfigPrefix+"eccentricitybounds/@max");
     
     int ContourPrecision = m_pConfig->getIntParam("/tracker/contourprecision/@value");
@@ -510,7 +514,7 @@ void TrackerThread::calcBlobs(BitmapPtr pTrackBmp, BitmapPtr pTouchBmp, long lon
     BlobVectorPtr pTouchComps;
     {
         ScopeTimer timer(ProfilingZoneComps);
-        boost::mutex::scoped_lock lock(*m_pMutex);
+        lock_guard lock(*m_pMutex);
         BitmapPtr pDestBmp;
         if (m_bCreateFingerImage) {
             Pixel32 Black(0x00, 0x00, 0x00, 0x00);
