@@ -1,6 +1,6 @@
 //
-//  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  libavg - Media Playback Engine.
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@
 #include "../base/Logger.h"
 #include "../base/Exception.h"
 #include "../base/StringHelper.h"
+#include "../base/GLMHelper.h"
 
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -62,15 +63,14 @@ namespace {
     }
 }
 
+namespace avg {
 
-V4LCamera::V4LCamera(std::string sDevice, int channel, IntPoint size, PixelFormat camPF,
-        PixelFormat destPF, double frameRate)
-    : Camera(camPF, destPF),
+V4LCamera::V4LCamera(string sDevice, int channel, IntPoint size, PixelFormat camPF,
+        PixelFormat destPF, float frameRate)
+    : Camera(camPF, destPF, size, frameRate),
       m_Fd(-1),
       m_Channel(channel),
-      m_sDevice(sDevice), 
-      m_ImgSize(size),
-      m_FrameRate(frameRate)
+      m_sDevice(sDevice)
 {
     m_v4lPF = getV4LPF(camPF);
     if (m_sDevice == "") {
@@ -87,27 +87,30 @@ V4LCamera::V4LCamera(std::string sDevice, int channel, IntPoint size, PixelForma
     m_FeaturesNames[V4L2_CID_WHITENESS] = "whiteness";
     m_FeaturesNames[V4L2_CID_GAMMA] = "gamma";
     m_FeaturesNames[V4L2_CID_SATURATION] = "saturation";
-    
-    struct stat st; 
+
+    struct stat st;
     if (stat(m_sDevice.c_str(), &st) == -1) {
-        fatalError(string("Unable to access v4l2 device '")+m_sDevice+"'." );
+        AVG_ASSERT_MSG(false, (string("Unable to access v4l2 device '" +
+                m_sDevice + "'.").c_str()));
     }
 
     if (!S_ISCHR (st.st_mode)) {
-        fatalError(string("'")+m_sDevice+" is not a v4l2 device.");
+        AVG_ASSERT_MSG(false, (string("'" + m_sDevice +
+                " is not a v4l2 device.").c_str()));
     }
 
     m_Fd = ::open(m_sDevice.c_str(), O_RDWR /* required */ | O_NONBLOCK, 0);
 
     if (m_Fd == -1) {
-        fatalError(string("Unable to open v4l2 device '") + m_sDevice + "'.");
+        AVG_ASSERT_MSG(false, (string("Unable to open v4l2 device '" + m_sDevice
+                + "'.").c_str()));
     }
-    
+
     initDevice();
-    AVG_TRACE(Logger::CONFIG, "V4L2 Camera opened");
+    AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO, "V4L2 Camera opened");
 }
 
-V4LCamera::~V4LCamera() 
+V4LCamera::~V4LCamera()
 {
     close();
 }
@@ -117,7 +120,7 @@ void V4LCamera::close()
     enum v4l2_buf_type type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     int rc = xioctl(m_Fd, VIDIOC_STREAMOFF, &type);
     if (rc == -1) {
-        AVG_TRACE(Logger::ERROR, "VIDIOC_STREAMOFF");
+        AVG_LOG_ERROR("VIDIOC_STREAMOFF");
     }
     vector<Buffer>::iterator it;
     for (it = m_vBuffers.begin(); it != m_vBuffers.end(); ++it) {
@@ -127,21 +130,22 @@ void V4LCamera::close()
     m_vBuffers.clear();
 
     ::close(m_Fd);
-    AVG_TRACE(Logger::CONFIG, "V4L2 Camera closed");
+    AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO, "V4L2 Camera closed");
 
     m_Fd = -1;
 }
 
-IntPoint V4LCamera::getImgSize()
-{
-    return m_ImgSize;
-}
-
 int V4LCamera::getV4LPF(PixelFormat pf)
 {
     switch (pf) {
         case I8:
             return V4L2_PIX_FMT_GREY;
+        case BAYER8:
+        case BAYER8_BGGR:
+        case BAYER8_GBRG:
+        case BAYER8_GRBG:
+        case BAYER8_RGGB:
+            return V4L2_PIX_FMT_GREY;
         case YCbCr411:
             return V4L2_PIX_FMT_Y41P;
         case YCbCr422:
@@ -154,7 +158,7 @@ int V4LCamera::getV4LPF(PixelFormat pf)
             return V4L2_PIX_FMT_BGR24;
         default:
             throw Exception(AVG_ERR_INVALID_ARGS,
-                    "Unsupported or illegal value for camera pixel format '" 
+                    "Unsupported or illegal value for camera pixel format '"
                 + getPixelFormatString(pf) + "'.");
     }
 }
@@ -163,37 +167,37 @@ BitmapPtr V4LCamera::getImage(bool bWait)
 {
     struct v4l2_buffer buf;
     CLEAR(buf);
-    
+
     // wait for incoming data blocking, timeout 2s
     if (bWait) {
         fd_set fds;
         struct timeval tv;
         int rc;
-    
+
         FD_ZERO(&fds);
         FD_SET(m_Fd, &fds);
-    
+
         /* Timeout. */
         tv.tv_sec = 2;
         tv.tv_usec = 0;
-    
+
         rc = select (m_Fd+1, &fds, NULL, NULL, &tv);
 
-        // caught signal or something else  
+        // caught signal or something else
         if (rc == -1) {
-            AVG_TRACE(Logger::WARNING, "V4L2: select failed.");
+            AVG_LOG_WARNING("V4L2: select failed.");
             return BitmapPtr();
         }
         // timeout
         if (rc == 0) {
-            AVG_TRACE(Logger::WARNING, "V4L2: Timeout while waiting for image data");
+            AVG_LOG_WARNING("V4L2: Timeout while waiting for image data");
             return BitmapPtr();
         }
     }
-    
+
     buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     buf.memory = V4L2_MEMORY_MMAP;
-    
+
     // dequeue filled buffer
     if (xioctl (m_Fd, VIDIOC_DQBUF, &buf) == -1) {
         if (errno == EAGAIN) {
@@ -203,31 +207,30 @@ BitmapPtr V4LCamera::getImage(bool bWait)
             AVG_ASSERT(false);
         }
     }
-    
+
     unsigned char * pCaptureBuffer = (unsigned char*)m_vBuffers[buf.index].start;
-   
-    double lineLen;
+
+    float lineLen;
     switch (getCamPF()) {
         case YCbCr411:
-            lineLen = m_ImgSize.x*1.5;
+            lineLen = getImgSize().x*1.5f;
             break;
         case YCbCr420p:
-            lineLen = m_ImgSize.x;
+            lineLen = getImgSize().x;
             break;
         default:
-            lineLen = m_ImgSize.x*getBytesPerPixel(getCamPF());
+            lineLen = getImgSize().x*getBytesPerPixel(getCamPF());
     }
-    BitmapPtr pCamBmp(new Bitmap(m_ImgSize, getCamPF(), pCaptureBuffer, lineLen,
+    BitmapPtr pCamBmp(new Bitmap(getImgSize(), getCamPF(), pCaptureBuffer, lineLen,
             false, "TempCameraBmp"));
 
     BitmapPtr pDestBmp = convertCamFrameToDestPF(pCamBmp);
 
     // enqueues free buffer for mmap
     if (-1 == xioctl (m_Fd, VIDIOC_QBUF, &buf)) {
-        throw(Exception(AVG_ERR_CAMERA_FATAL, 
-                "V4L Camera: failed to enqueue image buffer."));
+        AVG_ASSERT_MSG(false, "V4L Camera: failed to enqueue image buffer.");
     }
-    
+
     return pDestBmp;
 }
 
@@ -236,28 +239,23 @@ bool V4LCamera::isCameraAvailable()
     return m_bCameraAvailable;
 }
 
-const std::string& V4LCamera::getDevice() const
+const string& V4LCamera::getDevice() const
 {
     return m_sDevice;
 }
 
-const std::string& V4LCamera::getDriverName() const
+const string& V4LCamera::getDriverName() const
 {
     return m_sDriverName;
 }
 
-double V4LCamera::getFrameRate() const
+string V4LCamera::getFeatureName(V4LCID_t v4lFeature)
 {
-    return m_FrameRate;
-}
-
-std::string V4LCamera::getFeatureName(V4LCID_t v4lFeature)
-{
-    std::string sName = m_FeaturesNames[v4lFeature];
+    string sName = m_FeaturesNames[v4lFeature];
     if (sName == "") {
         sName = "UNKNOWN";
     }
-    
+
     return sName;
 }
 
@@ -277,21 +275,21 @@ V4LCID_t V4LCamera::getFeatureID(CameraFeature feature) const
     } else if (feature == CAM_FEATURE_SATURATION) {
         v4lFeature = V4L2_CID_SATURATION;
     } else {
-        AVG_TRACE(Logger::WARNING, "feature " << cameraFeatureToString(feature)
+        AVG_LOG_WARNING("feature " << cameraFeatureToString(feature)
                 << " not supported for V4L2.");
         return -1;
     }
-    
+
     return v4lFeature;
 }
 
 bool V4LCamera::isFeatureSupported(V4LCID_t v4lFeature) const
 {
     struct v4l2_queryctrl queryCtrl;
-    
+
     CLEAR(queryCtrl);
     queryCtrl.id = v4lFeature;
-    
+
     if (ioctl (m_Fd, VIDIOC_QUERYCTRL, &queryCtrl) == -1) {
         if (errno != EINVAL) {
             cerr << "Got " << strerror(errno) << endl;
@@ -310,9 +308,9 @@ bool V4LCamera::isFeatureSupported(V4LCID_t v4lFeature) const
 int V4LCamera::getFeature(CameraFeature feature) const
 {
     V4LCID_t v4lFeature = getFeatureID(feature);
-    
+
     FeatureMap::const_iterator it = m_Features.find(v4lFeature);
-    
+
     if (it == m_Features.end()) {
         return 0;
     } else {
@@ -323,191 +321,191 @@ int V4LCamera::getFeature(CameraFeature feature) const
 void V4LCamera::setFeature(V4LCID_t v4lFeature, int value)
 {
     if (!m_bCameraAvailable) {
-        AVG_TRACE(Logger::WARNING, "setFeature() called before opening device: ignored");
+        AVG_LOG_WARNING("setFeature() called before opening device: ignored");
         return;
     }
 
     if (!isFeatureSupported(v4lFeature)) {
-        AVG_TRACE(Logger::WARNING, "Camera feature " << getFeatureName(v4lFeature) <<
+        AVG_LOG_WARNING("Camera feature " << getFeatureName(v4lFeature) <<
             " is not supported by hardware");
         return;
     }
-    
+
     struct v4l2_control control;
-    
+
     CLEAR(control);
     control.id = v4lFeature;
     control.value = value;
 
-//    AVG_TRACE(Logger::APP, "Setting feature " << getFeatureName(v4lFeature) << 
+//    AVG_TRACE(Logger::category::APP, "Setting feature " << getFeatureName(v4lFeature) <<
 //      " to "<< value);
 
     if (ioctl(m_Fd, VIDIOC_S_CTRL, &control) == -1) {
-        AVG_TRACE(Logger::ERROR, "Cannot set feature " <<
-            m_FeaturesNames[v4lFeature]);
+        AVG_LOG_ERROR("Cannot set feature " << m_FeaturesNames[v4lFeature]);
     }
 }
 
 void V4LCamera::setFeatureOneShot(CameraFeature feature)
 {
-    AVG_TRACE(Logger::WARNING, "setFeatureOneShot is not supported by V4L cameras.");
+    AVG_LOG_WARNING("setFeatureOneShot is not supported by V4L cameras.");
 }
 
 int V4LCamera::getWhitebalanceU() const
 {
-    AVG_TRACE(Logger::WARNING, "getWhitebalance is not supported by V4L cameras.");
+    AVG_LOG_WARNING("getWhitebalance is not supported by V4L cameras.");
     return 0;
 }
 
 int V4LCamera::getWhitebalanceV() const
 {
-    AVG_TRACE(Logger::WARNING, "getWhitebalance is not supported by V4L cameras.");
+    AVG_LOG_WARNING("getWhitebalance is not supported by V4L cameras.");
     return 0;
 }
 
 void V4LCamera::setWhitebalance(int u, int v, bool bIgnoreOldValue)
 {
-    setFeature(V4L2_CID_RED_BALANCE, u); 
-    setFeature(V4L2_CID_BLUE_BALANCE, v); 
+    setFeature(V4L2_CID_RED_BALANCE, u);
+    setFeature(V4L2_CID_BLUE_BALANCE, v);
 }
 
-int dumpCameras_open(int j)
+int V4LCamera::checkCamera(int j)
 {
     stringstream minorDeviceNumber;
     minorDeviceNumber << j;
     string address = "/dev/video";
-    string result = address + minorDeviceNumber.str();  
+    string result = address + minorDeviceNumber.str();
     int fd = ::open(result.c_str(), O_RDWR /* required */ | O_NONBLOCK, 0);
     return fd;
 }
 
-v4l2_capability dumpCameraCapabilities(int fd)
+v4l2_capability getCamCapabilities(int fd)
 {
     v4l2_capability capability;
     memset(&capability, 0, sizeof(capability));
-    int rc = ioctl(fd, VIDIOC_QUERYCAP, &capability);
-    if (rc != -1) {
-        cout << capability.card << ":" << endl;
-        cout << "    Driver:  " << capability.driver << endl;
-        cout << "  Location:  " << capability.bus_info;
-        cout << endl << endl;
-    }
+    ioctl(fd, VIDIOC_QUERYCAP, &capability);
     return capability;
 }
 
-void dumpSupportedImgFormats(int fd)
+PixelFormat V4LCamera::intToPixelFormat(unsigned int pixelformat)
+{
+    switch (pixelformat) {
+        case v4l2_fourcc('Y','U','Y','V'):
+            return YUYV422;
+        case v4l2_fourcc('U','Y','V','Y'):
+            return YCbCr422;
+        case v4l2_fourcc('G','R','E','Y'):
+            return I8;
+        case v4l2_fourcc('Y','1','6',' '):
+            return I16;
+        case v4l2_fourcc('R','G','B','3'):
+            return R8G8B8;
+        case v4l2_fourcc('B','G','R','3'):
+            return B8G8R8;
+        default:
+            return NO_PIXELFORMAT;
+    }
+}
+
+int V4LCamera::countCameras()
+{
+    int numberOfCameras = 0;
+    for (int j = 0; j < 256; j++) {
+        int fd = checkCamera(j);
+        if (fd != -1) {
+            numberOfCameras++;
+        }
+    }
+    return numberOfCameras;
+}
+
+CameraInfo* V4LCamera::getCameraInfos(int deviceNumber)
+{
+    int fd = checkCamera(deviceNumber);
+    if (fd == -1) {
+        AVG_ASSERT(false);
+        return NULL;
+    }
+    stringstream ss;
+    ss << "/dev/video" << deviceNumber;
+    CameraInfo* camInfo = new CameraInfo("video4linux", ss.str());
+    v4l2_capability capability = getCamCapabilities(fd);
+    if (capability.capabilities & V4L2_CAP_VIDEO_CAPTURE) {
+        getCameraImageFormats(fd, camInfo);
+        getCameraControls(fd, camInfo);
+    }
+    return camInfo;
+}
+
+void V4LCamera::getCameraImageFormats(int fd, CameraInfo* camInfo)
 {
-    cout << "Suported Image Formats:" << endl;
     for (int i = 0;; i++) {
+//        cerr << i << endl;
         v4l2_fmtdesc fmtDesc;
         memset(&fmtDesc, 0, sizeof(fmtDesc));
         fmtDesc.index = i;
         fmtDesc.type  = V4L2_BUF_TYPE_VIDEO_CAPTURE;
         int rc = ioctl(fd, VIDIOC_ENUM_FMT, &fmtDesc);
         if (rc == -1) {
-            break; 
+            break;
         }
         v4l2_frmsizeenum frmSizeEnum;
         memset(&frmSizeEnum, 0, sizeof (frmSizeEnum));
         frmSizeEnum.index = 0;
         frmSizeEnum.pixel_format = fmtDesc.pixelformat;
-        bool bSupported = false;
         while (ioctl(fd, VIDIOC_ENUM_FRAMESIZES, &frmSizeEnum) == 0) {
-            string sAvgPixelformat;
-            switch (fmtDesc.pixelformat) {
-                case v4l2_fourcc('Y','U','Y','V'):
-                    sAvgPixelformat = "YUYV422";
-                    bSupported = true;
-                    break; 
-                case v4l2_fourcc('U','Y','V','Y'):
-                    sAvgPixelformat = "YUV422";
-                    bSupported = true;
-                    break; 
-                case v4l2_fourcc('G','R','E','Y'):
-                    sAvgPixelformat = "I8"; 
-                    bSupported = true;
-                    break; 
-                case v4l2_fourcc('Y','1','6',' '):
-                    sAvgPixelformat = "I16";
-                    bSupported = true;
-                    break; 
-                case v4l2_fourcc('R','G','B','3'):
-                    sAvgPixelformat = "RGB";
-                    bSupported = true;
-                    break; 
-                case v4l2_fourcc('B','G','R','3'):
-                    sAvgPixelformat = "BGR";
-                    bSupported = true;
-                    break; 
-                default:
-                    break;
-            }
-            
-            if (bSupported) {
+/*            fprintf(stdout, "  pixelformat  :%c%c%c%c\\n",
+                                fmtDesc.pixelformat & 0xFF,
+                                (fmtDesc.pixelformat >> 8) & 0xFF,
+                                (fmtDesc.pixelformat >> 16) & 0xFF, 
+                                (fmtDesc.pixelformat >> 24) & 0xFF);
+*/
+            PixelFormat pixFormat = intToPixelFormat(fmtDesc.pixelformat);
+            if (pixFormat != NO_PIXELFORMAT) {
                 v4l2_frmivalenum frmIvalEnum;
-                cout << "   " << sAvgPixelformat << " ";
-                cout << "  (" << frmSizeEnum.discrete.width << ", ";
-                cout << frmSizeEnum.discrete.height << ")";
-                cout << "   fps: ";
                 memset (&frmIvalEnum, 0, sizeof (frmIvalEnum));
-                frmIvalEnum.index = 0;      
-                frmIvalEnum.pixel_format = frmSizeEnum.pixel_format;  
-                frmIvalEnum.width = frmSizeEnum.discrete.width;   
+                frmIvalEnum.index = 0;
+                frmIvalEnum.pixel_format = frmSizeEnum.pixel_format;
+                frmIvalEnum.width = frmSizeEnum.discrete.width;
                 frmIvalEnum.height = frmSizeEnum.discrete.height;
+                IntPoint size;
+                size.x = frmSizeEnum.discrete.width;
+                size.y = frmSizeEnum.discrete.height;
+                std::vector<float> framerates;
                 while (ioctl(fd, VIDIOC_ENUM_FRAMEINTERVALS, &frmIvalEnum) == 0) {
-                    cout << frmIvalEnum.discrete.denominator << "/";
-                    frmIvalEnum.index++;                                 
+                    framerates.push_back(frmIvalEnum.discrete.denominator);
+                    frmIvalEnum.index++;
                 }
-                cout << endl;
+                CameraImageFormat camImFormat = CameraImageFormat(size, pixFormat,
+                        framerates);
+                camInfo->addImageFormat(camImFormat);
             }
             frmSizeEnum.index++;
         }
     }
 }
 
-void dumpCameraControls(int fd)
+void V4LCamera::getCameraControls(int fd, CameraInfo* camInfo)
 {
-    cout << endl << "Camera Controls:" << endl;
     v4l2_queryctrl queryCtrl;
     for (queryCtrl.id = V4L2_CID_BASE; queryCtrl.id < V4L2_CID_LASTP1; queryCtrl.id++) {
         int rc = ioctl (fd, VIDIOC_QUERYCTRL, &queryCtrl);
-        if (rc != -1) {
-            if (queryCtrl.flags & V4L2_CTRL_FLAG_DISABLED) {
-                continue;
-            }
-            cout << "  " << queryCtrl.name << ":" << endl;
-            cout << "    Min: " << queryCtrl.minimum << " | ";
-            cout << "Max: " << queryCtrl.maximum << " | ";
-            cout << "Default: "<< queryCtrl.default_value << endl;
-        } else {
-            if (errno != EINVAL) {
-                perror("VIDIOC_QUERYCTRL");
-                exit(EXIT_FAILURE);
-            }          
+        if (rc == -1) {
+            continue;
         }
+        if (queryCtrl.flags & V4L2_CTRL_FLAG_DISABLED) {
+            continue;
+        }
+        stringstream ss;
+        ss << queryCtrl.name;
+        std::string sControlName = ss.str();
+        int min = queryCtrl.minimum;
+        int max = queryCtrl.maximum;
+        int defaultValue = queryCtrl.default_value;
+        CameraControl camControl = CameraControl(sControlName, min, max, defaultValue);
+        camInfo->addControl(camControl);
     }
 }
 
-void V4LCamera::dumpCameras()
-{ 
-    for(int j = 0; j < 256; j++){
-        int fd = dumpCameras_open(j);
-        if (fd != -1) {
-            cout << "------------------------Video4linux Camera-------------------------";
-            cout << endl;
-            cout << "/dev/video" << j << " ";
-            v4l2_capability capability = dumpCameraCapabilities(fd);
-            if (capability.capabilities & V4L2_CAP_VIDEO_CAPTURE) { 
-                dumpSupportedImgFormats(fd);   
-                dumpCameraControls(fd);
-            } 
-            cout << "-------------------------------------------------------------------";
-            cout << endl;
-        }
-    }
-}
-
-
 void V4LCamera::setFeature(CameraFeature feature, int value, bool bIgnoreOldValue)
 {
     // ignore -1 coming from default unbiased cameranode parameters
@@ -518,8 +516,6 @@ void V4LCamera::setFeature(CameraFeature feature, int value, bool bIgnoreOldValu
     V4LCID_t v4lFeature = getFeatureID(feature);
     m_Features[v4lFeature] = value;
 
-//    AVG_TRACE(Logger::WARNING,"Setting feature " << sFeature <<
-//        " to " << value);
     if (m_bCameraAvailable) {
         setFeature(v4lFeature, value);
     }
@@ -527,7 +523,7 @@ void V4LCamera::setFeature(CameraFeature feature, int value, bool bIgnoreOldValu
 
 void V4LCamera::startCapture()
 {
-//  AVG_TRACE(Logger::APP, "Entering startCapture()...");
+//  AVG_TRACE(Logger::category::APP, "Entering startCapture()...");
 
     unsigned int i;
     enum v4l2_buf_type type;
@@ -544,7 +540,7 @@ void V4LCamera::startCapture()
         int err = xioctl(m_Fd, VIDIOC_QBUF, &buf);
         AVG_ASSERT(err != -1);
     }
-    
+
     type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     int err= xioctl (m_Fd, VIDIOC_STREAMON, &type);
     AVG_ASSERT(err != -1);
@@ -552,8 +548,8 @@ void V4LCamera::startCapture()
 
 void V4LCamera::initDevice()
 {
-//  AVG_TRACE(Logger::APP, "Entering initDevice()...");
-    
+//  AVG_TRACE(Logger::category::APP, "Entering initDevice()...");
+
     struct v4l2_capability cap;
     struct v4l2_cropcap CropCap;
     struct v4l2_crop Crop;
@@ -562,17 +558,17 @@ void V4LCamera::initDevice()
 
     if (xioctl(m_Fd, VIDIOC_QUERYCAP, &cap) == -1) {
         close();
-        fatalError(m_sDevice + " is not a valid V4L2 device.");
+        AVG_ASSERT_MSG(false, (m_sDevice + " is not a valid V4L2 device.").c_str());
     }
 
     if (!(cap.capabilities & V4L2_CAP_VIDEO_CAPTURE)) {
         close();
-        fatalError(m_sDevice + " does not support capturing");
+        AVG_ASSERT_MSG(false, (m_sDevice + " does not support capturing").c_str());
     }
 
     if (!(cap.capabilities & V4L2_CAP_STREAMING)) {
         close();
-        fatalError(m_sDevice + " does not support streaming i/os");
+        AVG_ASSERT_MSG(false, (m_sDevice + " does not support streaming i/os").c_str());
     }
     m_sDriverName = (const char *)cap.driver;
 
@@ -594,58 +590,65 @@ void V4LCamera::initDevice()
                     break;
             }
         }
-    } else {   
+    } else {
             /* Errors ignored. */
     }
 
     CLEAR(fmt);
 
     fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-    fmt.fmt.pix.width = m_ImgSize.x; 
-    fmt.fmt.pix.height = m_ImgSize.y;
+    fmt.fmt.pix.width = getImgSize().x;
+    fmt.fmt.pix.height = getImgSize().y;
     fmt.fmt.pix.pixelformat = m_v4lPF;
     fmt.fmt.pix.field = V4L2_FIELD_ANY;
     int rc = xioctl(m_Fd, VIDIOC_S_FMT, &fmt);
-    if (int(fmt.fmt.pix.width) != m_ImgSize.x || int(fmt.fmt.pix.height) != m_ImgSize.y
-        || rc == -1)
+    if (int(fmt.fmt.pix.width) != getImgSize().x ||
+            int(fmt.fmt.pix.height) != getImgSize().y || rc == -1)
     {
-        throw(Exception(AVG_ERR_CAMERA_NONFATAL, 
+        throw(Exception(AVG_ERR_CAMERA_NONFATAL,
                 string("Unable to set V4L camera image format: '")
                 +strerror(errno)
-                +"'. Try using avg_showcamera.py --dump to find out what the device supports."));
+                +"'. Try using avg_showcamera.py --list to find out what the device supports."));
     }
 
     CLEAR(StreamParam);
-
     StreamParam.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-    StreamParam.parm.capture.timeperframe.numerator = 1;
-    StreamParam.parm.capture.timeperframe.denominator = (int) m_FrameRate;
-    rc = xioctl(m_Fd, VIDIOC_S_PARM, &StreamParam);
-    if (m_FrameRate != StreamParam.parm.capture.timeperframe.denominator || rc == -1) {
-        throw(Exception(AVG_ERR_CAMERA_NONFATAL,
-                string("Unable to set V4L camera framerate: '")
-                +strerror(errno)
-                +"'. Try using avg_showcamera.py --dump to find out what the device supports."));
+    rc = xioctl(m_Fd, VIDIOC_G_PARM, &StreamParam);
+
+    if(StreamParam.parm.capture.capability == V4L2_CAP_TIMEPERFRAME) {
+        CLEAR(StreamParam);
+
+        StreamParam.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+        StreamParam.parm.capture.timeperframe.numerator = 1;
+        StreamParam.parm.capture.timeperframe.denominator = (int)getFrameRate();
+        rc = xioctl(m_Fd, VIDIOC_S_PARM, &StreamParam);
+        if (getFrameRate() != StreamParam.parm.capture.timeperframe.denominator ||
+                rc == -1) 
+        {
+            throw(Exception(AVG_ERR_CAMERA_NONFATAL,
+                        string("Unable to set V4L camera framerate: '")
+                        +strerror(errno)
+                        +"'. Try using avg_showcamera.py --list to find out what the device supports."));
+        }
     }
-    m_FrameRate = (double)StreamParam.parm.capture.timeperframe.denominator / \
-            (double)StreamParam.parm.capture.timeperframe.numerator;
+
+    initMMap();
 
-    initMMap ();
-    
     // TODO: string channel instead of numeric
     // select channel
     if (xioctl(m_Fd, VIDIOC_S_INPUT, &m_Channel) == -1) {
         close();
-        fatalError(string("Cannot set MUX channel ")+toString(m_Channel));
+        AVG_ASSERT_MSG(false, (string("Cannot set MUX channel " +
+                toString(m_Channel))).c_str());
     }
-    
+
     m_bCameraAvailable = true;
 
     for (FeatureMap::iterator it=m_Features.begin(); it != m_Features.end(); it++) {
         setFeature(it->first, it->second);
     }
 
-    
+
 }
 
 void V4LCamera::initMMap()
@@ -660,13 +663,14 @@ void V4LCamera::initMMap()
     if (xioctl(m_Fd, VIDIOC_REQBUFS, &req) == -1) {
         if (EINVAL == errno) {
             close();
-            fatalError(m_sDevice+" does not support memory mapping");
+            AVG_ASSERT_MSG(false, (m_sDevice +
+                    " does not support memory mapping").c_str());
         } else {
             cerr << "errno: " << strerror(errno);
             AVG_ASSERT(false);
         }
     }
-    
+
     if (req.count < 2) {
         cerr << "Insufficient buffer memory on " << m_sDevice;
         AVG_ASSERT(false);
@@ -689,7 +693,7 @@ void V4LCamera::initMMap()
         }
 
         tmp.length = buf.length;
-        
+
         tmp.start = mmap (NULL /* start anywhere */,
             buf.length,
             PROT_READ | PROT_WRITE /* required */,
@@ -699,10 +703,11 @@ void V4LCamera::initMMap()
         if (MAP_FAILED == tmp.start) {
             AVG_ASSERT(false);
         }
-                
+
         m_vBuffers.push_back(tmp);
     }
 }
+}
 
 
 
