@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,8 @@
 
 #include "ArgList.h"
 
-#include "Node.h"
+#include "ExportedObject.h"
+#include "FontStyle.h"
 
 #include "../base/Logger.h"
 #include "../base/Exception.h"
@@ -36,6 +37,8 @@ using namespace std;
 
 namespace avg {
 
+typedef std::vector<std::vector<glm::vec2> > CollVec2Vector;
+
 ArgList::ArgList()
 {
 }
@@ -52,18 +55,18 @@ ArgList::ArgList(const ArgList& argTemplates, const xmlNodePtr xmlNode)
     }
 }
 
-ArgList::ArgList(const ArgList& argTemplates, const boost::python::dict& PyDict)
+ArgList::ArgList(const ArgList& argTemplates, const py::dict& PyDict)
 {
     // TODO: Check if all required args are being set.
     copyArgsFrom(argTemplates);
-    boost::python::list keys = PyDict.keys();
-    int nKeys = boost::python::len(keys);
+    py::list keys = PyDict.keys();
+    int nKeys = py::len(keys);
     for (int i = 0; i < nKeys; i++)
     {
-        boost::python::object keyObj = keys[i];
-        boost::python::object valObj = PyDict[keyObj];
+        py::object keyObj = keys[i];
+        py::object valObj = PyDict[keyObj];
         
-        boost::python::extract<string> keyStrProxy(keyObj);
+        py::extract<string> keyStrProxy(keyObj);
         if (!keyStrProxy.check()) {
             throw Exception(AVG_ERR_INVALID_ARGS, "Argument name must be a string.");
         }
@@ -93,7 +96,7 @@ const ArgBasePtr ArgList::getArg(const string& sName) const
     return valIt->second;
 }
 
-void ArgList::getOverlayedArgVal(DPoint* pResult, const string& sName, 
+void ArgList::getOverlayedArgVal(glm::vec2* pResult, const string& sName, 
         const string& sOverlay1, const string& sOverlay2, const string& sID) const
 {
     if (hasArg(sName)) {
@@ -102,7 +105,7 @@ void ArgList::getOverlayedArgVal(DPoint* pResult, const string& sName,
                     string("Duplicate node arguments (")+sName+" and "+
                     sOverlay1+","+sOverlay2+") for node '"+sID+"'"));
         }
-        *pResult = getArgVal<DPoint>(sName);
+        *pResult = getArgVal<glm::vec2>(sName);
     }
 }
 
@@ -113,7 +116,7 @@ const ArgMap& ArgList::getArgMap() const
 
 void ArgList::setArg(const ArgBase& newArg)
 {
-    m_Args.insert(ArgMap::value_type(newArg.getName(), ArgBasePtr(newArg.createCopy())));
+    m_Args[newArg.getName()] =  ArgBasePtr(newArg.createCopy());
 }
 
 void ArgList::setArgs(const ArgList& args)
@@ -123,20 +126,19 @@ void ArgList::setArgs(const ArgList& args)
     }
 }
     
-void ArgList::setMembers(Node * pNode) const
+void ArgList::setMembers(ExportedObject * pObj) const
 {
     for (ArgMap::const_iterator it = m_Args.begin(); it != m_Args.end(); it++) {
         const ArgBasePtr pCurArg = it->second;
-        pCurArg->setMember(pNode);
+        pCurArg->setMember(pObj);
     }
-    pNode->setArgs(*this);
+    pObj->setArgs(*this);
 }
 
 template<class T>
-void setArgValue(Arg<T>* pArg, const std::string & sName, 
-        const boost::python::object& value)
+void setArgValue(Arg<T>* pArg, const std::string & sName, const py::object& value)
 {
-    boost::python::extract<T> valProxy(value);
+    py::extract<T> valProxy(value);
     if (!valProxy.check()) {
         string sTypeName = getFriendlyTypeName(pArg->getValue());
         throw Exception(AVG_ERR_INVALID_ARGS, "Type error in argument "+sName+": "
@@ -145,47 +147,57 @@ void setArgValue(Arg<T>* pArg, const std::string & sName,
     pArg->setValue(valProxy());
 }
 
-void ArgList::setArgValue(const std::string & sName, const boost::python::object& value)
+void ArgList::setArgValue(const std::string & sName, const py::object& value)
 {
     ArgBasePtr pArg = getArg(sName);
     Arg<string>* pStringArg = dynamic_cast<Arg<string>* >(&*pArg);
     Arg<UTF8String>* pUTF8StringArg = dynamic_cast<Arg<UTF8String>* >(&*pArg);
     Arg<int>* pIntArg = dynamic_cast<Arg<int>* >(&*pArg);
-    Arg<double>* pDoubleArg = dynamic_cast<Arg<double>* >(&*pArg);
     Arg<float>* pFloatArg = dynamic_cast<Arg<float>* >(&*pArg);
     Arg<bool>* pBoolArg = dynamic_cast<Arg<bool>* >(&*pArg);
-    Arg<DPoint>* pDPointArg = dynamic_cast<Arg<DPoint>* >(&*pArg);
-    Arg<IntTriple>* pIntTripleArg = dynamic_cast<Arg<IntTriple>* >(&*pArg);
-    Arg<DTriple>* pDTripleArg = dynamic_cast<Arg<DTriple>* >(&*pArg);
-    Arg<vector<double> >* pDVectorArg = dynamic_cast<Arg<vector<double> >* >(&*pArg);
-    Arg<vector<DPoint> >* pDPointVectorArg = 
-            dynamic_cast<Arg<vector<DPoint> >* >(&*pArg);
-    Arg<vector<IntTriple> >* pIntTripleVectorArg = 
-            dynamic_cast<Arg<vector<IntTriple> >* >(&*pArg);
+    Arg<glm::vec2>* pVec2Arg = dynamic_cast<Arg<glm::vec2>* >(&*pArg);
+    Arg<glm::vec3>* pVec3Arg = dynamic_cast<Arg<glm::vec3>* >(&*pArg);
+    Arg<glm::ivec3>* pIVec3Arg = dynamic_cast<Arg<glm::ivec3>* >(&*pArg);
+    Arg<vector<float> >* pFVectorArg = dynamic_cast<Arg<vector<float> >* >(&*pArg);
+    Arg<vector<int> >* pIVectorArg = dynamic_cast<Arg<vector<int> >* >(&*pArg);
+    Arg<vector<glm::vec2> >* pVec2VectorArg = 
+            dynamic_cast<Arg<vector<glm::vec2> >* >(&*pArg);
+    Arg<vector<glm::ivec3> >* pIVec3VectorArg = 
+            dynamic_cast<Arg<vector<glm::ivec3> >* >(&*pArg);
+    Arg<CollVec2Vector>* pCollVec2VectorArg =
+            dynamic_cast<Arg<CollVec2Vector>* >(&*pArg);
+    Arg<FontStyle>* pFontStyleArg = dynamic_cast<Arg<FontStyle>* >(&*pArg);
+    Arg<FontStylePtr>* pFontStylePtrArg = dynamic_cast<Arg<FontStylePtr>* >(&*pArg);
     if(pStringArg) {
         avg::setArgValue(pStringArg, sName, value);
     } else if (pUTF8StringArg) {
         avg::setArgValue(pUTF8StringArg, sName, value);
     } else if (pIntArg) {
         avg::setArgValue(pIntArg, sName, value);
-    } else if (pDoubleArg) {
-        avg::setArgValue(pDoubleArg, sName, value);
     } else if (pFloatArg) {
         avg::setArgValue(pFloatArg, sName, value);
     } else if (pBoolArg) {
         avg::setArgValue(pBoolArg, sName, value);
-    } else if (pDPointArg) {
-        avg::setArgValue(pDPointArg, sName, value);
-    } else if (pDVectorArg) {
-        avg::setArgValue(pDVectorArg, sName, value);
-    } else if (pDPointVectorArg) {
-        avg::setArgValue(pDPointVectorArg, sName, value);
-    } else if (pIntTripleArg) {
-        avg::setArgValue(pIntTripleArg, sName, value);
-    } else if (pDTripleArg) {
-        avg::setArgValue(pDTripleArg, sName, value);
-    } else if (pIntTripleVectorArg) {
-        avg::setArgValue(pIntTripleVectorArg, sName, value);
+    } else if (pVec2Arg) {
+        avg::setArgValue(pVec2Arg, sName, value);
+    } else if (pVec3Arg) {
+        avg::setArgValue(pVec3Arg, sName, value);
+    } else if (pIVec3Arg) {
+        avg::setArgValue(pIVec3Arg, sName, value);
+    } else if (pFVectorArg) {
+        avg::setArgValue(pFVectorArg, sName, value);
+    } else if (pIVectorArg) {
+        avg::setArgValue(pIVectorArg, sName, value);
+    } else if (pVec2VectorArg) {
+        avg::setArgValue(pVec2VectorArg, sName, value);
+    } else if (pIVec3VectorArg) {
+        avg::setArgValue(pIVec3VectorArg, sName, value);
+    } else if (pCollVec2VectorArg) {
+        avg::setArgValue(pCollVec2VectorArg, sName, value);
+    } else if (pFontStyleArg) {
+        avg::setArgValue(pFontStyleArg, sName, value);
+    } else if (pFontStylePtrArg) {
+        avg::setArgValue(pFontStylePtrArg, sName, value);
     } else {
         AVG_ASSERT(false);
     }
@@ -197,45 +209,55 @@ void ArgList::setArgValue(const std::string & sName, const std::string & sValue)
     Arg<string>* pStringArg = dynamic_cast<Arg<string>* >(&*pArg);
     Arg<UTF8String>* pUTF8StringArg = dynamic_cast<Arg<UTF8String>* >(&*pArg);
     Arg<int>* pIntArg = dynamic_cast<Arg<int>* >(&*pArg);
-    Arg<double>* pDoubleArg = dynamic_cast<Arg<double>* >(&*pArg);
     Arg<float>* pFloatArg = dynamic_cast<Arg<float>* >(&*pArg);
     Arg<bool>* pBoolArg = dynamic_cast<Arg<bool>* >(&*pArg);
-    Arg<DPoint>* pDPointArg = dynamic_cast<Arg<DPoint>* >(&*pArg);
-    Arg<IntTriple>* pIntTripleArg = dynamic_cast<Arg<IntTriple>* >(&*pArg);
-    Arg<vector<double> >* pDVectorArg = dynamic_cast<Arg<vector<double> >* >(&*pArg);
-    Arg<vector<DPoint> >* pDPointVectorArg = 
-            dynamic_cast<Arg<vector<DPoint> >* >(&*pArg);
-    Arg<vector<IntTriple> >* pIntTripleVectorArg = 
-            dynamic_cast<Arg<vector<IntTriple> >* >(&*pArg);
-
+    Arg<glm::vec2>* pVec2Arg = dynamic_cast<Arg<glm::vec2>* >(&*pArg);
+    Arg<glm::vec3>* pVec3Arg = dynamic_cast<Arg<glm::vec3>* >(&*pArg);
+    Arg<glm::ivec3>* pIVec3Arg = dynamic_cast<Arg<glm::ivec3>* >(&*pArg);
+    Arg<vector<float> >* pFVectorArg = dynamic_cast<Arg<vector<float> >* >(&*pArg);
+    Arg<vector<int> >* pIVectorArg = dynamic_cast<Arg<vector<int> >* >(&*pArg);
+    Arg<vector<glm::vec2> >* pVec2VectorArg = 
+            dynamic_cast<Arg<vector<glm::vec2> >* >(&*pArg);
+    Arg<vector<glm::ivec3> >* pIVec3VectorArg = 
+            dynamic_cast<Arg<vector<glm::ivec3> >* >(&*pArg);
+    Arg<CollVec2Vector>* pCollVec2VectorArg =
+            dynamic_cast<Arg<CollVec2Vector>* >(&*pArg);
     if (pStringArg) {
         pStringArg->setValue(sValue);
     } else if (pUTF8StringArg) {
         pUTF8StringArg->setValue(sValue);
     } else if (pIntArg) {
         pIntArg->setValue(stringToInt(sValue));
-    } else if (pDoubleArg) {
-        pDoubleArg->setValue(stringToDouble(sValue));
     } else if (pFloatArg) {
-        pFloatArg->setValue(float(stringToDouble(sValue)));
+        pFloatArg->setValue(stringToFloat(sValue));
     } else if (pBoolArg) {
         pBoolArg->setValue(stringToBool(sValue));
-    } else if (pDPointArg) {
-        pDPointArg->setValue(stringToDPoint(sValue));
-    } else if (pIntTripleArg) {
-        pIntTripleArg->setValue(stringToIntTriple(sValue));
-    } else if (pDVectorArg) {
-        vector<double> v;
+    } else if (pVec2Arg) {
+        pVec2Arg->setValue(stringToVec2(sValue));
+    } else if (pVec3Arg) {
+        pVec3Arg->setValue(stringToVec3(sValue));
+    } else if (pIVec3Arg) {
+        pIVec3Arg->setValue(stringToIVec3(sValue));
+    } else if (pFVectorArg) {
+        vector<float> v;
         fromString(sValue, v);
-        pDVectorArg->setValue(v);
-    } else if (pDPointVectorArg) {
-        vector<DPoint> v;
+        pFVectorArg->setValue(v);
+    } else if (pIVectorArg) {
+        vector<int> v;
         fromString(sValue, v);
-        pDPointVectorArg->setValue(v);
-    } else if (pIntTripleVectorArg) {
-        vector<IntTriple> v;
+        pIVectorArg->setValue(v);
+    } else if (pVec2VectorArg) {
+        vector<glm::vec2> v;
         fromString(sValue, v);
-        pIntTripleVectorArg->setValue(v);
+        pVec2VectorArg->setValue(v);
+    } else if (pIVec3VectorArg) {
+        vector<glm::ivec3> v;
+        fromString(sValue, v);
+        pIVec3VectorArg->setValue(v);
+    } else if (pCollVec2VectorArg) {
+        CollVec2Vector v;
+        fromString(sValue, v);
+        pCollVec2VectorArg->setValue(v);
     } else {
         AVG_ASSERT(false);
     }   
