@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,6 @@
 
 #include "../base/ObjectCounter.h"
 #include "../base/Exception.h"
-#include "../graphics/ShaderRegistry.h"
 
 #include <string>
 
@@ -31,9 +30,9 @@ using namespace std;
 
 namespace avg {
 
-BlurFXNode::BlurFXNode() 
-    : FXNode(),
-      m_StdDev(1)
+BlurFXNode::BlurFXNode(float radius) 
+    : FXNode(false),
+      m_StdDev(radius)
 {
     ObjectCounter::get()->incRef(&typeid(*this));
 }
@@ -45,10 +44,6 @@ BlurFXNode::~BlurFXNode()
 
 void BlurFXNode::connect()
 {
-    if (!GLTexture::isFloatFormatSupported()) {
-        throw Exception(AVG_ERR_UNSUPPORTED, 
-                "Cannot create BlurFX: OpenGL configuration doesn't support Blur (no float textures).");
-    }
     setDirty();
     FXNode::connect();
 }
@@ -59,12 +54,7 @@ void BlurFXNode::disconnect()
     FXNode::disconnect();
 }
 
-void BlurFXNode::setParam(double stdDev)
-{
-    setRadius(stdDev);
-}
-
-void BlurFXNode::setRadius(double stdDev)
+void BlurFXNode::setRadius(float stdDev)
 {
     m_StdDev = stdDev;
     if (m_pFilter) {
@@ -73,7 +63,7 @@ void BlurFXNode::setRadius(double stdDev)
     setDirty();
 }
 
-double BlurFXNode::getRadius() const
+float BlurFXNode::getRadius() const
 {
     return m_StdDev;
 }
