@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -33,22 +33,21 @@ namespace avg {
 
 class AVG_API BlurFXNode: public FXNode {
 public:
-    BlurFXNode();
+    BlurFXNode(float radius=1.f);
     virtual ~BlurFXNode();
 
     void connect();
     virtual void disconnect();
-    void setParam(double stdDev);
 
-    void setRadius(double stdDev);
-    double getRadius() const;
+    void setRadius(float stdDev);
+    float getRadius() const;
 
 private:
     virtual GPUFilterPtr createFilter(const IntPoint& size);
 
     GPUBlurFilterPtr m_pFilter;
 
-    double m_StdDev;
+    float m_StdDev;
 };
 
 typedef boost::shared_ptr<BlurFXNode> BlurFXNodePtr;
