@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,12 +32,13 @@ namespace avg {
 class AVG_API CanvasNode : public DivNode
 {
     public:
-        static NodeDefinition createDefinition();
+        static void registerType();
         
         CanvasNode(const ArgList& args);
         virtual ~CanvasNode();
 
         virtual std::string getEffectiveMediaDir();
+
 };
 
 typedef boost::shared_ptr<CanvasNode> CanvasNodePtr;
