@@ -1,6 +1,6 @@
 
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 
 #include "../api.h"
 
-#include "../base/Point.h"
+#include "../base/GLMHelper.h"
 
 namespace avg {
 
@@ -41,11 +41,10 @@ struct AVG_API DisplayParams {
     IntPoint m_WindowSize;
     bool m_bShowCursor;
     int m_VBRate;
-    double m_Framerate;
+    float m_Framerate;
     bool m_bHasWindowFrame;
-    double m_DotsPerMM;
 
-    double m_Gamma[3];
+    float m_Gamma[3];
 };
 
 }
