@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,19 +25,18 @@
 #include "../api.h"
 #include "AreaNode.h"
 
+#include "../graphics/SubVertexArray.h"
+
 #include "../base/UTF8String.h"
-#include "../graphics/VertexArray.h"
 
 #include <string>
 
 namespace avg {
 
-class VertexArray;
-
 class AVG_API DivNode : public AreaNode
 {
     public:
-        static NodeDefinition createDefinition();
+        static void registerType();
         
         DivNode(const ArgList& args);
         virtual ~DivNode();
@@ -59,20 +58,18 @@ class AVG_API DivNode : public AreaNode
         void removeChild(NodePtr pNode, bool bKill);
         void removeChild(unsigned i, bool bKill);
 
-        virtual DPoint getPivot() const;
+        virtual glm::vec2 getPivot() const;
 
         bool getCrop() const;
         void setCrop(bool bCrop);
 
-        const std::string& getElementOutlineColor() const;
-        void setElementOutlineColor(const std::string& sColor);
-
         const UTF8String& getMediaDir() const;
         void setMediaDir(const UTF8String& mediaDir);
 
-        void getElementsByPos(const DPoint& pos, std::vector<NodeWeakPtr>& pElements);
-        virtual void preRender();
-        virtual void render(const DRect& rect);
+        void getElementsByPos(const glm::vec2& pos, std::vector<NodePtr>& pElements);
+        virtual void preRender(const VertexArrayPtr& pVA, bool bIsParentActive, 
+                float parentEffectiveOpacity);
+        virtual void render();
         virtual void renderOutlines(const VertexArrayPtr& pVA, Pixel32 color);
 
         virtual std::string getEffectiveMediaDir();
@@ -86,10 +83,8 @@ class AVG_API DivNode : public AreaNode
 
         UTF8String m_sMediaDir;
         bool m_bCrop;
-        std::string m_sElementOutlineColor;
-        Pixel32 m_ElementOutlineColor;
 
-        VertexArrayPtr m_pClipVertexes;
+        SubVertexArray m_ClipVA;
 
         std::vector<NodePtr> m_Children;
 };
