@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,7 +26,7 @@
 
 #include "MaterialInfo.h"
 
-#include "../base/Point.h"
+#include "../base/GLMHelper.h"
 #include "../graphics/Bitmap.h"
 
 #include <boost/shared_ptr.hpp>
@@ -76,7 +76,6 @@ class AVG_API Image
 
     private:
         void setupSurface();
-        PixelFormat calcSurfacePF(const Bitmap& Bmp);
         bool changeSource(Source newSource);
         void assertValid() const;
 
