@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -41,7 +41,7 @@ class AVG_API MainCanvas: public Canvas
         virtual BitmapPtr screenshot() const;
 
     private:
-        virtual void render();
+        virtual void renderTree();
 
         SDLDisplayEnginePtr m_pDisplayEngine;
 };
