@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -41,10 +41,10 @@ class AVG_API OffscreenCanvas: public Canvas
         virtual ~OffscreenCanvas();
         virtual void setRoot(NodePtr pRootNode);
         virtual void initPlayback();
-        virtual void stopPlayback();
+        virtual void stopPlayback(bool bIsAbort);
 
         virtual BitmapPtr screenshot() const;
-        virtual BitmapPtr screenshot(bool bIgnoreAlpha) const;
+        virtual BitmapPtr screenshotIgnoreAlpha() const;
         bool getHandleEvents() const;
         int getMultiSampleSamples() const;
         bool getMipmap() const;
@@ -65,7 +65,7 @@ class AVG_API OffscreenCanvas: public Canvas
 
         void addDependentCanvas(CanvasPtr pCanvas);
         void removeDependentCanvas(CanvasPtr pCanvas);
-        bool hasDependentCanvas(CanvasPtr pCanvas) const;
+        const std::vector<CanvasPtr>& getDependentCanvases() const;
         unsigned getNumDependentCanvases() const;
 
         static bool isSupported();
@@ -73,7 +73,7 @@ class AVG_API OffscreenCanvas: public Canvas
         void dump() const;
  
     protected:
-        virtual void render();
+        virtual void renderTree();
 
     private:
         FBOPtr m_pFBO;
@@ -82,7 +82,6 @@ class AVG_API OffscreenCanvas: public Canvas
 
         bool m_bIsRendered;
         CameraNode* m_pCameraNodeRef;
-        int m_cameraFrameRate;
 };
 
 typedef boost::shared_ptr<OffscreenCanvas> OffscreenCanvasPtr;
