@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,7 @@
 #include "OffscreenCanvasNode.h"
 #include "Player.h"
 
-#include "NodeDefinition.h"
+#include "TypeDefinition.h"
 
 #include "../base/FileHelper.h"
 
@@ -30,10 +30,10 @@ using namespace std;
 
 namespace avg {
 
-NodeDefinition OffscreenCanvasNode::createDefinition()
+void OffscreenCanvasNode::registerType()
 {
-    return NodeDefinition("canvas", Node::buildNode<OffscreenCanvasNode>)
-        .extendDefinition(CanvasNode::createDefinition())
+    TypeDefinition def = TypeDefinition("canvas", "canvasbase",
+            ExportedObject::buildObject<OffscreenCanvasNode>)
         .addArg(Arg<bool>("handleevents", false, false, 
                 offsetof(OffscreenCanvasNode, m_bHandleEvents)))
         .addArg(Arg<int>("multisamplesamples", 1, false, 
@@ -42,6 +42,7 @@ NodeDefinition OffscreenCanvasNode::createDefinition()
                 offsetof(OffscreenCanvasNode, m_bMipmap)))
         .addArg(Arg<bool>("autorender", true, false,
                 offsetof(OffscreenCanvasNode, m_bAutoRender)));
+    TypeRegistry::get()->registerType(def);
 }
 
 OffscreenCanvasNode::OffscreenCanvasNode(const ArgList& args)
