@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,6 @@
 
 #include "../base/ObjectCounter.h"
 #include "../base/Exception.h"
-#include "../graphics/ShaderRegistry.h"
 
 #include <string>
 
@@ -31,18 +30,15 @@ using namespace std;
 
 namespace avg {
 
-ShadowFXNode::ShadowFXNode() 
-    : FXNode(),
-      m_Offset(0,0),
-      m_StdDev(1),
-      m_Opacity(1),
-      m_Color(255,255,255,255)
+ShadowFXNode::ShadowFXNode(glm::vec2 offset, float radius, float opacity, string sColor) 
+    : FXNode(false),
+      m_Offset(offset),
+      m_StdDev(radius),
+      m_Opacity(opacity)
 {
+    m_sColorName = sColor;
+    m_Color = colorStringToColor(sColor);
     ObjectCounter::get()->incRef(&typeid(*this));
-    if (!GLTexture::isFloatFormatSupported()) {
-        throw Exception(AVG_ERR_UNSUPPORTED, 
-                "OpenGL configuration doesn't support Shadow (no float textures).");
-    }
 }
 
 ShadowFXNode::~ShadowFXNode()
@@ -52,10 +48,6 @@ ShadowFXNode::~ShadowFXNode()
 
 void ShadowFXNode::connect()
 {
-    if (!GLTexture::isFloatFormatSupported()) {
-        throw Exception(AVG_ERR_UNSUPPORTED, 
-                "Cannot create ShadowFX: OpenGL configuration doesn't support Blur (no float textures).");
-    }
     FXNode::connect();
 }
 
@@ -65,46 +57,35 @@ void ShadowFXNode::disconnect()
     FXNode::disconnect();
 }
 
-void ShadowFXNode::setParams(const DPoint& offset, double stdDev, double opacity, 
-        const string& sColor)
-{
-    m_Offset = offset;
-    m_StdDev = stdDev;
-    m_Opacity = opacity;
-    m_sColorName = sColor;
-    m_Color = colorStringToColor(sColor);
-    updateFilter();
-}
-    
-void ShadowFXNode::setOffset(const DPoint& offset)
+void ShadowFXNode::setOffset(const glm::vec2& offset)
 {
     m_Offset = offset;
     updateFilter();
 }
 
-DPoint ShadowFXNode::getOffset() const
+glm::vec2 ShadowFXNode::getOffset() const
 {
     return m_Offset;
 }
 
-void ShadowFXNode::setRadius(double radius)
+void ShadowFXNode::setRadius(float radius)
 {
     m_StdDev = radius;
     updateFilter();
 }
 
-double ShadowFXNode::getRadius() const
+float ShadowFXNode::getRadius() const
 {
     return m_StdDev;
 }
 
-void ShadowFXNode::setOpacity(double opacity)
+void ShadowFXNode::setOpacity(float opacity)
 {
     m_Opacity = opacity;
     updateFilter();
 }
 
-double ShadowFXNode::getOpacity() const
+float ShadowFXNode::getOpacity() const
 {
     return m_Opacity;
 }
