@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@
 #define _TestHelper_H_
 
 #include "../api.h"
+#include "../base/ObjectCounter.h"
 #include "../graphics/Bitmap.h"
 #include "Event.h"
 #include "IInputDevice.h"
@@ -51,11 +52,12 @@ class AVG_API TestHelper : public IInputDevice
                 bool rightButtonState,
                 int xPosition, int yPosition, int button);
         void fakeTouchEvent(int id, Event::Type eventType, Event::Source source,
-                const DPoint& pos, const DPoint& speed=DPoint(0, 0));
+                const glm::vec2& pos, const glm::vec2& speed=glm::vec2(0, 0));
         void fakeKeyEvent(Event::Type eventType,
                 unsigned char scanCode, int keyCode, 
                 const std::string& keyString, int unicode, int modifiers);
         void dumpObjects();
+        TypeMap getObjectCount();
 
         // From IInputDevice
         virtual std::vector<EventPtr> pollEvents();
