@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,11 +29,10 @@
 #include "Node.h"
 #include "RasterNode.h"
 
-#include "../base/Point.h"
+#include "../base/GLMHelper.h"
 #include "../base/IFrameEndListener.h"
 #include "../base/UTF8String.h"
 
-#include "../audio/IAudioSource.h"
 #include "../video/VideoDecoder.h"
 
 namespace avg {
@@ -42,12 +41,12 @@ class VideoDecoder;
 class TextureMover;
 typedef boost::shared_ptr<TextureMover> TextureMoverPtr;
 
-class AVG_API VideoNode: public RasterNode, IFrameEndListener, IAudioSource
+class AVG_API VideoNode: public RasterNode, IFrameEndListener
 {
     public:
         enum VideoAccelType {NONE, VDPAU};
 
-        static NodeDefinition createDefinition();
+        static void registerType();
         
         VideoNode(const ArgList& args);
         virtual ~VideoNode();
@@ -62,9 +61,9 @@ class AVG_API VideoNode: public RasterNode, IFrameEndListener, IAudioSource
 
         const UTF8String& getHRef() const;
         void setHRef(const UTF8String& href);
-        double getVolume();
-        void setVolume(double volume);
-        double getFPS() const;
+        float getVolume();
+        void setVolume(float volume);
+        float getFPS() const;
         int getQueueLength() const;
         void checkReload();
 
@@ -74,7 +73,10 @@ class AVG_API VideoNode: public RasterNode, IFrameEndListener, IAudioSource
         void seekToFrame(int frameNum);
         std::string getStreamPixelFormat() const;
         long long getDuration() const;
+        long long getVideoDuration() const;
+        long long getAudioDuration() const;
         int getBitrate() const;
+        std::string getContainerFormat() const;
         std::string getVideoCodec() const;
         std::string getAudioCodec() const;
         int getAudioSampleRate() const;
@@ -89,18 +91,17 @@ class AVG_API VideoNode: public RasterNode, IFrameEndListener, IAudioSource
         void setEOFCallback(PyObject * pEOFCallback);
         bool isAccelerated() const;
 
-        virtual void render(const DRect& rect);
-        virtual void preRender();
+        virtual void preRender(const VertexArrayPtr& pVA, bool bIsParentActive, 
+                float parentEffectiveOpacity);
+        virtual void render();
         virtual void onFrameEnd();
         
-        virtual int fillAudioBuffer(AudioBufferPtr pBuffer);
         virtual IntPoint getMediaSize();
 
         static VideoAccelType getVideoAccelConfig();
 
     private:
         bool renderFrame();
-        FrameAvailableCode renderToSurface();
         void seek(long long destTime);
         void onEOF();
         void updateStatusDueToDecoderEOF();
@@ -126,7 +127,7 @@ class AVG_API VideoNode: public RasterNode, IFrameEndListener, IAudioSource
         std::string m_Filename;
         bool m_bLoop;
         bool m_bThreaded;
-        double m_FPS;
+        float m_FPS;
         int m_QueueLength;
         bool m_bEOFPending;
         PyObject * m_pEOFCallback;
@@ -139,11 +140,13 @@ class AVG_API VideoNode: public RasterNode, IFrameEndListener, IAudioSource
         long long m_StartTime;
         long long m_PauseTime;
         long long m_PauseStartTime;
-        double m_JitterCompensation;
+        float m_JitterCompensation;
 
         VideoDecoder * m_pDecoder;
-        double m_Volume;
+        float m_Volume;
         bool m_bUsesHardwareAcceleration;
+        bool m_bEnableSound;
+        int m_AudioID;
 
         GLTexturePtr m_pTextures[4];
 };
