@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
 
 #include "WordsNode.h"
 #include "OGLSurface.h"
-#include "NodeDefinition.h"
+#include "TypeDefinition.h"
 #include "TextEngine.h"
 
 #include "../base/Logger.h"
@@ -46,7 +46,7 @@ using namespace std;
 
 namespace avg {
 
-NodeDefinition WordsNode::createDefinition()
+void WordsNode::registerType()
 {
     static const string sDTDElements = 
         "<!ELEMENT span (#PCDATA|span|b|big|i|s|sub|sup|small|tt|u)*>\n"
@@ -84,42 +84,50 @@ NodeDefinition WordsNode::createDefinition()
             "small", "tt", "u", "br"};
     vector<string> sChildren = vectorFromCArray(sizeof(sChildArray)/sizeof(*sChildArray),
             sChildArray); 
-    return NodeDefinition("words", Node::buildNode<WordsNode>)
-        .extendDefinition(RasterNode::createDefinition())
+    TypeDefinition def = TypeDefinition("words", "rasternode", 
+            ExportedObject::buildObject<WordsNode>)
         .addChildren(sChildren)
         .addDTDElements(sDTDElements)
-        .addArg(Arg<string>("font", "arial", false, offsetof(WordsNode, m_sFontName)))
-        .addArg(Arg<string>("variant", "", false, offsetof(WordsNode, m_sFontVariant)))
+        .addArg(Arg<string>("font", "sans"))
+        .addArg(Arg<string>("variant", ""))
         .addArg(Arg<UTF8String>("text", ""))
-        .addArg(Arg<string>("color", "FFFFFF", false, offsetof(WordsNode, m_sColorName)))
-        .addArg(Arg<double>("fontsize", 15, false, offsetof(WordsNode, m_FontSize)))
-        .addArg(Arg<int>("indent", 0, false, offsetof(WordsNode, m_Indent)))
-        .addArg(Arg<double>("linespacing", -1, false, offsetof(WordsNode, m_LineSpacing)))
+        .addArg(Arg<string>("color", "FFFFFF"))
+        .addArg(Arg<float>("aagamma", 1.0f))
+        .addArg(Arg<float>("fontsize", 15))
+        .addArg(Arg<int>("indent", 0, false))
+        .addArg(Arg<float>("linespacing", 0))
         .addArg(Arg<string>("alignment", "left"))
         .addArg(Arg<string>("wrapmode", "word"))
-        .addArg(Arg<bool>("justify", false, false, offsetof(WordsNode, m_bJustify)))
+        .addArg(Arg<bool>("justify", false))
         .addArg(Arg<bool>("rawtextmode", false, false, 
                 offsetof(WordsNode, m_bRawTextMode)))
-        .addArg(Arg<double>("letterspacing", 0, false, 
-                offsetof(WordsNode, m_LetterSpacing)))
-        .addArg(Arg<bool>("hint", true, false, offsetof(WordsNode, m_bHint)))
+        .addArg(Arg<float>("letterspacing", 0))
+        .addArg(Arg<bool>("hint", true))
+        .addArg(Arg<FontStyle>("fontstyle", FontStyle()))
         ;
+    TypeRegistry::get()->registerType(def);
 }
 
 WordsNode::WordsNode(const ArgList& args)
     : m_LogicalSize(0,0),
       m_pFontDescription(0),
       m_pLayout(0),
-      m_RedrawState(FONT_CHANGED)
+      m_bRenderNeeded(true)
 {
     m_bParsedText = false;
-
     args.setMembers(this);
-    setAlignment(args.getArgVal<string>("alignment"));
-    setWrapMode(args.getArgVal<string>("wrapmode"));
+
+    m_FontStyle = args.getArgVal<FontStyle>("fontstyle");
+    m_FontStyle.setDefaultedArgs(args);
+#ifdef _WIN32
+    if (m_FontStyle.getFont() == "sans") {
+        m_FontStyle.setFont("Arial");
+        m_FontStyle.setFontVariant("Regular");
+    }
+#endif
+    updateFont();
     setText(args.getArgVal<UTF8String>("text"));
-    m_Color = colorStringToColor(m_sColorName);
-    setViewport(-32767, -32767, -32767, -32767);
+    
     ObjectCounter::get()->incRef(&typeid(*this));
 }
 
@@ -146,7 +154,7 @@ void WordsNode::setTextFromNodeValue(const string& sText)
 void WordsNode::connectDisplay()
 {
     RasterNode::connectDisplay();
-    setDirty(FONT_CHANGED);
+    getSurface()->setAlphaGamma(m_FontStyle.getAAGamma());
 }
 
 void WordsNode::connect(CanvasPtr pCanvas)
@@ -160,132 +168,126 @@ void WordsNode::disconnect(bool bKill)
     if (m_pFontDescription) {
         pango_font_description_free(m_pFontDescription);
         m_pFontDescription = 0;
-        setDirty(FONT_CHANGED);
+        updateFont();
     }
     RasterNode::disconnect(bKill);
 }
 
 string WordsNode::getAlignment() const
 {
-    switch(m_Alignment) {
-        case PANGO_ALIGN_LEFT:
-            return "left";
-        case PANGO_ALIGN_CENTER:
-            return "center";
-        case PANGO_ALIGN_RIGHT:
-            return "right";
-        default:
-            AVG_ASSERT(false);
-            return "";
-    }
+    return m_FontStyle.getAlignment();
 }
 
 void WordsNode::setAlignment(const string& sAlign)
 {
-    if (sAlign == "left") {
-        m_Alignment = PANGO_ALIGN_LEFT;
-    } else if (sAlign == "center") {
-        m_Alignment = PANGO_ALIGN_CENTER;
-    } else if (sAlign == "right") {
-        m_Alignment = PANGO_ALIGN_RIGHT;
-    } else {
-        throw(Exception(AVG_ERR_UNSUPPORTED, 
-                "WordsNode alignment "+sAlign+" not supported."));
-    }
-
-    setDirty(LAYOUT_CHANGED);
+    m_FontStyle.setAlignment(sAlign);
+    updateLayout();
 }
 
 bool WordsNode::getJustify() const
 {
-    return m_bJustify;
+    return m_FontStyle.getJustify();
 }
 
 void WordsNode::setJustify(bool bJustify)
 {
-    m_bJustify = bJustify;
-    setDirty(LAYOUT_CHANGED);
+    m_FontStyle.setJustify(bJustify);
+    updateLayout();
 }
 
-double WordsNode::getLetterSpacing() const
+float WordsNode::getLetterSpacing() const
 {
-    return m_LetterSpacing;
+    return m_FontStyle.getLetterSpacing();
 }
 
-void WordsNode::setLetterSpacing(double letterSpacing)
+void WordsNode::setLetterSpacing(float letterSpacing)
 {
-    m_LetterSpacing = letterSpacing;
-    setDirty(LAYOUT_CHANGED);
+    m_FontStyle.setLetterSpacing(letterSpacing);
+    updateLayout();
 }
 
 bool WordsNode::getHint() const
 {
-    return m_bHint;
+    return m_FontStyle.getHint();
 }
 
 void WordsNode::setHint(bool bHint)
 {
-    setDirty(LAYOUT_CHANGED);
-    m_bHint = bHint;
+    m_FontStyle.setHint(bHint);
+    updateLayout();
 }
 
-double WordsNode::getWidth() const
+float WordsNode::getWidth() const
 {
-    const_cast<WordsNode*>(this)->updateLayout();
     return AreaNode::getWidth();
 }
 
-void WordsNode::setWidth(double width)
+void WordsNode::setWidth(float width)
 {
-    setDirty(LAYOUT_CHANGED);
     AreaNode::setWidth(width);
+    updateLayout();
 }
 
-double WordsNode::getHeight() const
+float WordsNode::getHeight() const
 {
-    const_cast<WordsNode*>(this)->updateLayout();
     return AreaNode::getHeight();
 }
 
-void WordsNode::setHeight(double width)
+void WordsNode::setHeight(float width)
 {
-    setDirty(LAYOUT_CHANGED);
     AreaNode::setHeight(width);
+    updateLayout();
 }
 
-DPoint WordsNode::getSize() const
+glm::vec2 WordsNode::getSize() const
 {
-    const_cast<WordsNode*>(this)->updateLayout();
     return AreaNode::getSize();
 }
 
-void WordsNode::setSize(const DPoint& pt)
+void WordsNode::setSize(const glm::vec2& pt)
 {
-    setDirty(LAYOUT_CHANGED);
     AreaNode::setSize(pt);
+    updateLayout();
 }
 
-void WordsNode::getElementsByPos(const DPoint& pos, vector<NodeWeakPtr>& pElements)
+glm::vec2 WordsNode::toLocal(const glm::vec2& globalPos) const
 {
-    updateLayout();
-    DPoint relPos = pos-DPoint(m_AlignOffset, 0);
-    AreaNode::getElementsByPos(relPos, pElements);
+    glm::vec2 localPos = globalPos - getRelViewport().tl - glm::vec2(m_AlignOffset, 0);
+    return getRotatedPivot(localPos, -getAngle(), getPivot());
+}
+
+glm::vec2 WordsNode::toGlobal(const glm::vec2& localPos) const
+{
+    glm::vec2 alignPos = localPos + glm::vec2(m_AlignOffset, 0);
+    glm::vec2 globalPos = getRotatedPivot(alignPos, getAngle(), getPivot());
+    return globalPos + getRelViewport().tl;
+}
+
+const FontStyle& WordsNode::getFontStyle() const
+{
+    return m_FontStyle;
+}
+
+void WordsNode::setFontStyle(const FontStyle& fontStyle)
+{
+    m_FontStyle = fontStyle;
+    updateFont();
 }
 
 const std::string& WordsNode::getFont() const
 {
-    return m_sFontName;
+    return m_FontStyle.getFont();
 }
 
 void WordsNode::setFont(const std::string& sName)
 {
-    m_sFontName = sName;
-    setDirty(FONT_CHANGED);
+    m_FontStyle.setFont(sName);
+    updateFont();
 }
 
 const std::string& WordsNode::getFontVariant() const
 {
-    return m_sFontVariant;
+    return m_FontStyle.getFontVariant();
 }
 
 void WordsNode::addFontDir(const std::string& sDir)
@@ -296,8 +298,8 @@ void WordsNode::addFontDir(const std::string& sDir)
 
 void WordsNode::setFontVariant(const std::string& sVariant)
 {
-    m_sFontVariant = sVariant;
-    setDirty(FONT_CHANGED);
+    m_FontStyle.setFontVariant(sVariant);
+    updateFont();
 }
 
 const UTF8String& WordsNode::getText() const 
@@ -317,59 +319,67 @@ void WordsNode::setText(const UTF8String& sText)
         m_sText = m_sRawText;
         if (m_bRawTextMode) {
             m_bParsedText = false;
+            updateLayout();
         } else {
             setParsedText(sText);
         }
-        setDirty(LAYOUT_CHANGED);
     }
 }
 
 const std::string& WordsNode::getColor() const
 {
-    return m_sColorName;
+    return m_FontStyle.getColor();
 }
 
 void WordsNode::setColor(const string& sColor)
 {
-    m_sColorName = sColor;
-    m_Color = colorStringToColor(m_sColorName);
-    setDirty(RENDER_NEEDED);
+    m_FontStyle.setColor(sColor);
 }
 
-double WordsNode::getFontSize() const
+float WordsNode::getAAGamma() const
 {
-    return m_FontSize;
+    return m_FontStyle.getAAGamma();
 }
 
-void WordsNode::setFontSize(double size)
+void WordsNode::setAAGamma(float gamma)
 {
-    if (size <= 1) {
-        throw Exception(AVG_ERR_INVALID_ARGS, "Words node: Font size < 1 is illegal.");
-    } 
-    m_FontSize = size;
-    setDirty(FONT_CHANGED);
+    m_FontStyle.setAAGamma(gamma);
+    if (getState() == Node::NS_CANRENDER) {
+        getSurface()->setAlphaGamma(gamma);
+    }
+}
+
+float WordsNode::getFontSize() const
+{
+    return m_FontStyle.getFontSize();
+}
+
+void WordsNode::setFontSize(float size)
+{
+    m_FontStyle.setFontSize(size);
+    updateFont();
 }
 
 int WordsNode::getIndent() const
 {
-    return m_Indent;
+    return m_FontStyle.getIndent();
 }
 
 void WordsNode::setIndent(int indent)
 {
-    m_Indent = indent;
-    setDirty(LAYOUT_CHANGED);
+    m_FontStyle.setIndent(indent);
+    updateLayout();
 }
 
-double WordsNode::getLineSpacing() const
+float WordsNode::getLineSpacing() const
 {
-    return m_LineSpacing;
+    return m_FontStyle.getLineSpacing();
 }
 
-void WordsNode::setLineSpacing(double lineSpacing)
+void WordsNode::setLineSpacing(float lineSpacing)
 {
-    m_LineSpacing = lineSpacing;
-    setDirty(LAYOUT_CHANGED);
+    m_FontStyle.setLineSpacing(lineSpacing);
+    updateLayout();
 }
 
 bool WordsNode::getRawTextMode() const
@@ -387,33 +397,34 @@ void WordsNode::setRawTextMode(bool rawTextMode)
             setParsedText(m_sText);
         }
         m_bRawTextMode = rawTextMode;
-        setDirty(LAYOUT_CHANGED);
+        updateLayout();
     }
 }
 
-DPoint WordsNode::getGlyphPos(int i)
+glm::vec2 WordsNode::getGlyphPos(int i)
 {
     PangoRectangle rect = getGlyphRect(i);
-    return DPoint(double(rect.x)/PANGO_SCALE, double(rect.y)/PANGO_SCALE);
+    return glm::vec2(float(rect.x)/PANGO_SCALE, float(rect.y)/PANGO_SCALE);
 }
 
-DPoint WordsNode::getGlyphSize(int i)
+glm::vec2 WordsNode::getGlyphSize(int i)
 {
     PangoRectangle rect = getGlyphRect(i);
-    return DPoint(double(rect.width)/PANGO_SCALE, double(rect.height)/PANGO_SCALE);
+    return glm::vec2(float(rect.width)/PANGO_SCALE, float(rect.height)/PANGO_SCALE);
 }
 
 int WordsNode::getNumLines()
 {
-    updateLayout();
-    return pango_layout_get_line_count(m_pLayout);
+    if(m_sText.length() != 0) {
+        return pango_layout_get_line_count(m_pLayout);
+    }
+    return 0;
 }
 
-PyObject* WordsNode::getCharIndexFromPos(DPoint p)
+PyObject* WordsNode::getCharIndexFromPos(glm::vec2 p)
 {
     int index;
     int trailing;
-    updateLayout();
     gboolean bXyToIndex = pango_layout_xy_to_index(m_pLayout,
                 int(p.x*PANGO_SCALE), int(p.y*PANGO_SCALE), &index, &trailing);
     if (bXyToIndex) {
@@ -426,52 +437,31 @@ PyObject* WordsNode::getCharIndexFromPos(DPoint p)
 
 std::string WordsNode::getTextAsDisplayed()
 {
-    updateLayout();
     return pango_layout_get_text(m_pLayout);
 }
 
-DPoint WordsNode::getLineExtents(int line)
+glm::vec2 WordsNode::getLineExtents(int line)
 {
-    if(line < 0 || line >= getNumLines()) {
+    if (line < 0 || line >= getNumLines()) {
         throw Exception(AVG_ERR_OUT_OF_RANGE, "WordsNode.getLineExtents: line index "
                 +toString(line)+" is out of range.");
     }
-    updateLayout();
     PangoRectangle logical_rect;
     PangoRectangle ink_rect;
     PangoLayoutLine *layoutLine = pango_layout_get_line_readonly(m_pLayout, line);
     pango_layout_line_get_pixel_extents(layoutLine, &ink_rect, &logical_rect);
-    return DPoint(double(logical_rect.width), double(logical_rect.height));
+    return glm::vec2(float(logical_rect.width), float(logical_rect.height));
 }
 
 void WordsNode::setWrapMode(const string& sWrapMode)
 {
-    if (sWrapMode == "word") {
-        m_WrapMode = PANGO_WRAP_WORD;
-    } else if (sWrapMode == "char") {
-        m_WrapMode = PANGO_WRAP_CHAR;
-    } else if (sWrapMode == "wordchar") {
-        m_WrapMode = PANGO_WRAP_WORD_CHAR;
-    } else {
-        throw(Exception(AVG_ERR_UNSUPPORTED, 
-                "WordsNode wrapping mode "+sWrapMode+" not supported."));
-    }
-    setDirty(LAYOUT_CHANGED);
+    m_FontStyle.setWrapMode(sWrapMode);
+    updateLayout();
 }
 
 string WordsNode::getWrapMode() const
 {
-    switch(m_WrapMode) {
-        case PANGO_WRAP_WORD:
-            return "word";
-        case PANGO_WRAP_CHAR:
-            return "char";
-        case PANGO_WRAP_WORD_CHAR:
-            return "wordchar";
-        default:
-            AVG_ASSERT(false);
-            return "";
-    }
+    return m_FontStyle.getWrapMode();
 }
 
 void WordsNode::parseString(PangoAttrList** ppAttrList, char** ppText)
@@ -497,31 +487,21 @@ void WordsNode::parseString(PangoAttrList** ppAttrList, char** ppText)
 
 void WordsNode::calcMaskCoords()
 {
-    updateLayout();
-
     // Calculate texture coordinates for the mask texture, normalized to
     // the extents of the text.
-    DPoint normMaskSize;
-    DPoint normMaskPos;
-    DPoint mediaSize = DPoint(getMediaSize());
-    DPoint effMaskPos = getMaskPos()-DPoint(m_InkOffset);
-    DPoint maskSize = getMaskSize();
-    switch (m_Alignment) {
-        case PANGO_ALIGN_LEFT:
-            break;
-        case PANGO_ALIGN_CENTER:
-            effMaskPos.x -= m_AlignOffset+getSize().x/2;
-            break;
-        case PANGO_ALIGN_RIGHT:
-            effMaskPos.x -= m_AlignOffset+getSize().x;
-            break;
-    }
-    if (maskSize == DPoint(0,0)) {
-        normMaskSize = DPoint(getSize().x/mediaSize.x, getSize().y/mediaSize.y);
-        normMaskPos = DPoint(effMaskPos.x/getSize().x, effMaskPos.y/getSize().y);
+    glm::vec2 normMaskSize;
+    glm::vec2 normMaskPos;
+    glm::vec2 mediaSize = glm::vec2(getMediaSize());
+    glm::vec2 effMaskPos = getMaskPos()-glm::vec2(m_InkOffset);
+    glm::vec2 maskSize = getMaskSize();
+    
+    if (maskSize == glm::vec2(0,0)) {
+        normMaskSize = glm::vec2(getSize().x/mediaSize.x, getSize().y/mediaSize.y);
+        normMaskPos = glm::vec2(effMaskPos.x/getSize().x, effMaskPos.y/getSize().y);
     } else {
-        normMaskSize = DPoint(maskSize.x/mediaSize.x, maskSize.y/mediaSize.y);
-        normMaskPos = DPoint(effMaskPos.x/getMaskSize().x, effMaskPos.y/getMaskSize().y);
+        normMaskSize = glm::vec2(maskSize.x/mediaSize.x, maskSize.y/mediaSize.y);
+        normMaskPos = glm::vec2(effMaskPos.x/getMaskSize().x, 
+                effMaskPos.y/getMaskSize().y);
     }
 /*    
     cerr << "calcMaskCoords" << endl;
@@ -535,126 +515,114 @@ void WordsNode::calcMaskCoords()
     getSurface()->setMaskCoords(normMaskPos, normMaskSize);
 }
 
-void WordsNode::setDirty(RedrawState newState)
-{
-    if (newState < m_RedrawState) {
-        m_RedrawState = newState;
-    }
-}
-
 static ProfilingZoneID UpdateFontProfilingZone("WordsNode: Update font");
 
 void WordsNode::updateFont()
 {
-    if (m_RedrawState == FONT_CHANGED) {
+    {
         ScopeTimer timer(UpdateFontProfilingZone);
 
         if (m_pFontDescription) {
             pango_font_description_free(m_pFontDescription);
         }
-        m_pFontDescription = TextEngine::get(m_bHint).getFontDescription(m_sFontName, 
-                m_sFontVariant);
+        TextEngine& engine = TextEngine::get(m_FontStyle.getHint());
+        m_pFontDescription = engine.getFontDescription(m_FontStyle.getFont(), 
+                m_FontStyle.getFontVariant());
         pango_font_description_set_absolute_size(m_pFontDescription,
-                (int)(m_FontSize * PANGO_SCALE));
-
-        m_RedrawState = LAYOUT_CHANGED;
+                (int)(m_FontStyle.getFontSize() * PANGO_SCALE));
     }
+    updateLayout();
 }
 
 static ProfilingZoneID UpdateLayoutProfilingZone("WordsNode: Update layout");
 
 void WordsNode::updateLayout()
 {
-    updateFont();
-    if (m_RedrawState == LAYOUT_CHANGED) {
-        ScopeTimer timer(UpdateLayoutProfilingZone);
+    ScopeTimer timer(UpdateLayoutProfilingZone);
 
-        if (m_sText.length() == 0) {
-            m_LogicalSize = IntPoint(0,0);
-            m_RedrawState = RENDER_NEEDED;
-        } else {
-            PangoContext* pContext = TextEngine::get(m_bHint).getPangoContext();
-            pango_context_set_font_description(pContext, m_pFontDescription);
+    if (m_sText.length() == 0) {
+        m_LogicalSize = IntPoint(0,0);
+        m_bRenderNeeded = true;
+    } else {
+        TextEngine& engine = TextEngine::get(m_FontStyle.getHint());
+        PangoContext* pContext = engine.getPangoContext();
+        pango_context_set_font_description(pContext, m_pFontDescription);
 
-            if (m_pLayout) {
-                g_object_unref(m_pLayout);
-            }
-            m_pLayout = pango_layout_new(pContext);
-
-            PangoAttrList * pAttrList = 0;
-#if PANGO_VERSION > PANGO_VERSION_ENCODE(1,18,2) 
-            PangoAttribute * pLetterSpacing = pango_attr_letter_spacing_new
-                    (int(m_LetterSpacing*1024));
-#endif
-            if (m_bParsedText) {
-                char * pText = 0;
-                parseString(&pAttrList, &pText);
-#if PANGO_VERSION > PANGO_VERSION_ENCODE(1,18,2) 
-                // Workaround for pango bug.
-                pango_attr_list_insert_before(pAttrList, pLetterSpacing);
-#endif            
-                pango_layout_set_text(m_pLayout, pText, -1);
-                g_free (pText);
-            } else {
-                pAttrList = pango_attr_list_new();
-#if PANGO_VERSION > PANGO_VERSION_ENCODE(1,18,2) 
-                pango_attr_list_insert_before(pAttrList, pLetterSpacing);
-#endif
-                pango_layout_set_text(m_pLayout, m_sText.c_str(), -1);
-            }
-            pango_layout_set_attributes(m_pLayout, pAttrList);
-            pango_attr_list_unref(pAttrList);
-
-            pango_layout_set_wrap(m_pLayout, m_WrapMode);
-            pango_layout_set_alignment(m_pLayout, m_Alignment);
-            pango_layout_set_justify(m_pLayout, m_bJustify);
-            if (getUserSize().x != 0) {
-                pango_layout_set_width(m_pLayout, int(getUserSize().x * PANGO_SCALE));
-            }
-            pango_layout_set_indent(m_pLayout, m_Indent * PANGO_SCALE);
-            if (m_Indent < 0) {
-                // For hanging indentation, we add a tabstop to support lists
-                PangoTabArray* pTabs = pango_tab_array_new_with_positions(1, false,
-                        PANGO_TAB_LEFT, -m_Indent * PANGO_SCALE);
-                pango_layout_set_tabs(m_pLayout, pTabs);
-                pango_tab_array_free(pTabs);
-            }
-            if (m_LineSpacing != -1) {
-                pango_layout_set_spacing(m_pLayout, (int)(m_LineSpacing*PANGO_SCALE));
-            }
-            PangoRectangle logical_rect;
-            PangoRectangle ink_rect;
-            pango_layout_get_pixel_extents(m_pLayout, &ink_rect, &logical_rect);
-            
-    /*        
-            cerr << getID() << endl;
-            cerr << "Ink: " << ink_rect.x << ", " << ink_rect.y << ", " 
-                    << ink_rect.width << ", " << ink_rect.height << endl;
-            cerr << "Logical: " << logical_rect.x << ", " << logical_rect.y << ", " 
-                    << logical_rect.width << ", " << logical_rect.height << endl;
-            cerr << "User Size: " << getUserSize() << endl;
-    */        
-            m_InkSize.y = ink_rect.height;
-            if (getUserSize().x == 0) {
-                m_InkSize.x = ink_rect.width;
-            } else {
-                m_InkSize.x = int(getUserSize().x);
-            }
-            if (m_InkSize.x == 0) {
-                m_InkSize.x = 1;
-            }
-            if (m_InkSize.y == 0) {
-                m_InkSize.y = 1;
-            }
-            m_LogicalSize.y = logical_rect.height;
-            m_LogicalSize.x = logical_rect.width;
-            m_InkOffset = IntPoint(ink_rect.x-logical_rect.x, ink_rect.y-logical_rect.y);
-            if (m_LineSpacing == -1) {
-                m_LineSpacing = pango_layout_get_spacing(m_pLayout)/PANGO_SCALE;
-            }
-            m_RedrawState = RENDER_NEEDED;
-            setViewport(-32767, -32767, -32767, -32767);
+        if (m_pLayout) {
+            g_object_unref(m_pLayout);
         }
+        m_pLayout = pango_layout_new(pContext);
+
+        PangoAttrList * pAttrList = 0;
+#if PANGO_VERSION > PANGO_VERSION_ENCODE(1,18,2) 
+        PangoAttribute * pLetterSpacing = pango_attr_letter_spacing_new
+            (int(m_FontStyle.getLetterSpacing()*1024));
+#endif
+        if (m_bParsedText) {
+            char * pText = 0;
+            parseString(&pAttrList, &pText);
+#if PANGO_VERSION > PANGO_VERSION_ENCODE(1,18,2) 
+            // Workaround for pango bug.
+            pango_attr_list_insert_before(pAttrList, pLetterSpacing);
+#endif            
+            pango_layout_set_text(m_pLayout, pText, -1);
+            g_free(pText);
+        } else {
+            pAttrList = pango_attr_list_new();
+#if PANGO_VERSION > PANGO_VERSION_ENCODE(1,18,2) 
+            pango_attr_list_insert_before(pAttrList, pLetterSpacing);
+#endif
+            pango_layout_set_text(m_pLayout, m_sText.c_str(), -1);
+        }
+        pango_layout_set_attributes(m_pLayout, pAttrList);
+        pango_attr_list_unref(pAttrList);
+
+        pango_layout_set_wrap(m_pLayout, m_FontStyle.getWrapModeVal());
+        pango_layout_set_alignment(m_pLayout, m_FontStyle.getAlignmentVal());
+        pango_layout_set_justify(m_pLayout, m_FontStyle.getJustify());
+        if (getUserSize().x != 0) {
+            pango_layout_set_width(m_pLayout, int(getUserSize().x * PANGO_SCALE));
+        }
+        int indent = m_FontStyle.getIndent() * PANGO_SCALE;
+        pango_layout_set_indent(m_pLayout, indent);
+        if (indent < 0) {
+            // For hanging indentation, we add a tabstop to support lists
+            PangoTabArray* pTabs = pango_tab_array_new_with_positions(1, false,
+                    PANGO_TAB_LEFT, -indent);
+            pango_layout_set_tabs(m_pLayout, pTabs);
+            pango_tab_array_free(pTabs);
+        }
+        pango_layout_set_spacing(m_pLayout, 
+                (int)(m_FontStyle.getLineSpacing()*PANGO_SCALE));
+        PangoRectangle logical_rect;
+        PangoRectangle ink_rect;
+        pango_layout_get_pixel_extents(m_pLayout, &ink_rect, &logical_rect);
+
+        /*        
+                  cerr << getID() << endl;
+                  cerr << "Ink: " << ink_rect.x << ", " << ink_rect.y << ", " 
+                  << ink_rect.width << ", " << ink_rect.height << endl;
+                  cerr << "Logical: " << logical_rect.x << ", " << logical_rect.y << ", " 
+                  << logical_rect.width << ", " << logical_rect.height << endl;
+                  cerr << "User Size: " << getUserSize() << endl;
+                  */        
+        m_InkSize.y = ink_rect.height;
+        if (getUserSize().x == 0) {
+            m_InkSize.x = ink_rect.width;
+        } else {
+            m_InkSize.x = int(getUserSize().x);
+        }
+        if (m_InkSize.x == 0) {
+            m_InkSize.x = 1;
+        }
+        if (m_InkSize.y == 0) {
+            m_InkSize.y = 1;
+        }
+        m_LogicalSize.y = logical_rect.height;
+        m_LogicalSize.x = logical_rect.width;
+        m_InkOffset = IntPoint(ink_rect.x-logical_rect.x, ink_rect.y-logical_rect.y);
+        m_bRenderNeeded = true;
+        setViewport(-32767, -32767, -32767, -32767);
     }
 }
 
@@ -662,15 +630,16 @@ static ProfilingZoneID RenderTextProfilingZone("WordsNode: render text");
 
 void WordsNode::renderText()
 {
-    AVG_ASSERT(m_RedrawState == RENDER_NEEDED || m_RedrawState == CLEAN);
-
     if (!(getState() == NS_CANRENDER)) {
         return;
     }
-    if (m_RedrawState == RENDER_NEEDED) {
+    if (m_bRenderNeeded) {
         if (m_sText.length() != 0) {
             ScopeTimer timer(RenderTextProfilingZone);
-            int maxTexSize = GLContext::getCurrent()->getMaxTexSize();
+            TextEngine& engine = TextEngine::get(m_FontStyle.getHint());
+            PangoContext* pContext = engine.getPangoContext();
+            pango_context_set_font_description(pContext, m_pFontDescription);
+            int maxTexSize = GLContext::getMain()->getMaxTexSize();
             if (m_InkSize.x > maxTexSize || m_InkSize.y > maxTexSize) {
                 throw Exception(AVG_ERR_UNSUPPORTED, 
                         "WordsNode size exceeded maximum (Size=" 
@@ -695,7 +664,7 @@ void WordsNode::renderText()
             PangoRectangle ink_rect;
             pango_layout_get_pixel_extents(m_pLayout, &ink_rect, &logical_rect);
             pango_ft2_render_layout(&bitmap, m_pLayout, -ink_rect.x, -ink_rect.y);
-            switch (m_Alignment) {
+            switch (m_FontStyle.getAlignmentVal()) {
                 case PANGO_ALIGN_LEFT:
                     m_AlignOffset = 0;
                     break;
@@ -711,10 +680,9 @@ void WordsNode::renderText()
 
             pMover->unlock();
             pMover->moveToTexture(*pTex);
-
-            bind();
+            newSurface();
         }
-        m_RedrawState = CLEAN;
+        m_bRenderNeeded = false;
     }
 }
 
@@ -722,45 +690,43 @@ void WordsNode::redraw()
 {
     AVG_ASSERT(m_sText.length() < 32767);
     
-    updateLayout();
     renderText();
 }
 
-void WordsNode::preRender()
+void WordsNode::preRender(const VertexArrayPtr& pVA, bool bIsParentActive, 
+        float parentEffectiveOpacity)
 {
-    Node::preRender();
+    Node::preRender(pVA, bIsParentActive, parentEffectiveOpacity);
     if (isVisible()) {
         redraw();
-    } else {
-        updateLayout();
     }
+    Pixel32 color = m_FontStyle.getColorVal();
     if (m_sText.length() != 0 && isVisible()) {
-        renderFX(getSize(), m_Color, false);
+        renderFX(getSize(), color, false);
     }
+    calcVertexArray(pVA, color);
 }
 
 static ProfilingZoneID RenderProfilingZone("WordsNode::render");
 
-void WordsNode::render(const DRect& rect)
+void WordsNode::render()
 {
     ScopeTimer timer(RenderProfilingZone);
     if (m_sText.length() != 0 && isVisible()) {
         IntPoint offset = m_InkOffset + IntPoint(m_AlignOffset, 0);
-        GLContext* pContext = GLContext::getCurrent();
-        if (offset != IntPoint(0,0)) {
-            pContext->pushTransform(DPoint(offset), 0, DPoint(0,0));
-        }
-        blta8(DPoint(getSurface()->getSize()), getEffectiveOpacity(), m_Color, 
-                getBlendMode());
-        if (offset != IntPoint(0,0)) {
-            pContext->popTransform();
+        glm::mat4 transform;
+        if (offset == IntPoint(0,0)) {
+            transform = getTransform();
+        } else {
+            transform = glm::translate(getTransform(), glm::vec3(offset.x, offset.y, 0));
         }
+        blta8(transform, glm::vec2(getSurface()->getSize()), getEffectiveOpacity(), 
+                m_FontStyle.getColorVal(), getBlendMode());
     }
 }
 
 IntPoint WordsNode::getMediaSize()
 {
-    updateLayout();
     return m_LogicalSize;
 }
 
@@ -798,7 +764,6 @@ PangoRectangle WordsNode::getGlyphRect(int i)
         throw(Exception(AVG_ERR_INVALID_ARGS, 
                 string("getGlyphRect: Index ") + toString(i) + " out of range."));
     }
-    updateLayout();
     const char* pText = pango_layout_get_text(m_pLayout);
     char * pChar = g_utf8_offset_to_pointer(pText, i);
     int byteOffset = pChar-pText;
@@ -818,16 +783,16 @@ PangoRectangle WordsNode::getGlyphRect(int i)
 void WordsNode::setParsedText(const UTF8String& sText)
 {
     m_sText = removeExcessSpaces(sText);
-    setDirty(LAYOUT_CHANGED);
 
     // This just does a syntax check and throws an exception if appropriate.
     // The results are discarded.
     PangoAttrList * pAttrList = 0;
     char * pText = 0;
     parseString(&pAttrList, &pText);
-    pango_attr_list_unref (pAttrList);
-    g_free (pText);
+    pango_attr_list_unref(pAttrList);
+    g_free(pText);
     m_bParsedText = true;
+    updateLayout();
 }
 
 UTF8String WordsNode::applyBR(const UTF8String& sText)
