@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 
 #include "../api.h"
 #include "RasterNode.h"
+#include "FontStyle.h"
 #include "../graphics/Pixel32.h"
 #include "../base/UTF8String.h"
 
@@ -37,7 +38,7 @@ namespace avg {
 class AVG_API WordsNode : public RasterNode
 {
     public:
-        static NodeDefinition createDefinition();
+        static void registerType();
         
         WordsNode(const ArgList& args);
         virtual ~WordsNode();
@@ -45,21 +46,27 @@ class AVG_API WordsNode : public RasterNode
         virtual void connectDisplay();
         virtual void connect(CanvasPtr pCanvas);
         virtual void disconnect(bool bKill);
-        virtual void preRender();
-        virtual void render(const DRect& rect);
+        virtual void preRender(const VertexArrayPtr& pVA, bool bIsParentActive, 
+                float parentEffectiveOpacity);
+        virtual void render();
 
-        virtual double getWidth() const;
-        virtual void setWidth(double width);
+        virtual float getWidth() const;
+        virtual void setWidth(float width);
 
-        virtual double getHeight() const;
-        virtual void setHeight(double width);
+        virtual float getHeight() const;
+        virtual void setHeight(float width);
 
-        virtual DPoint getSize() const;
-        virtual void setSize(const DPoint& pt);
+        virtual glm::vec2 getSize() const;
+        virtual void setSize(const glm::vec2& pt);
+
+        glm::vec2 toLocal(const glm::vec2& globalPos) const;
+        glm::vec2 toGlobal(const glm::vec2& localPos) const;
 
-        void getElementsByPos(const DPoint& pos, std::vector<NodeWeakPtr>& pElements);
         void setTextFromNodeValue(const std::string& sText);
 
+        const FontStyle& getFontStyle() const;
+        void setFontStyle(const FontStyle& fontStyle);
+
         const std::string& getFont() const;
         void setFont(const std::string& sName);
 
@@ -72,14 +79,17 @@ class AVG_API WordsNode : public RasterNode
         const std::string& getColor() const;
         void setColor(const std::string& sColor);
         
-        double getFontSize() const;
-        void setFontSize(double size);
+        virtual float getAAGamma() const;
+        virtual void setAAGamma(float gamma);
+
+        float getFontSize() const;
+        void setFontSize(float size);
         
         int getIndent() const;
         void setIndent(int indent);
         
-        double getLineSpacing() const;
-        void setLineSpacing(double lineSpacing);
+        float getLineSpacing() const;
+        void setLineSpacing(float lineSpacing);
         
         bool getRawTextMode() const;
         void setRawTextMode(bool rawTextMode);
@@ -93,20 +103,20 @@ class AVG_API WordsNode : public RasterNode
         bool getJustify() const;
         void setJustify(bool bJustify);
 
-        double getLetterSpacing() const;
-        void setLetterSpacing(double letterSpacing);
+        float getLetterSpacing() const;
+        void setLetterSpacing(float letterSpacing);
 
         bool getHint() const;
         void setHint(bool bHint);
 
-        DPoint getGlyphPos(int i);
-        DPoint getGlyphSize(int i);
+        glm::vec2 getGlyphPos(int i);
+        glm::vec2 getGlyphSize(int i);
         virtual IntPoint getMediaSize();
         
         int getNumLines();
-        PyObject* getCharIndexFromPos(DPoint p);
+        PyObject* getCharIndexFromPos(glm::vec2 p);
         std::string getTextAsDisplayed();
-        DPoint getLineExtents(int line);
+        glm::vec2 getLineExtents(int line);
     
         static const std::vector<std::string>& getFontFamilies();
         static const std::vector<std::string>& getFontVariants(
@@ -114,10 +124,7 @@ class AVG_API WordsNode : public RasterNode
         static void addFontDir(const std::string& sDir);
 
     private:
-        enum RedrawState {FONT_CHANGED, LAYOUT_CHANGED, RENDER_NEEDED, CLEAN};
-
         virtual void calcMaskCoords();
-        void setDirty(RedrawState newState);
         void updateFont();
         void updateLayout();
         void renderText();
@@ -129,21 +136,10 @@ class AVG_API WordsNode : public RasterNode
         PangoRectangle getGlyphRect(int i);
 
         // Exposed Attributes
-        std::string m_sFontName;
-        std::string m_sFontVariant;
+        FontStyle m_FontStyle;
         UTF8String m_sText;
         UTF8String m_sRawText;
-        std::string m_sColorName;
-        Pixel32 m_Color;
-        double m_FontSize;
-        int m_Indent;
-        double m_LineSpacing;
-        PangoAlignment m_Alignment;
-        PangoWrapMode m_WrapMode;
-        bool m_bJustify;
-        double m_LetterSpacing;
-        bool m_bHint;
-       
+
         bool m_bParsedText;
         bool m_bRawTextMode;
         IntPoint m_LogicalSize;
@@ -153,7 +149,7 @@ class AVG_API WordsNode : public RasterNode
         PangoFontDescription * m_pFontDescription;
         PangoLayout * m_pLayout;
 
-        RedrawState m_RedrawState;
+        bool m_bRenderNeeded;
 };
 
 }
