@@ -1,3 +1,8 @@
+'''
+libavg is a high-level development platform for media-centric applications.
+https://www.libavg.de
+'''
+
 # Work around libstdc++ Mesa bug
 # (https://bugs.launchpad.net/ubuntu/+source/mesa/+bug/259219)
 from platform import system
@@ -8,12 +13,17 @@ if system() == 'Linux':
 del system
 
 from avg import *
-import anim
-import draggable
+player = avg.Player.get()
+
+from enumcompat import *
+
 import textarea
 import statemachine
-from grabbable import Grabbable
-from app import AVGApp, App
+from avgapp import AVGApp
 from appstarter import AVGAppStarter, AVGMTAppStarter, AppStarter
-import utils
-import gameapp
+import utils, methodref
+import gesture
+import filter
+import persist
+import app
+
