@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -23,18 +23,16 @@
 
 import shutil
 
-from libavg import avg
+from libavg import avg, player
 from testcase import *
 
-g_IsMaskSupported = None
-
 class ImageTestCase(AVGTestCase):
     def __init__(self, testFuncName):
         AVGTestCase.__init__(self, testFuncName)
 
     def testImageHRef(self):
         def createXmlNode(pos):
-            node = Player.createNode(
+            node = player.createNode(
                     """<image pos="%s" href="rgb24-32x32.png"/>"""%str(pos))
             self.assertEqual(node.getMediaSize(), avg.Point2D(32, 32))
             return node
@@ -46,7 +44,6 @@ class ImageTestCase(AVGTestCase):
             return node
 
         def addNodes(y):
-            
             xmlNode = createXmlNode((16, y))
             root.appendChild(xmlNode)
             
@@ -68,29 +65,29 @@ class ImageTestCase(AVGTestCase):
             if self._isCurrentDirWriteable():
                 # Can't check unicode filenames into svn or the windows client breaks.
                 # So we rename the file locally.
-                shutil.copyfile("oe.png", u"ö.png")
+                shutil.copyfile("media/oe.png", u"media/ö.png")
                 node = createXmlNode((16, 16))
                 root.appendChild(node)
                 node.href = u"ö.png"
-                os.remove(u"ö.png")
+                os.remove(u"media/ö.png")
 
         def compareUnicode():
             if self._isCurrentDirWriteable():
-                self.compareImage("testImgHRef3", False)
+                self.compareImage("testImgHRef3")
 
         root = self.loadEmptyScene()
         addNodes(16)
-        self.start((
-                 lambda: self.compareImage("testImgHRef1", False),
+        self.start(False,
+                (lambda: self.compareImage("testImgHRef1"),
                  lambda: addNodes(48),
-                 lambda: self.compareImage("testImgHRef2", False),
+                 lambda: self.compareImage("testImgHRef2"),
                  setUnicodeHref,
                  compareUnicode
                 ))
       
     def testImagePos(self):
         def createXmlNode(pos):
-            return Player.createNode(
+            return player.createNode(
                     """<image pos="%s" href="rgb24-32x32.png"/>"""%str(pos))        
 
         def createDictNode(root, p):
@@ -115,15 +112,15 @@ class ImageTestCase(AVGTestCase):
 
         root = self.loadEmptyScene()
         addNodes(16)
-        self.start((
-                 lambda: self.compareImage("testImgPos1", False),
+        self.start(False,
+                (lambda: self.compareImage("testImgPos1"),
                  lambda: addNodes(48),
-                 lambda: self.compareImage("testImgPos2", False),
+                 lambda: self.compareImage("testImgPos2"),
                 ))
 
     def testImageSize(self):
         def createXmlNode(pos, size):
-            return Player.createNode(
+            return player.createNode(
                     """<image pos="%s" size="%s" href="rgb24-64x64.png"/>"""
                     %(str(pos), str(size))) 
 
@@ -148,10 +145,10 @@ class ImageTestCase(AVGTestCase):
 
         root = self.loadEmptyScene()
         addNodes(16)
-        self.start((
-                 lambda: self.compareImage("testImgSize1", False),
+        self.start(False,
+                (lambda: self.compareImage("testImgSize1"),
                  lambda: addNodes(48),
-                 lambda: self.compareImage("testImgSize2", False),
+                 lambda: self.compareImage("testImgSize2"),
                 ))
        
     def testImageWarp(self):
@@ -171,15 +168,24 @@ class ImageTestCase(AVGTestCase):
             node.unlink()
 
         def addNode():
-            node = createNode((16, 16))
-            root.appendChild(node)
-            moveVertex(node)
-        
+            self.node = createNode((16, 16))
+            root.appendChild(self.node)
+            moveVertex(self.node)
+       
+        def changeHref():
+            self.node.href = "rgb24-65x65.png"
+            grid = self.node.getWarpedVertexCoords()
+            self.assert_(len(grid) == 10)
+            self.assert_(len(grid[0]) == 6)
+            
+
         root = self.loadEmptyScene()
         testEarlyAccessException()
-        self.start((
-                 lambda: addNode(),
-                 lambda: self.compareImage("testImgWarp1", False),
+        self.start(False,
+                (lambda: addNode(),
+                 lambda: self.compareImage("testImgWarp1"),
+                 lambda: changeHref(),
+                 lambda: self.compareImage("testImgWarp2"),
                 ))
 
     def testBitmap(self):
@@ -193,35 +199,57 @@ class ImageTestCase(AVGTestCase):
         
         def loadFromBitmap(p, orighref):
             node = avg.ImageNode(pos=p, size=(32, 32), href=orighref)
-            bmp = avg.Bitmap('rgb24-65x65.png')
+            bmp = avg.Bitmap('media/rgb24-65x65.png')
             self.assertEqual(bmp.getSize(), (65,65))
             node.setBitmap(bmp)
             self.assertEqual(node.getMediaSize(), (65,65))
             root.appendChild(node)
         
         def testStringConversion():
-            bmp = avg.Bitmap('rgb24-65x65.png')
+            bmp = avg.Bitmap('media/rgb24-65x65.png')
             s = bmp.getPixels()
-            bmp1 = avg.Bitmap(bmp.getSize(), avg.B8G8R8X8, "sample")
+            bmp1 = avg.Bitmap(bmp.getSize(), bmp.getFormat(), "sample")
             bmp1.setPixels(s)
             self.assert_(self.areSimilarBmps(bmp, bmp1, 0.01, 0.01))
 
+        def testCropRect():
+            bmp = avg.Bitmap('media/rgb24-65x65.png')
+            bmp1 = avg.Bitmap(bmp, (32,32), (64,64))
+            self.assert_(bmp1.getSize() == (32,32))
+            node = avg.ImageNode(pos=(96,0), parent=root)
+            node.setBitmap(bmp1)
+
+        def testBlt():
+            srcBmp = avg.Bitmap('media/rgb24-65x65.png')
+            destBmp = avg.Bitmap((65,65), srcBmp.getFormat(), "bmp")
+            destBmp.blt(srcBmp, (0,0))
+            destBmp.blt(srcBmp, (32,32))
+            node = avg.ImageNode(pos=(96,32), size=(32,32), parent=root)
+            node.setBitmap(destBmp)
+            
+        def testResize():
+            srcBmp = avg.Bitmap('media/rgb24-32x32.png')
+            destBmp = srcBmp.getResized((64,64))
+            self.assert_(destBmp.getSize() == (64,64))
+            node = avg.ImageNode(pos=(128,0), size=(32,32), parent=root)
+            node.setBitmap(destBmp)
+
         def testUnicode():
             if self._isCurrentDirWriteable():
                 # Can't check unicode filenames into svn or the windows client breaks.
                 # So we rename the file locally.
-                shutil.copyfile("oe.png", u"ö.png")
-                bmp = avg.Bitmap(u"ö.png")
-                os.remove(u"ö.png")
+                shutil.copyfile("media/oe.png", u"media/ö.png")
+                avg.Bitmap(u"media/ö.png")
+                os.remove(u"media/ö.png")
 
         def testGetPixel():
-            bmp = avg.Bitmap('rgb24-65x65.png')
+            bmp = avg.Bitmap('media/rgb24-65x65.png')
             self.assertEqual(bmp.getPixel((1,1)), (255,0,0,255))
             self.assertEqual(bmp.getPixel((33,1)), (0,255,0,255))
-            bmp = avg.Bitmap('rgb24alpha-64x64.png')
+            bmp = avg.Bitmap('media/rgb24alpha-64x64.png')
             self.assertEqual(bmp.getPixel((1,1)), (0,0,0,0))
             self.assertEqual(bmp.getPixel((63,1)), (83,255,83,142))
-            bmp = avg.Bitmap('greyscale.png')
+            bmp = avg.Bitmap('media/greyscale.png')
             self.assertEqual(bmp.getPixel((1,1)), (255,255,255,255))
             self.assertEqual(bmp.getPixel((1,63)), (0,0,0,255))
             self.assertException(lambda: bmp.getPixel((64,0)))
@@ -229,7 +257,13 @@ class ImageTestCase(AVGTestCase):
         def setNullBitmap():
             node.setBitmap(None)
 
-        node = avg.ImageNode(href="rgb24-65x65.png", size=(32, 32))
+        def testSubBitmap():
+            srcBmp = avg.Bitmap('media/rgb24-32x32.png')
+            destBmp = avg.Bitmap(srcBmp, (16,16), (32,32))
+            self.assertEqual(srcBmp.getPixel((16,16)), destBmp.getPixel((0,0)))
+            self.assertException(lambda: avg.Bitmap(srcBmp, (16,16), (16,32)))
+
+        node = avg.ImageNode(href="media/rgb24-65x65.png", size=(32, 32))
         getBitmap(node)
 
         root = self.loadEmptyScene()
@@ -241,13 +275,20 @@ class ImageTestCase(AVGTestCase):
         loadFromBitmap((64,0), "rgb24alpha-64x64.png")
         testStringConversion()
         testUnicode()
-        self.start((
-                 lambda: getBitmap(node),
+        self.start(False,
+                (lambda: getBitmap(node),
                  lambda: loadFromBitmap((32,32), ""),
                  lambda: loadFromBitmap((64,32), "rgb24alpha-64x64.png"),
-                 lambda: self.compareImage("testBitmap1", False),
+                 lambda: self.compareImage("testBitmap1"),
+                 testCropRect,
+                 lambda: self.compareImage("testBitmap2"),
+                 testBlt,
+                 lambda: self.compareImage("testBitmap3"),
+                 testResize,
+                 lambda: self.compareImage("testBitmap4"),
                  testGetPixel,
-                 lambda: self.assertException(setNullBitmap)
+                 lambda: self.assertException(setNullBitmap),
+                 testSubBitmap,
                 ))
 
     def testBitmapManager(self):
@@ -262,17 +303,27 @@ class ImageTestCase(AVGTestCase):
         def loadValidBitmap():
             def validBitmapCb(bitmap):
                 self.assert_(not isinstance(bitmap, Exception))
-                Player.setTimeout(0, loadUnexistentBitmap)
+                player.setTimeout(0, loadBitmapWithPixelFormat)
 
-            avg.BitmapManager.get().loadBitmap("rgb24alpha-64x64.png", validBitmapCb)
+            avg.BitmapManager.get().loadBitmap("media/rgb24alpha-64x64.png",
+                    validBitmapCb)
+
+        def loadBitmapWithPixelFormat():
+            def validBitmapCb(bitmap):
+                self.assert_(not isinstance(bitmap, Exception))
+                self.assert_(bitmap.getFormat() == avg.B5G6R5)
+                player.setTimeout(0, loadUnexistentBitmap)
+
+            avg.BitmapManager.get().loadBitmap("media/rgb24alpha-64x64.png",
+                    validBitmapCb, avg.B5G6R5)
 
         def loadUnexistentBitmap():
             avg.BitmapManager.get().loadBitmap("nonexistent.png",
                     lambda bmp: expectException(
                             returnValue=bmp,
-                            nextAction=lambda: Player.setTimeout(0, loadBrokenImage)))
+                            nextAction=lambda: player.setTimeout(0, loadBrokenBitmap)))
 
-        def loadBrokenImage():
+        def loadBrokenBitmap():
             import tempfile
             tempFileName = os.path.join(tempfile.gettempdir(),
                     "broken.png")
@@ -280,27 +331,55 @@ class ImageTestCase(AVGTestCase):
 
             def cleanupAndTestReturnValue(returnValue):
                 os.unlink(tempFileName)
-                expectException(returnValue=returnValue, nextAction=Player.stop)
+                expectException(returnValue=returnValue, nextAction=player.stop)
 
             avg.BitmapManager.get().loadBitmap(tempFileName,
                     cleanupAndTestReturnValue)
-        
+
         def reportStuck():
             raise RuntimeError("BitmapManager didn't reply "
                     "within %dms timeout" % WAIT_TIMEOUT)
-            Player.stop()
+            player.stop()
             
-        root = self.loadEmptyScene()
-        
-        Player.setTimeout(WAIT_TIMEOUT, reportStuck)
-        Player.setResolution(0, 0, 0, 0)
-        loadValidBitmap()
-        Player.play()
+        for multithread in [False, True]:
+            self.loadEmptyScene()
+            if multithread:
+                avg.BitmapManager.get().setNumThreads(2)
+            player.setTimeout(WAIT_TIMEOUT, reportStuck)
+            player.setResolution(0, 0, 0, 0)
+            loadValidBitmap()
+            player.play()
+        avg.BitmapManager.get().setNumThreads(1)
         
+    def testBitmapManagerException(self):
+        def bitmapCb(bitmap):
+            raise RuntimeError
+
+        self.loadEmptyScene()
+        avg.BitmapManager.get().loadBitmap("rgb24alpha-64x64.png", bitmapCb),
+        self.assertException(player.play)
+
     def testBlendMode(self):
+        def isBlendMinMaxSupported():
+            def tryInsertNode():
+                try:
+                    avg.ImageNode(href="rgb24-65x65.png", blendmode="min", parent=root)
+                except RuntimeError:
+                    self.supported = False
+            root = self.loadEmptyScene()
+            self.supported = True
+            self.start(False,
+                    (tryInsertNode,
+                    ))
+            return self.supported
+            
+
         def setBlendMode():
             blendNode.blendmode="add"
         
+        if not(isBlendMinMaxSupported()):
+            self.skip("Blend modes min and max not supported.")
+            return
         root = self.loadEmptyScene()
         avg.ImageNode(href="freidrehen.jpg", parent=root)
         blendNode = avg.ImageNode(opacity=0.6, href="rgb24-65x65.png", parent=root)
@@ -311,10 +390,10 @@ class ImageTestCase(AVGTestCase):
         avg.ImageNode(pos=(48,48), opacity=1, href="rgb24-65x65.png", blendmode="max",
                 parent=root)
 
-        self.start((
-                 lambda: self.compareImage("testBlend1", False),
+        self.start(False,
+                (lambda: self.compareImage("testBlend1"),
                  setBlendMode,
-                 lambda: self.compareImage("testBlend2", False)
+                 lambda: self.compareImage("testBlend2")
                 ))
 
     def testImageMask(self):
@@ -342,34 +421,31 @@ class ImageTestCase(AVGTestCase):
         def setMaskNotFound():
             node.maskhref = "nonexistentmask.png"        
             
-        if not(self._hasShaderSupport()):
-            return
         root = self.loadEmptyScene()
         createNode((0,0))
         node = root.getChild(0)
         setNoAttach((32,0))
         setAttach((64,0))
-        self.start((
-                 lambda: createNode((0, 32)),
+        self.start(False,
+                (lambda: createNode((0, 32)),
                  lambda: setNoAttach((32,32)),
                  lambda: setAttach((64,32)),
-                 lambda: self.compareImage("testImgMask1", False),
+                 lambda: self.compareImage("testImgMask1"),
                  changeHRef,
-                 lambda: self.compareImage("testImgMask2", False),
+                 lambda: self.compareImage("testImgMask2"),
                  changeBaseHRef,
-                 lambda: self.compareImage("testImgMask3", False),
+                 lambda: self.compareImage("testImgMask3"),
                  setMaskNotFound
                 ))
 
     def testImageMaskCanvas(self):
-        if not(self._hasShaderSupport()):
-            return
         root = self.loadEmptyScene()
-        canvas = Player.createCanvas(id="testcanvas", size=(64,64))
+        canvas = player.createCanvas(id="testcanvas", size=(64,64), mediadir="media")
         avg.ImageNode(href="rgb24-64x64.png", parent=canvas.getRootNode())
         avg.RectNode(size=(160,120), fillcolor="FFFFFF", fillopacity=1, parent=root)
-        node = avg.ImageNode(href="canvas:testcanvas", maskhref="mask.png", parent=root)
-        self.start([lambda: self.compareImage("testImgMaskCanvas", False)])
+        avg.ImageNode(href="canvas:testcanvas", maskhref="mask.png", parent=root)
+        self.start(False,
+                (lambda: self.compareImage("testImgMaskCanvas"),))
 
     def testImageMaskPos(self):
         def createNode(p):
@@ -390,16 +466,14 @@ class ImageTestCase(AVGTestCase):
             node.maskpos = (32, 32)
 
         root = self.loadEmptyScene()
-        if not(self._hasShaderSupport()):
-            return
         createNode((0,0))
         setNoAttach((32,0))
         setAttach((64,0))
-        self.start((
-                 lambda: createNode((0, 32)),
+        self.start(False,
+                (lambda: createNode((0, 32)),
                  lambda: setNoAttach((32,32)),
                  lambda: setAttach((64,32)),
-                 lambda: self.compareImage("testImgMaskPos", False)
+                 lambda: self.compareImage("testImgMaskPos")
                 ))
 
     def testImageMaskSize(self):
@@ -426,33 +500,30 @@ class ImageTestCase(AVGTestCase):
             node.masksize = (0, 0)
 
         root = self.loadEmptyScene()
-        if not(self._hasShaderSupport()):
-            return
         createNode((0,0))
         node = root.getChild(0)
         setNoAttach((32,0))
         setAttach((64,0))
-        self.start((
-                 lambda: createNode((0, 32)),
+        self.start(False,
+                (lambda: createNode((0, 32)),
                  lambda: setNoAttach((32,32)),
                  lambda: setAttach((64,32)),
-                 lambda: self.compareImage("testImgMaskSize1", False),
+                 lambda: self.compareImage("testImgMaskSize1"),
                  setPos,
-                 lambda: self.compareImage("testImgMaskSize2", False),
+                 lambda: self.compareImage("testImgMaskSize2"),
                  resetPos,
-                 lambda: self.compareImage("testImgMaskSize3", False)
+                 lambda: self.compareImage("testImgMaskSize3")
                 ))
 
     def testImageMipmap(self):
         root = self.loadEmptyScene()
-        avg.ImageNode(size=(64,64), href="checker.png", parent=root)
-        avg.ImageNode(pos=(64,0), size=(64,64), href="checker.png", mipmap=True, 
-                parent=root)
-        self.start([lambda: self.compareImage("testMipmap", False)])
+        avg.ImageNode(size=(64,64), href="checker.png", mipmap=True, parent=root)
+        self.start(False,
+                (lambda: self.compareImage("testMipmap"),))
 
     def testImageCompression(self):
         def loadBitmap():
-            bmp = avg.Bitmap("colorramp.png")
+            bmp = avg.Bitmap("media/colorramp.png")
             self.image.setBitmap(bmp)
 
         def relink():
@@ -466,15 +537,30 @@ class ImageTestCase(AVGTestCase):
         self.image = avg.ImageNode(href="rgb24-64x64.png", compression="B5G6R5",
                 parent=root)
         self.assertEqual(self.image.compression, "B5G6R5")
-        self.start([
-                 lambda: self.compareImage("testTexCompression1", False),
+        self.start(False, 
+                [lambda: self.compareImage("testTexCompression1"),
                  loadBitmap,
-                 lambda: self.compareImage("testTexCompression2", False),
+                 lambda: self.compareImage("testTexCompression2"),
                  relink,
-                 lambda: self.compareImage("testTexCompression2", False),
+                 lambda: self.compareImage("testTexCompression2"),
                  checkAlpha,
                 ])
 
+    def testSpline(self):
+        spline = avg.CubicSpline([(0,3),(1,2),(2,1),(3,0)])
+        self.assertAlmostEqual(spline.interpolate(0), 3)
+        self.assertAlmostEqual(spline.interpolate(0.5), 2.5)
+        self.assertAlmostEqual(spline.interpolate(1), 2)
+        self.assertAlmostEqual(spline.interpolate(-1), 4)
+        self.assertAlmostEqual(spline.interpolate(4), -1)
+
+        spline = avg.CubicSpline([(2,0),(4,1),(6,3),(8,6)])
+        self.assertAlmostEqual(spline.interpolate(2), 0)
+        self.assert_(spline.interpolate(3) < 0.5)
+        self.assert_(spline.interpolate(3) > 0.0)
+        self.assert_(spline.interpolate(7) < 4.5)
+        self.assert_(spline.interpolate(7) > 4)
+
 
 def imageTestSuite(tests):
     availableTests = (
@@ -484,6 +570,7 @@ def imageTestSuite(tests):
             "testImageWarp",
             "testBitmap",
             "testBitmapManager",
+            "testBitmapManagerException",
             "testBlendMode",
             "testImageMask",
             "testImageMaskCanvas",
@@ -491,8 +578,6 @@ def imageTestSuite(tests):
             "testImageMaskSize",
             "testImageMipmap",
             "testImageCompression",
+            "testSpline",
             )
     return createAVGTestSuite(availableTests, ImageTestCase, tests)
-
-Player = avg.Player.get()
-
