@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -24,12 +24,13 @@ import unittest
 
 import optparse
 import os
+import sys
 
-import libavg
+from libavg import avg, player
 import testcase
 
 
-class TestApp:
+class TestApp(object):
     EXIT_OK = 0
     EXIT_FAILURE = 1
         
@@ -45,7 +46,7 @@ class TestApp:
         self.__testSuite = unittest.TestSuite()
         self.__optionParser = None
         self.__commandlineOptions = None
-        self.__player = libavg.avg.Player.get()
+        player.keepWindowOpen()
             
     def getSuiteFactory(self, name):
         return self.__registerdSuiteFactoriesDict[name]
@@ -80,7 +81,7 @@ class TestApp:
             yield self.__RegisterdSuitesDict[name]
     
     def __runVideoTest(self):
-        self.__player.loadFile("image.avg")
+        player.loadFile("image.avg")
     
     def __run(self):
         testRunner = unittest.TextTestRunner(verbosity = 2)
@@ -92,7 +93,12 @@ class TestApp:
             for test in suite:
                 if test.skipped():
                     numSkipped += 1
-        print "Skipped", numSkipped, "tests."
+        if numSkipped > 0:
+            sys.stderr.write("Skipped "+str(numSkipped)+" tests:\n")
+            for suite in self.__testSuite:
+                for test in suite:
+                    if test.skipped():
+                        print "  " + str(test) + ": " + test.skipReason()
 
         if testResult.wasSuccessful():
             self.__exitOk = TestApp.EXIT_OK
@@ -105,43 +111,57 @@ class TestApp:
         self.__populateTestSuite()
 
     def __setupGlobalPlayerOptions(self):
-        self.__player.setOGLOptions(self.__commandlineOptions.usepow2textures, 
-                self.__commandlineOptions.useshaders,
-                self.__commandlineOptions.usepixelbuffers,
-                1)
-        
+        if self.__commandlineOptions.shaderusage == "FULL":
+            shaderUsage = avg.SHADERUSAGE_FULL
+        elif self.__commandlineOptions.shaderusage == "MINIMAL":
+            shaderUsage = avg.SHADERUSAGE_MINIMAL
+        elif self.__commandlineOptions.shaderusage == "FRAGMENT_ONLY":
+            shaderUsage = avg.SHADERUSAGE_FRAGMENT_ONLY
+        elif self.__commandlineOptions.shaderusage == "AUTO":
+            shaderUsage = avg.SHADERUSAGE_AUTO
+        else:
+            sys.stderr.write("\nUnknown value for --shaderusage command-line parameter.\n")
+            self.__optionParser.print_help()
+            sys.exit(-1)
+
+        player.setOGLOptions(self.__commandlineOptions.usepow2textures, 
+                self.__commandlineOptions.usepixelbuffers, 1, shaderUsage, True)
+
     def __setupCommandlineParser(self):
         self.__optionParser = optparse.OptionParser(
             usage = '%prog [options] [<suite> [testcase] [testcase] [...]]')
-        
+
         self.__optionParser.add_option("--usepow2textures", 
-                                       dest = "usepow2textures", 
-                                       action = 'store_true',
-                                       default = False, 
-                                       help = "Use power of 2 textures")
+                dest = "usepow2textures", 
+                action = 'store_true',
+                default = False, 
+                help = "Use power of 2 textures")
         
-        self.__optionParser.add_option("--noshaders", 
-                                       dest = "useshaders",
-                                       action = 'store_false',
-                                       default = True, 
-                                       help = "Use shaders")
-                
         self.__optionParser.add_option("--nopixelbuffers", 
-                                       dest = "usepixelbuffers",
-                                       action = 'store_false',
-                                       default = True, 
-                                       help = "Use pixel buffers")
+                dest = "usepixelbuffers",
+                action = 'store_false',
+                default = True, 
+                help = "Use pixel buffers")
+
+        self.__optionParser.add_option("--shaderusage",
+                dest = "shaderusage",
+                default = "AUTO", 
+                help = "Configure usage of shaders. Valid values are FULL, MINIMAL, FRAGMENT_ONLY and AUTO.")
         
     def __parseCommandline(self):
         self.__commandlineOptions, args = self.__optionParser.parse_args()
-        
+
+        # MFX 2013-11-10: cleanup argv consuming testapp args to avoid clashes
+        # with libavg.app.App ArgvExtender
+        sys.argv = [sys.argv[0]]
+
         if len(args): # suite
             suiteFactory = args.pop(0)
             if not(self.isSuiteFactoryRegistered(suiteFactory)):
-                print "Unknown test suite, registered suites:"
+                sys.stderr.write("Unknown test suite, registered suites:\n")
                 for factory in self.getSuiteFactoryNames():
-                    print factory
-                print ''
+                    sys.stderr.write(factory+"\n")
+                sys.stderr.write("\n")
                 self.__optionParser.print_usage()
 
             self.__suitesToRun.append(self.getSuiteFactory(suiteFactory))
@@ -155,14 +175,17 @@ class TestApp:
             self.__testSuite.addTest(suite(self.__suitesTestSubsets))
         
     def __dumpConfig(self):
-        log = libavg.avg.Logger.get()
-        log.pushCategories()
-        log.setCategories(log.APP | log.WARNING | log.CONFIG  | 0)
-        self.__player.loadString("""
+        player.enableGLErrorChecks(True)
+        cats = avg.logger.getCategories()
+        for cat in [avg.logger.Category.APP, avg.logger.Category.CONFIG,
+                avg.logger.Category.DEPREC]:
+            avg.logger.configureCategory(cat, avg.logger.Severity.INFO)
+        player.loadString("""
                 <avg id="avg" width="160" height="120">
                 </avg>
                 """)
-        self.__player.setTimeout(0, self.__player.stop)
-        self.__player.setFramerate(10000)
-        self.__player.play()
-        log.popCategories()
+        player.setTimeout(0, player.stop)
+        player.setFramerate(10000)
+        player.play()
+        for cat, severity in cats.iteritems():
+            avg.logger.configureCategory(cat, severity)
