@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,9 +20,11 @@
 //
 
 #include "FFMpegDemuxer.h"
+
 #include "../base/ScopeTimer.h"
 #include "../base/ObjectCounter.h"
 #include "../base/Exception.h"
+#include "../base/Logger.h"
 
 #include <cstring>
 #include <iostream>
@@ -51,23 +53,36 @@ AVPacket * FFMpegDemuxer::getPacket(int streamIndex)
     // Make sure enableStream was called on streamIndex.
     AVG_ASSERT(m_PacketLists.size() > 0);
     AVG_ASSERT(streamIndex > -1 && streamIndex < 10);
+
     if (m_PacketLists.find(streamIndex) == m_PacketLists.end()) {
         cerr << this << ": getPacket: Stream " << streamIndex << " not found." << endl;
         dump();
         AVG_ASSERT(false);
     }
-    PacketList & CurPacketList = m_PacketLists.find(streamIndex)->second;
-    AVPacket * pPacket;
-    if (!CurPacketList.empty()) {
-        pPacket = CurPacketList.front();
-        CurPacketList.pop_front();
+
+    PacketList& curPacketList = m_PacketLists.find(streamIndex)->second;
+    AVPacket* pPacket;
+    if (!curPacketList.empty()) {
+        // The stream has packets queued already.
+        pPacket = curPacketList.front();
+        curPacketList.pop_front();
     } else {
+        // No packets queued for this stream -> read and queue packets until we get one
+        // that is meant for this stream.
         do {
             pPacket = new AVPacket;
             memset(pPacket, 0, sizeof(AVPacket));
             int err = av_read_frame(m_pFormatContext, pPacket);
-            // TODO: Check url_ferror here too.
             if (err < 0) {
+                // EOF or error
+#if LIBAVUTIL_VERSION_MAJOR > 50
+                if (err != int(AVERROR_EOF)) {
+                    char sz[256];
+                    av_strerror(err, sz, 256);
+                    AVG_TRACE(Logger::category::PLAYER, Logger::severity::ERROR,
+                            "Error decoding video: " << sz);
+                }
+#endif
                 av_free_packet(pPacket);
                 delete pPacket;
                 pPacket = 0;
@@ -75,27 +90,28 @@ AVPacket * FFMpegDemuxer::getPacket(int streamIndex)
             }
             if (pPacket->stream_index != streamIndex) {
                 if (m_PacketLists.find(pPacket->stream_index) != m_PacketLists.end()) {
+                    // Relevant stream, but not ours
                     av_dup_packet(pPacket);
-                    PacketList& OtherPacketList = 
+                    PacketList& otherPacketList = 
                             m_PacketLists.find(pPacket->stream_index)->second;
-                    OtherPacketList.push_back(pPacket);
+                    otherPacketList.push_back(pPacket);
                 } else {
+                    // Disabled stream
                     av_free_packet(pPacket);
                     delete pPacket;
                     pPacket = 0;
                 } 
             } else {
+                // Our stream
                 av_dup_packet(pPacket);
             }
         } while (!pPacket || pPacket->stream_index != streamIndex);
     }
 
-//    double timeBase = av_q2d(m_pFormatContext->streams[streamIndex]->time_base);
-//    cerr << "FFMpegDemuxer: " << streamIndex << ": " << pPacket->dts*timeBase << endl;
     return pPacket;
 }
-
-void FFMpegDemuxer::seek(double destTime)
+        
+void FFMpegDemuxer::seek(float destTime)
 {
 #if LIBAVFORMAT_BUILD <= 4616
     av_seek_frame(m_pFormatContext, -1, destTime*1000000);
@@ -103,16 +119,11 @@ void FFMpegDemuxer::seek(double destTime)
 #if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
     av_seek_frame(m_pFormatContext, -1, destTime*1000000, 0);
 #else
-    av_seek_frame(m_pFormatContext, -1, (long long)(destTime*AV_TIME_BASE), AVSEEK_FLAG_BACKWARD);
+   av_seek_frame(m_pFormatContext, -1, (long long)(destTime*AV_TIME_BASE),
+            AVSEEK_FLAG_BACKWARD);
 #endif
 #endif
     clearPacketCache();
-    map<int, PacketList>::iterator it;
-    for (it = m_PacketLists.begin(); it != m_PacketLists.end(); ++it) {
-        int CurStreamIndex = it->first;
-        AVStream * pStream = m_pFormatContext->streams[CurStreamIndex];
-        avcodec_flush_buffers(pStream->codec);
-    }
 }
 
 void FFMpegDemuxer::clearPacketCache()
@@ -120,12 +131,12 @@ void FFMpegDemuxer::clearPacketCache()
     map<int, PacketList>::iterator it;
     for (it = m_PacketLists.begin(); it != m_PacketLists.end(); ++it) {
         PacketList::iterator it2;
-        PacketList* thePacketList = &(it->second);
-        for (it2 = thePacketList->begin(); it2 != thePacketList->end(); ++it2) {
+        PacketList* pPacketList = &(it->second);
+        for (it2 = pPacketList->begin(); it2 != pPacketList->end(); ++it2) {
             av_free_packet(*it2);
             delete *it2;
         }
-        thePacketList->clear();
+        pPacketList->clear();
     }
 }
 
