@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -19,21 +19,16 @@
 //  Current versions can be found at www.libavg.de
 //
 
-#include "FFMpegDecoder.h"
-
-#ifdef _WIN32
-#pragma warning(push)
-#pragma warning(disable:4251)
-#endif
-#ifdef _WIN32
-#pragma warning(pop)
-#endif
-
 #include "AsyncVideoDecoder.h"
+#include "SyncVideoDecoder.h"
+#ifdef AVG_ENABLE_VDPAU
+#include "VDPAUDecoder.h"
+#endif
 
 #include "../graphics/Filterfliprgba.h"
 #include "../graphics/Filterfliprgb.h"
 #include "../graphics/GraphicsTest.h"
+#include "../graphics/BitmapLoader.h"
 
 #include "../base/StringHelper.h"
 #include "../base/TimeSource.h"
@@ -47,25 +42,26 @@
 #include <sstream>
 #include <cmath>
 
+#include <glib-object.h>
+
 using namespace avg;
 using namespace std;
-
+using namespace boost;
 
 class DecoderTest: public GraphicsTest {
     public:
-        DecoderTest(const string& sClassName, bool bThreadedDecoder, 
-                bool bThreadedDemuxer, bool bUseHardwareAcceleration)
-          : GraphicsTest(sClassName+getDecoderName(bThreadedDecoder, 
-                    bThreadedDemuxer), 2),
-            m_bThreadedDecoder(bThreadedDecoder),
-            m_bThreadedDemuxer(bThreadedDemuxer),
+        DecoderTest(const string& sClassName, bool bThreaded, 
+                bool bUseHardwareAcceleration)
+          : GraphicsTest(sClassName+getDecoderName(bThreaded, bUseHardwareAcceleration),
+                2),
+            m_bThreaded(bThreaded),
             m_bUseHardwareAcceleration(bUseHardwareAcceleration)
         {}
 
     protected:
-        bool isDemuxerThreaded() 
+        bool isThreaded() 
         {
-            return m_bThreadedDemuxer;
+            return m_bThreaded;
         }
 
         bool useHardwareAcceleration()
@@ -76,16 +72,13 @@ class DecoderTest: public GraphicsTest {
         VideoDecoderPtr createDecoder() 
         {
             VideoDecoderPtr pDecoder;
-            pDecoder = VideoDecoderPtr(new FFMpegDecoder());
-            if (m_bThreadedDecoder) {
-                pDecoder = VideoDecoderPtr(new AsyncVideoDecoder(pDecoder, 8));
+            if (m_bThreaded) {
+                pDecoder = VideoDecoderPtr(new AsyncVideoDecoder(8));
+            } else {
+                pDecoder = VideoDecoderPtr(new SyncVideoDecoder());
             }
-            return pDecoder;
-        }
 
-        AudioBufferPtr createAudioBuffer(int numFrames)
-        {
-            return AudioBufferPtr(new AudioBuffer(numFrames, *getAudioParams()));
+            return pDecoder;
         }
 
         const AudioParams* getAudioParams()
@@ -94,54 +87,99 @@ class DecoderTest: public GraphicsTest {
             return &AP;
         }
 
+        int processAudioMsg(AudioMsgQueuePtr pMsgQ, AudioMsgQueuePtr pStatusQ)
+        {
+            AudioMsgPtr pMsg = pMsgQ->pop(false);
+            if (pMsg) {
+                switch (pMsg->getType()) {
+                    case AudioMsg::AUDIO: {
+                        AudioBufferPtr pBuffer = pMsg->getAudioBuffer();
+                        AudioMsgPtr pStatusMsg(new AudioMsg);
+                        pStatusMsg->setAudioTime(pMsg->getAudioTime());
+                        pStatusQ->push(AudioMsgPtr(pStatusMsg));
+                        return pBuffer->getNumFrames();
+                    }
+                    case AudioMsg::SEEK_DONE: {
+                        AudioMsgPtr pStatusMsg(new AudioMsg);
+                        pStatusMsg->setSeekDone(pMsg->getSeekSeqNum(),
+                                pMsg->getSeekTime());
+                        pStatusQ->push(AudioMsgPtr(pStatusMsg));
+                        return -1;
+                    }
+                    default:
+                        pStatusQ->push(pMsg);
+                        return 0;
+                }
+            } else {
+                return 0;
+            }
+        }
+
+        void processAudioSeek(AudioMsgQueuePtr pMsgQ, AudioMsgQueuePtr pStatusQ)
+        {
+            int framesDecoded = 0;
+            while (framesDecoded != -1) {
+                // The real AudioSource blocks on pMsgQ->pop()
+                msleep(10);
+                framesDecoded = processAudioMsg(pMsgQ, pStatusQ);
+            }
+        }
+
         virtual void testEqual(Bitmap& resultBmp, const std::string& sFName, 
-                avg::PixelFormat pf = NO_PIXELFORMAT, double maxAverage=1.0,
-                double maxStdDev=1.0)
+                avg::PixelFormat pf = NO_PIXELFORMAT, float maxAverage=1.0,
+                float maxStdDev=1.0)
         {
             GraphicsTest::testEqual(resultBmp, sFName, pf, maxAverage, maxStdDev);
         }
 
         void testEqual(Bitmap& resultBmp, Bitmap& BaselineBmp,
-                const std::string& sFName, double maxAverage=1.0, double maxStdDev=1.0)
+                const std::string& sFName, float maxAverage=1.0f, float maxStdDev=1.0f)
         {
             GraphicsTest::testEqual(resultBmp, BaselineBmp, sFName, 
                     maxAverage, maxStdDev);
         }
 
-    private:
-        string getDecoderName(bool bThreadedDecoder, bool bThreadedDemuxer)
+        string getMediaLoc(const string& sFilename)
         {
-            string sName = "(";
-            if (bThreadedDecoder) {
-                sName += "Threaded decoder, ";
-            } else {
-                sName += "Sync decoder, ";
-            }
-            if (bThreadedDemuxer) {
-                return sName+string("Threaded demuxer)");
-            } else {
-                return sName+string("Sync demuxer)");
-            }
+            return getSrcDirName()+"../test/media/"+sFilename;
         }
 
-        bool m_bThreadedDecoder;
-        bool m_bThreadedDemuxer;
+    private:
+        string getDecoderName(bool bThreaded, bool bUseHardwareAcceleration)
+        {
+            string sName = "(";
+            if (bThreaded) {
+                sName += "Threaded";
+            } else {
+                sName += "Sync";
+            }
+            if (bUseHardwareAcceleration) {
+                sName += ", VDPAU)";
+            } else {
+                sName += ")";
+            }
+            return sName;
+        }
+
+        bool m_bThreaded;
         bool m_bUseHardwareAcceleration;
 };
 
 class VideoDecoderTest: public DecoderTest {
     public:
-        VideoDecoderTest(bool bThreadedDecoder, bool bThreadedDemuxer,
-                bool bUseHardwareAcceleration)
-            : DecoderTest("VideoDecoderTest", bThreadedDecoder, bThreadedDemuxer,
-                      bUseHardwareAcceleration)
+        VideoDecoderTest(bool bThreaded, bool bUseHardwareAcceleration)
+            : DecoderTest("VideoDecoderTest", bThreaded, bUseHardwareAcceleration)
         {}
 
         void runTests()
         {
-            basicFileTest("mpeg1-48x48.mpg", 30);
+            basicFileTest("mpeg1-48x48.mov", 30);
+#ifndef AVG_ENABLE_RPI
             basicFileTest("mjpeg-48x48.avi", 202);
             testSeeks("mjpeg-48x48.avi");
+#else
+            cerr << "Skipping mjpeg tests: SW decoding too slow on RaspberryPi." << endl;
+#endif
         }
 
     private:
@@ -151,12 +189,13 @@ class VideoDecoderTest: public DecoderTest {
                 cerr << "    Testing " << sFilename << endl;
 
                 VideoDecoderPtr pDecoder = createDecoder();
-                pDecoder->open(getSrcDirName()+"testfiles/"+sFilename, 
-                        isDemuxerThreaded(), useHardwareAcceleration());
+                pDecoder->open(getMediaLoc(sFilename), useHardwareAcceleration(), true);
                 IntPoint frameSize = pDecoder->getSize();
                 TEST(frameSize == IntPoint(48, 48));
                 TEST(pDecoder->getVideoInfo().m_bHasVideo);
-                TEST(pDecoder->getNominalFPS() != 0);
+                TEST(pDecoder->getVideoInfo().m_Duration != 0);
+                TEST(pDecoder->getStreamFPS() != 0);
+                TEST(pDecoder->getFPS() != 0);
                 pDecoder->startDecoding(false, getAudioParams());
                 TEST(pDecoder->getPixelFormat() == B8G8R8X8);
                 BitmapPtr pBmp(new Bitmap(frameSize, B8G8R8X8));
@@ -164,6 +203,9 @@ class VideoDecoderTest: public DecoderTest {
                 // Test first two frames.
                 pDecoder->renderToBmp(pBmp, -1);
                 testEqual(*pBmp, sFilename+"_1", B8G8R8X8);
+                TEST(pDecoder->getCurFrame() == 0);
+                TEST(pDecoder->getCurTime() == 0);
+
                 pDecoder->renderToBmp(pBmp, -1);
                 testEqual(*pBmp, sFilename+"_2", B8G8R8X8);
                 pDecoder->close();
@@ -182,8 +224,7 @@ class VideoDecoderTest: public DecoderTest {
             cerr << "    Testing " << sFilename << " (seek)" << endl;
 
             VideoDecoderPtr pDecoder = createDecoder();
-            pDecoder->open(getSrcDirName()+"testfiles/"+sFilename, isDemuxerThreaded(),
-                    useHardwareAcceleration());
+            pDecoder->open(getMediaLoc(sFilename), useHardwareAcceleration(), true);
             pDecoder->startDecoding(false, getAudioParams());
 
             // Seek forward
@@ -201,25 +242,24 @@ class VideoDecoderTest: public DecoderTest {
             IntPoint frameSize = pDecoder->getSize();
 
             BitmapPtr pBmp(new Bitmap(frameSize, B8G8R8X8));
-            pDecoder->seek(double(frameNum)/pDecoder->getNominalFPS());
+            pDecoder->seek(float(frameNum)/pDecoder->getStreamFPS());
             pDecoder->renderToBmp(pBmp, -1);
             testEqual(*pBmp, sFilename+"_"+toString(frameNum), B8G8R8X8);
 
         }
 
-        void readWholeFile(const string& sFilename, double speedFactor, 
+        void readWholeFile(const string& sFilename, float speedFactor, 
                 int expectedNumFrames)
         {
             // Read whole file, test last image.
             VideoDecoderPtr pDecoder = createDecoder();
-            pDecoder->open(getSrcDirName()+"testfiles/"+sFilename, isDemuxerThreaded(),
-                    useHardwareAcceleration());
+            pDecoder->open(getMediaLoc(sFilename), useHardwareAcceleration(), true);
             IntPoint frameSize = pDecoder->getSize();
-            double timePerFrame = (1.0/pDecoder->getFPS())*speedFactor;
+            float timePerFrame = (1.0f/pDecoder->getFPS())*speedFactor;
             pDecoder->startDecoding(false, getAudioParams());
             BitmapPtr pBmp(new Bitmap(frameSize, B8G8R8X8));
             int numFrames = 0;
-            double curTime = 0;
+            float curTime = 0;
 
             while (!pDecoder->isEOF()) {
                 FrameAvailableCode frameAvailable = pDecoder->renderToBmp(pBmp, curTime);
@@ -241,7 +281,9 @@ class VideoDecoderTest: public DecoderTest {
 //            cerr << "numFrames: " << numFrames << 
 //                    ", expectedNumFrames: " << expectedNumFrames << endl;
             TEST(numFrames == expectedNumFrames);
-            if (speedFactor == 1) {
+            if (speedFactor == 1 && !useHardwareAcceleration()) {
+                // The last frame is broken with VDPAU sometimes. Not sure why this is,
+                // possibly a libav bug.
                 testEqual(*pBmp, sFilename+"_end", B8G8R8X8);
             }
             
@@ -257,12 +299,13 @@ class VideoDecoderTest: public DecoderTest {
 
 class AudioDecoderTest: public DecoderTest {
     public:
-        AudioDecoderTest(bool bThreadedDecoder, bool bThreadedDemuxer)
-          : DecoderTest("AudioDecoderTest", bThreadedDecoder, bThreadedDemuxer, true)
+        AudioDecoderTest()
+          : DecoderTest("AudioDecoderTest", true, false)
         {}
 
         void runTests()
         {
+
             testOneFile("22.050Hz_16bit_mono.wav");
 
             testOneFile("44.1kHz_16bit_mono.wav");
@@ -287,38 +330,36 @@ class AudioDecoderTest: public DecoderTest {
                 
                 {
                     cerr << "      Reading complete file." << endl;
-                    VideoDecoderPtr pDecoder = createDecoder();
-                    pDecoder->open(getSrcDirName()+"testfiles/"+sFilename, 
-                            isDemuxerThreaded(), useHardwareAcceleration());
+                    AsyncVideoDecoderPtr pDecoder = 
+                            dynamic_pointer_cast<AsyncVideoDecoder>(createDecoder());
+                    pDecoder->open(getMediaLoc(sFilename), useHardwareAcceleration(), true);
                     TEST(pDecoder->getVideoInfo().m_bHasAudio);
-                    pDecoder->setVolume(0.5);
-                    TEST(pDecoder->getVolume() == 0.5);
                     pDecoder->startDecoding(false, getAudioParams());
+                    AudioMsgQueuePtr pMsgQ = pDecoder->getAudioMsgQ();
+                    AudioMsgQueuePtr pStatusQ = pDecoder->getAudioStatusQ();
                     int totalFramesDecoded = 0;
-                    bool bCheckTimestamps = (sFilename.find(".ogg") == string::npos &&
-                            sFilename.find(".mp3") == string::npos);
-                    readAudioToEOF(pDecoder, totalFramesDecoded, bCheckTimestamps);
+                    readAudioToEOF(pDecoder, pMsgQ, pStatusQ, totalFramesDecoded, true);
 
                     // Check if we've decoded the whole file.
                     int framesInDuration = int(pDecoder->getVideoInfo().m_Duration*44100);
 //                    cerr << "framesInDuration: " << framesInDuration << endl;
+//                    cerr << "framesDecoded: " << totalFramesDecoded << endl;
                     TEST(abs(totalFramesDecoded-framesInDuration) < 65);
                 }
                 {
                     cerr << "      Seek test." << endl;
-                    VideoDecoderPtr pDecoder = createDecoder();
-                    pDecoder->open(getSrcDirName()+"testfiles/"+sFilename,
-                            isDemuxerThreaded(), useHardwareAcceleration());
-                    double duration = pDecoder->getVideoInfo().m_Duration;
+                    AsyncVideoDecoderPtr pDecoder = 
+                            dynamic_pointer_cast<AsyncVideoDecoder>(createDecoder());
+                    pDecoder->open(getMediaLoc(sFilename), useHardwareAcceleration(), true);
+                    float duration = pDecoder->getVideoInfo().m_Duration;
                     pDecoder->startDecoding(false, getAudioParams());
+                    AudioMsgQueuePtr pMsgQ = pDecoder->getAudioMsgQ();
+                    AudioMsgQueuePtr pStatusQ = pDecoder->getAudioStatusQ();
                     pDecoder->seek(duration/2);
-                    AudioBufferPtr pAudioBuffer = createAudioBuffer(4);
-                    pDecoder->fillAudioBuffer(pAudioBuffer);
-                    // 60 ms accuracy for seeks.
-                    TEST(abs(duration/2-pDecoder->getCurTime(SS_AUDIO)) < 0.06); 
-                    int totalFramesDecoded = 4;
+                    processAudioSeek(pMsgQ, pStatusQ);
+                    int totalFramesDecoded = 0;
 
-                    readAudioToEOF(pDecoder, totalFramesDecoded, false);
+                    readAudioToEOF(pDecoder, pMsgQ, pStatusQ, totalFramesDecoded, false);
                     if (sFilename.find(".mp3") == string::npos) {
                         // Check if we've decoded half the file.
                         // TODO: Find out why there are problems with this
@@ -329,7 +370,6 @@ class AudioDecoderTest: public DecoderTest {
 //                        cerr << "framesInDuration: " << framesInDuration << endl;
                         TEST(abs(totalFramesDecoded-framesInDuration/2) < 65);
                     }
-
                 }
 
             } catch (Exception & ex) {
@@ -338,24 +378,24 @@ class AudioDecoderTest: public DecoderTest {
             }
         }
 
-        void readAudioToEOF(VideoDecoderPtr pDecoder, int& totalFramesDecoded,
+        void readAudioToEOF(AsyncVideoDecoderPtr pDecoder, AudioMsgQueuePtr pMsgQ, 
+                AudioMsgQueuePtr pStatusQ, int& totalFramesDecoded,
                 bool bCheckTimestamps) 
         {
             int numWrongTimestamps = 0;
             while (!pDecoder->isEOF()) {
-                AudioBufferPtr pBuffer = createAudioBuffer(256);
                 int framesDecoded = 0;
                 while (framesDecoded == 0 && !pDecoder->isEOF()) {
-                    framesDecoded = pDecoder->fillAudioBuffer(pBuffer);
-//                    cerr << "framesDecoded: " << framesDecoded << endl;
+                    framesDecoded = processAudioMsg(pMsgQ, pStatusQ);
+                    AVG_ASSERT(framesDecoded != -1);
+                    pDecoder->updateAudioStatus();
                     msleep(0);
                 }
                 totalFramesDecoded += framesDecoded;
-                double curTime = double(totalFramesDecoded)/44100;
-                if (abs(curTime-pDecoder->getCurTime(SS_AUDIO)) > 0.02) {
+                float curTime = float(totalFramesDecoded)/44100;
+                if (abs(curTime-pDecoder->getCurTime()) > 0.02f) {
                     numWrongTimestamps++;
                 }
-//                cerr << curTime << "->" << pDecoder->getCurTime(SS_AUDIO) << endl;
             }
             if (bCheckTimestamps) {
                 if (numWrongTimestamps>0) {
@@ -365,12 +405,11 @@ class AudioDecoderTest: public DecoderTest {
         }
 };
 
+
 class AVDecoderTest: public DecoderTest {
     public:
-        AVDecoderTest(bool bThreadedDecoder, bool bThreadedDemuxer,
-                bool bUseHardwareAcceleration)
-          : DecoderTest("AVDecoderTest", bThreadedDecoder, bThreadedDemuxer,
-                    bUseHardwareAcceleration)
+        AVDecoderTest(bool bUseHardwareAcceleration)
+          : DecoderTest("AVDecoderTest", true, bUseHardwareAcceleration)
         {}
 
         void runTests()
@@ -382,19 +421,23 @@ class AVDecoderTest: public DecoderTest {
         void basicFileTest(const string& sFilename, int expectedNumFrames)
         {
             VideoDecoderPtr pDecoder = createDecoder();
-            pDecoder->open(getSrcDirName()+"testfiles/"+sFilename, isDemuxerThreaded(),
-                    useHardwareAcceleration());
+            pDecoder->open(getMediaLoc(sFilename), useHardwareAcceleration(), true);
             TEST(pDecoder->getVideoInfo().m_bHasVideo);
-            TEST(pDecoder->getNominalFPS() != 0);
+            TEST(pDecoder->getStreamFPS() != 0);
             pDecoder->startDecoding(false, getAudioParams());
-            if (isDemuxerThreaded()) {
-                TEST(pDecoder->getVideoInfo().m_bHasAudio);
-            }
+            AudioMsgQueuePtr pMsgQ;
+            AudioMsgQueuePtr pStatusQ;
+            
+            pMsgQ = dynamic_pointer_cast<AsyncVideoDecoder>(pDecoder) ->getAudioMsgQ();
+            pStatusQ = dynamic_pointer_cast<AsyncVideoDecoder>(pDecoder)
+                ->getAudioStatusQ();
+            TEST(pDecoder->getVideoInfo().m_bHasAudio);
+            
             IntPoint frameSize = pDecoder->getSize();
             BitmapPtr pBmp(new Bitmap(frameSize, B8G8R8X8));
             int numFrames = 0;
             int totalFramesDecoded = 0;
-            double curTime = 0;
+            float curTime = 0;
             
             while (!pDecoder->isEOF()) {
                 FrameAvailableCode frameAvailable;
@@ -404,38 +447,28 @@ class AVDecoderTest: public DecoderTest {
                 } while (frameAvailable == FA_STILL_DECODING);
                 if (frameAvailable == FA_NEW_FRAME) {
 //                    stringstream ss;
-//                    ss << "testfiles/result/" << sFilename << numFrames << ".png";
+//                    ss << sFilename << numFrames << ".png";
 //                    pBmp->save(ss.str());
                     numFrames++;
                 }
-                if (isDemuxerThreaded()) {
-                    AudioBufferPtr pBuffer = createAudioBuffer(256);
-                    int framesDecoded = 0;
-                    while (framesDecoded == 0 && !pDecoder->isEOF(SS_AUDIO)) {
-                        framesDecoded = pDecoder->fillAudioBuffer(pBuffer);
-                        msleep(0);
-                    }
-                    totalFramesDecoded += framesDecoded;
-//                    cerr << "framesDecoded: " << framesDecoded << endl;
+                int framesDecoded = 0;
+                while (framesDecoded == 0 && !pDecoder->isEOF()) {
+                    framesDecoded = processAudioMsg(pMsgQ, pStatusQ);
+                    dynamic_pointer_cast<AsyncVideoDecoder>(pDecoder)
+                        ->updateAudioStatus();
+                    msleep(0);
                 }
-                curTime += 1.0/pDecoder->getFPS();
+                totalFramesDecoded += framesDecoded;
+
+                curTime += 1.0f/pDecoder->getFPS();
             }
-            TEST(pDecoder->isEOF(SS_VIDEO));
-//            cerr << "numFrames: " << numFrames << endl;
+            TEST(pDecoder->isEOF());
             TEST(numFrames == expectedNumFrames);
             testEqual(*pBmp, sFilename+"_end", B8G8R8X8);
 
-            if (isDemuxerThreaded()) {
-                // Check if audio length was ok.
-                // TODO: Currently, getDuration() is the duration of the video stream.
-                // This causes the test to fail.
-                //int framesInDuration = int(pDecoder->getDuration()*44100);
-                //cerr << "framesDecoded: " << totalFramesDecoded << ", framesInDuration: " << framesInDuration << endl;
-                //TEST (abs(totalFramesDecoded-framesInDuration) < 45);
-            }
-
             // Test loop.
             pDecoder->seek(0);
+            processAudioSeek(pMsgQ, pStatusQ);
             pDecoder->renderToBmp(pBmp, -1);
             testEqual(*pBmp, sFilename+"_loop", B8G8R8X8);
 
@@ -452,70 +485,48 @@ public:
         addAudioTests();
         addVideoTests(false);
 #ifdef AVG_ENABLE_VDPAU
-        if (VDPAU::isAvailable()) {
+        if (VDPAUDecoder::isAvailable()) {
             addVideoTests(true);
+        } else {
+            cerr << "Skipping VDPAU tests: VDPAU configured but not available." << endl;
         }
+#else
+        cerr << "Skipping VDPAU tests: VDPAU not configured." << endl;
 #endif
     }
 private:
 
     void addAudioTests()
     {
-        addTest(TestPtr(new AudioDecoderTest(false, true)));
-        addTest(TestPtr(new AudioDecoderTest(true, true)));
+        addTest(TestPtr(new AudioDecoderTest()));
     }
 
     void addVideoTests(bool bUseHardwareAcceleration)
     {
-        addTest(TestPtr(new VideoDecoderTest(false, false, bUseHardwareAcceleration)));
-        addTest(TestPtr(new VideoDecoderTest(false, true, bUseHardwareAcceleration)));
-        addTest(TestPtr(new VideoDecoderTest(true, true, bUseHardwareAcceleration)));
+        addTest(TestPtr(new VideoDecoderTest(false, bUseHardwareAcceleration)));
+        addTest(TestPtr(new VideoDecoderTest(true, bUseHardwareAcceleration)));
 
-        addTest(TestPtr(new AVDecoderTest(false, false, bUseHardwareAcceleration)));
-        addTest(TestPtr(new AVDecoderTest(false, true, bUseHardwareAcceleration)));
-        addTest(TestPtr(new AVDecoderTest(true, true, bUseHardwareAcceleration)));
+        addTest(TestPtr(new AVDecoderTest(bUseHardwareAcceleration)));
     }
 };
 
 
-void deleteOldResultImages() 
-{
-    string sDirName("resultimages");
-    Directory dir(sDirName);
-    int err = dir.open(true);
-    if (err) {
-        cerr << "Creating directory " << sDirName << " failed." << strerror(err) << endl;
-    } else {
-        cerr << "Deleting files in " << sDirName << endl;
-        DirEntryPtr pEntry = dir.getNextEntry();
-        while (pEntry) {
-            if (pEntry->getName()[0] != '.') {
-                pEntry->remove();
-            }
-            pEntry = dir.getNextEntry();
-        }
-    }
-}
-
 int main(int nargs, char** args)
 {
-    ThreadProfilerPtr pProfiler = ThreadProfiler::get();
+    ThreadProfiler* pProfiler = ThreadProfiler::get();
     pProfiler->setName("main");
 
-    deleteOldResultImages();
+    GraphicsTest::createResultImgDir();
 
+    BitmapLoader::init(true);
     VideoTestSuite suite;
     bool bOk;
     
     suite.runTests();
     bOk = suite.isOk();
-    if (bOk) {
-        return 0;
-    } else {
-        return 1;
-    }
 /*    
-    while(true) {
+    for (int i=0; i<300; ++i) {
+//    while(true) {
         suite.runTests();
         bOk = suite.isOk();
         if (!bOk) {
@@ -523,5 +534,10 @@ int main(int nargs, char** args)
         }
     }
 */    
+    if (bOk) {
+        return 0;
+    } else {
+        return 1;
+    }
 }
 
