@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,11 +22,15 @@
 #include "WrapHelper.h"
 
 #include "../player/BoostPython.h"
+#include "../player/BitmapManager.h"
 
 #include "../graphics/Bitmap.h"
-#include "../graphics/BitmapManager.h"
+#include "../graphics/BitmapLoader.h"
+#include "../graphics/FilterResizeBilinear.h"
 
-#include "../base/Point.h"
+#include "../base/CubicSpline.h"
+
+#include "../glm/gtx/vector_angle.hpp"
 
 #include <vector>
 #include <sstream>
@@ -39,17 +43,15 @@ template<class POINT>
 class_<POINT> export_point(const string& sName)
 {
     return class_<POINT>(sName.c_str(), no_init)
-        .def("__len__", &DPointHelper::len)
-        .def("__getitem__", &DPointHelper::getItem)
-        .def("__str__", &DPointHelper::str)
-        .def("__repr__", &DPointHelper::repr)
-        .def("__hash__", &DPointHelper::getHash)
-        .def("getNormalized", &DPoint::safeGetNormalized)
-        .def("getNorm", &DPoint::getNorm)
-        .def("getRotated", &DPoint::getRotated)
-        .def("getRotated", &DPoint::getRotatedPivot)
-        .def("isNaN", &DPoint::isNaN)
-        .def("isInf", &DPoint::isInf)
+        .def("__len__", &Vec2Helper::len)
+        .def("__getitem__", &Vec2Helper::getItem)
+        .def("__str__", &Vec2Helper::str)
+        .def("__repr__", &Vec2Helper::repr)
+        .def("__hash__", &Vec2Helper::getHash)
+        .def("getNormalized", &Vec2Helper::safeGetNormalized)
+        .def("getNorm", &Vec2Helper::getNorm)
+        .def("getRotated", &getRotated)
+        .def("getRotated", &getRotatedPivot)
         .def(self == self)
         .def(self != self)
         .def(-self)
@@ -58,10 +60,10 @@ class_<POINT> export_point(const string& sName)
         .def(float() * self)
         .def(self * float())
         .def(self / float())
-        .def("getAngle", &DPoint::getAngle)
-        .def("fromPolar", &DPoint::fromPolar)
+        .def("getAngle", &getAngle)
+        .def("fromPolar", &fromPolar)
         .staticmethod("fromPolar")
-        .def("angle", &vecAngle)
+        .def("angle", &Vec2Helper::vecAngle)
         .staticmethod("angle")
     ;
 }
@@ -75,33 +77,63 @@ struct Pixel32_to_python_tuple
     }
 };
 
-ConstDPoint Bitmap_getSize(Bitmap* This)
+ConstVec2 Bitmap_getSize(Bitmap* This)
 {
-    return (DPoint)(This->getSize());
+    return (glm::vec2)(This->getSize());
 }
 
-DPoint* createPoint()
+BitmapPtr Bitmap_getResized(BitmapPtr This, const glm::vec2& size)
 {
-    return new DPoint(0,0);
+    return FilterResizeBilinear(IntPoint(size)).apply(This);
 }
 
+glm::vec2* createPoint()
+{
+    return new glm::vec2(0,0);
+}
+
+BitmapPtr createBitmapFromFile(const UTF8String& sFName)
+{
+    return loadBitmap(sFName);
+}
+
+BitmapPtr createBitmapWithRect(BitmapPtr pBmp,
+        const glm::vec2& tlPos, const glm::vec2& brPos)
+{
+    if (tlPos.x >= brPos.x || tlPos.y >= brPos.y) {
+        throw Exception(AVG_ERR_OUT_OF_RANGE, 
+                "Can't create a bitmap with zero or negative width/height.");
+    }
+    IntPoint size = pBmp->getSize();
+    if (tlPos.x < 0 || tlPos.y < 0 || brPos.x > size.x || brPos.y > size.y) {
+        throw Exception(AVG_ERR_OUT_OF_RANGE, 
+                "Attempt to create a subbitmap that doesn't fit into the parent bitmap.");
+    }
+    IntRect rect = IntRect(IntPoint(tlPos), IntPoint(brPos));
+    return BitmapPtr(new Bitmap(*pBmp, rect));
+}
+
+BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(loadBitmap_overloads, BitmapManager::loadBitmapPy, 
+        2, 3);
+
 void export_bitmap()
 {
-    export_point<DPoint>("Point2D")
+    export_point<glm::vec2>("Point2D")
         .def("__init__", make_constructor(createPoint))
-        .def(init<double, double>())
-        .def(init<const DPoint&>())
-        .def("__setitem__", &DPointHelper::setItem)
-        .add_property("x", &DPointHelper::getX, &DPointHelper::setX,"")
-        .add_property("y", &DPointHelper::getY, &DPointHelper::setY,"")
-    ;
-    export_point<ConstDPoint>("ConstPoint2D")
-        .add_property("x", &DPointHelper::getX, "")
-        .add_property("y", &DPointHelper::getY, "")
+        .def(init<float, float>())
+        .def(init<const glm::vec2&>())
+        .def("__setitem__", &Vec2Helper::setItem)
+        .add_property("x", &Vec2Helper::getX, &Vec2Helper::setX,"")
+        .add_property("y", &Vec2Helper::getY, &Vec2Helper::setY,"")
     ;
 
-    implicitly_convertible<ConstDPoint, DPoint>();
-    implicitly_convertible<DPoint, ConstDPoint>();
+    export_point<ConstVec2>("ConstPoint2D")
+        .add_property("x", &Vec2Helper::getX, "")
+        .add_property("y", &Vec2Helper::getY, "")
+    ;
+
+    implicitly_convertible<ConstVec2, glm::vec2>();
+    implicitly_convertible<glm::vec2, ConstVec2>();
 
     enum_<PixelFormat>("pixelformat")
         .value("B5G6R5", B5G6R5)
@@ -123,6 +155,7 @@ void export_bitmap()
         .value("YCbCr422", YCbCr422)
         .value("YUYV422", YUYV422)
         .value("YCbCr420p", YCbCr420p)
+        .value("YCbCrJ420p", YCbCrJ420p)
         .value("YCbCrA420p", YCbCrA420p)
         .value("BAYER8", BAYER8)
         .value("BAYER8_RGGB", BAYER8_RGGB)
@@ -133,12 +166,17 @@ void export_bitmap()
         .value("I32F", I32F)
         .export_values();
 
+    def("getSupportedPixelFormats", &getSupportedPixelFormats);
+
     to_python_converter<Pixel32, Pixel32_to_python_tuple>();
 
     class_<Bitmap, boost::shared_ptr<Bitmap> >("Bitmap", no_init)
-        .def(init<DPoint, PixelFormat, UTF8String>())
+        .def(init<glm::vec2, PixelFormat, UTF8String>())
         .def(init<Bitmap>())
-        .def(init<UTF8String>())
+        .def("__init__", make_constructor(createBitmapWithRect))
+        .def("__init__", make_constructor(createBitmapFromFile))
+        .def("blt", &Bitmap::blt)
+        .def("getResized", &Bitmap_getResized)
         .def("save", &Bitmap::save)
         .def("getSize", &Bitmap_getSize)
         .def("getFormat", &Bitmap::getPixelFormat)
@@ -157,6 +195,13 @@ void export_bitmap()
         .def("get", &BitmapManager::get,
                 return_value_policy<reference_existing_object>())
         .staticmethod("get")
-        .def("loadBitmap", &BitmapManager::loadBitmap)
+        .def("loadBitmap", &BitmapManager::loadBitmapPy, loadBitmap_overloads())
+        .def("setNumThreads", &BitmapManager::setNumThreads)
+    ;
+
+    class_<CubicSpline, boost::noncopyable>("CubicSpline", no_init)
+        .def(init<const vector<glm::vec2>&>())
+        .def(init<const vector<glm::vec2>&, bool>())
+        .def("interpolate", &CubicSpline::interpolate)
     ;
 }
