@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -42,8 +42,7 @@ void export_fx()
         ;
 
     class_<BlurFXNode, bases<FXNode>, boost::shared_ptr<BlurFXNode>,
-            boost::noncopyable>("BlurFXNode")
-        .def("setParam", &BlurFXNode::setParam)
+            boost::noncopyable>("BlurFXNode", init<optional<float> >())
         .add_property("radius", &BlurFXNode::getRadius,
                 &BlurFXNode::setRadius)
         ;
@@ -69,7 +68,7 @@ void export_fx()
         ;
 
     class_<HueSatFXNode, bases<FXNode>, boost::shared_ptr<HueSatFXNode>,
-            boost::noncopyable > ("HueSatFXNode", init< optional<float, float, float,
+            boost::noncopyable > ("HueSatFXNode", init<optional<float, float, float,
             bool> >())
         .add_property("hue", &HueSatFXNode::getHue,
                 &HueSatFXNode::setHue)
@@ -82,8 +81,8 @@ void export_fx()
         .def("__repr__", &HueSatFXNode::toString)
         ;
 
-    class_<InvertFXNode, bases<FXNode>, boost::shared_ptr<InvertFXNode>, boost::noncopyable>(
-            "InvertFXNode")
+    class_<InvertFXNode, bases<FXNode>, boost::shared_ptr<InvertFXNode>, 
+            boost::noncopyable>("InvertFXNode")
         ;
 
     class_<NullFXNode, bases<FXNode>, boost::shared_ptr<NullFXNode>, boost::noncopyable>(
@@ -91,8 +90,8 @@ void export_fx()
         ;
 
     class_<ShadowFXNode, bases<FXNode>, boost::shared_ptr<ShadowFXNode>,
-            boost::noncopyable>("ShadowFXNode")
-        .def("setParams", &ShadowFXNode::setParams)
+            boost::noncopyable>("ShadowFXNode", 
+            init<optional<glm::vec2, float, float, std::string> >())
         .add_property("offset", &ShadowFXNode::getOffset, &ShadowFXNode::setOffset)
         .add_property("radius", &ShadowFXNode::getRadius, &ShadowFXNode::setRadius)
         .add_property("opacity", &ShadowFXNode::getOpacity, &ShadowFXNode::setOpacity)
