@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,7 @@
 #include "../player/CameraNode.h"
 #include "../player/ImageNode.h"
 #include "../player/VideoNode.h"
+#include "../player/FontStyle.h"
 #include "../player/WordsNode.h"
 
 using namespace boost::python;
@@ -34,12 +35,11 @@ using namespace std;
 char imageNodeName[] = "image";
 char cameraNodeName[] = "camera";
 char videoNodeName[] = "video";
+char fontStyleName[] = "fontstyle";
 char wordsNodeName[] = "words";
 
 void export_raster()
 {
-    to_python_converter<VertexGrid, to_list<VertexGrid> >();    
-    from_python_sequence<VertexGrid, variable_capacity_policy>();
 
     class_<RasterNode, bases<AreaNode>, boost::noncopyable>("RasterNode", no_init) 
         .def("getOrigVertexCoords", &RasterNode::getOrigVertexCoords)
@@ -108,11 +108,44 @@ void export_raster()
         .def("setWhitebalance", &CameraNode::setWhitebalance)
         .def("doOneShotWhitebalance", &CameraNode::doOneShotWhitebalance)
         .def("isAvailable", &CameraNode::isAvailable)
-        .def("dumpCameras", make_function(&CameraNode::dumpCameras))
-        .staticmethod("dumpCameras")
+        .def("getCamerasInfos", make_function(&CameraNode::getCamerasInfos))
+        .staticmethod("getCamerasInfos")
         .def("resetFirewireBus", &CameraNode::resetFirewireBus)
         .staticmethod("resetFirewireBus")
     ;
+
+    //Wrap std::vector from CameraInfo to Pyhton list
+    to_python_converter<CamerasInfosVector, to_list<CamerasInfosVector> >();
+    from_python_sequence<CamerasInfosVector, variable_capacity_policy>();
+
+    to_python_converter<CameraImageFormatsVector, to_list<CameraImageFormatsVector> >();
+    from_python_sequence<CameraImageFormatsVector, variable_capacity_policy>();
+
+    to_python_converter<CameraControlsVector, to_list<CameraControlsVector> >();
+    from_python_sequence<CameraControlsVector, variable_capacity_policy>();
+
+    to_python_converter<FrameratesVector, to_list<FrameratesVector> >();
+    from_python_sequence<FrameratesVector, variable_capacity_policy>();
+
+    class_<CameraImageFormat>("CameraImageFormat", no_init)
+        .add_property("size", &CameraImageFormat::getSize)
+        .add_property("pixelFormat", &CameraImageFormat::getPixelFormat)
+        .add_property("framerates", &CameraImageFormat::getFramerates)
+    ;
+
+    class_<CameraControl>("CameraControl", no_init)
+        .add_property("controlName", &CameraControl::getControlName)
+        .add_property("min", &CameraControl::getMin)
+        .add_property("max", &CameraControl::getMax)
+        .add_property("default", &CameraControl::getDefault)
+    ;
+
+    class_<CameraInfo>("CameraInfo", no_init)
+        .add_property("driver", &CameraInfo::getDriver)
+        .add_property("deviceID", &CameraInfo::getDeviceID)
+        .add_property("imageFormats", &CameraInfo::getImageFormats)
+        .add_property("controls", &CameraInfo::getControls)
+    ;
         
     enum_<VideoNode::VideoAccelType>("VideoAccelType")
         .value("NO_ACCELERATION", VideoNode::NONE)
@@ -131,7 +164,10 @@ void export_raster()
         .def("seekToFrame", &VideoNode::seekToFrame)
         .def("getStreamPixelFormat", &VideoNode::getStreamPixelFormat)
         .def("getDuration", &VideoNode::getDuration)
+        .def("getVideoDuration", &VideoNode::getVideoDuration)
+        .def("getAudioDuration", &VideoNode::getAudioDuration)
         .def("getBitrate", &VideoNode::getBitrate)
+        .def("getContainerFormat", &VideoNode::getContainerFormat)
         .def("getVideoCodec", &VideoNode::getVideoCodec)
         .def("getAudioCodec", &VideoNode::getAudioCodec)
         .def("getAudioSampleRate", &VideoNode::getAudioSampleRate)
@@ -155,8 +191,43 @@ void export_raster()
         .add_property("accelerated", &VideoNode::isAccelerated)
     ;
 
+    class_<FontStyle, bases<ExportedObject> >("FontStyle", no_init)
+        .def("__init__", raw_constructor(createExportedObject<fontStyleName>))
+        .def("__copy__", copyObject<FontStyle>)
+        .add_property("font", 
+                make_function(&FontStyle::getFont,
+                        return_value_policy<copy_const_reference>()),
+                make_function(&FontStyle::setFont,
+                        return_value_policy<copy_const_reference>()))
+        .add_property("variant", 
+                make_function(&FontStyle::getFontVariant,
+                        return_value_policy<copy_const_reference>()),
+                make_function(&FontStyle::setFontVariant,
+                        return_value_policy<copy_const_reference>()))
+        .add_property("color", 
+                make_function(&FontStyle::getColor,
+                        return_value_policy<copy_const_reference>()),
+                make_function(&FontStyle::setColor,
+                        return_value_policy<copy_const_reference>()))
+        .add_property("aagamma", &FontStyle::getAAGamma, &FontStyle::setAAGamma)
+        .add_property("fontsize", &FontStyle::getFontSize, &FontStyle::setFontSize)
+        .add_property("indent", &FontStyle::getIndent, &FontStyle::setIndent)
+        .add_property("linespacing", &FontStyle::getLineSpacing, 
+                &FontStyle::setLineSpacing)
+        .add_property("alignment", &FontStyle::getAlignment, &FontStyle::setAlignment)
+        .add_property("wrapmode", &FontStyle::getWrapMode, &FontStyle::setWrapMode)
+        .add_property("justify", &FontStyle::getJustify, &FontStyle::setJustify)
+        .add_property("letterspacing", &FontStyle::getLetterSpacing, 
+                &FontStyle::setLetterSpacing)
+        .add_property("hint", &FontStyle::getHint, &FontStyle::setHint)
+    ;
+
     class_<WordsNode, bases<RasterNode> >("WordsNode", no_init)
         .def("__init__", raw_constructor(createNode<wordsNodeName>))
+        .add_property("fontstyle",
+                make_function(&WordsNode::getFontStyle,
+                        return_value_policy<copy_const_reference>()),
+                &WordsNode::setFontStyle)
         .add_property("font", 
                 make_function(&WordsNode::getFont,
                         return_value_policy<copy_const_reference>()),
@@ -170,13 +241,13 @@ void export_raster()
         .add_property("text", 
                 make_function(&WordsNode::getText,
                         return_value_policy<copy_const_reference>()),
-                make_function(&WordsNode::setText,
-                        return_value_policy<copy_const_reference>()))
+                &WordsNode::setText)
         .add_property("color", 
                 make_function(&WordsNode::getColor,
                         return_value_policy<copy_const_reference>()),
                 make_function(&WordsNode::setColor,
                         return_value_policy<copy_const_reference>()))
+        .add_property("aagamma", &WordsNode::getAAGamma, &WordsNode::setAAGamma)
         .add_property("fontsize", &WordsNode::getFontSize, &WordsNode::setFontSize)
         .add_property("parawidth", &deprecatedGet<WordsNode>, &deprecatedSet<WordsNode>)
         .add_property("indent", &WordsNode::getIndent, &WordsNode::setIndent)
